function [xtim,df]=msinprep(varargin)
%MSINPREP Generate time series from complex amplitudes (multisine).
%
%       [xtim,df]=MSINPREP(Fdat,N,fs,dev)
%
%       It is assumed that the frequencies are harmonically related, and an
%       integer number of periods is to be generated. The function can be
%       asked to introduce a predistortion to compensate a zero order hold.
%
%       Output arguments:
%       xtim = generated time series
%       df = calculated common divider of frequencies
%
%       Input arguments:
%       Fdat = the output of msinclip (fiddata object, or maybe a structure),
%          containing the frequency points and the complex Fourier amplitudes as input.
%       N = length of the time series. If it is not given or empty, it will
%           be defined as fs/df, where df is the maximal common divider of
%           frequencies in freqv
%       fs = sampling frequency. If it is not given, it will be chosen
%           as N*df, where df is the maximal common divider of the frequencies
%           in freqv. If N is not given, either, fs=df*2*(max(freqv)/df+1)
%       dev = device for which the series is prepared. Possibilities:
%           'screen': for plotting etc, no modification
%           'DAC': for D/A converter, the amplitudes will be predistorted by
%                the inverse tf of the zero order hold
%           default: type of the input of Fdat, if undefined then 'screen'
%                (no predistortion)
%
%       Usage: [xtim,df]=msinprep(Fdat,N,fs,dev);
%       Example: [cx,crestx]=msinclip(fiddata([],[],[1:15]'/256));
%                arbitgen=msinprep(cx,512,1);
%
%       See also: MSINCLIP, OPTEXCIT, CRESTMIN.

%Old fdident help
%MSINPREP Generate time series from complex amplitudes (multisine).
%
%       [xtim,df]=MSINPREP(freqv,cx,N,fs,dev)
%
%       It is assumed that the frequencies are harmonically related, and an
%       integer number of periods is to be generated. The function can be
%       asked to introduce a predistortion to compensate a zero order hold.
%
%       Output arguments:
%       xtim = generated time series (tiddata object)
%       df = calculated common divider of frequencies
%
%       Input arguments:
%       freqv = frequency vector, where the complex amplitudes are given
%       cx = vector of complex amplitudes: fft of xtim gives back cx values
%            at appropriate frequencies (approximately only for dev='DAC')
%       N = length of the time series. If it is not given or empty, it will
%           be defined as fs/df, where df is the maximal common divider of
%           frequencies in freqv
%       fs = sampling frequency. If it is not given, it will be chosen
%           as N*df, where df is the maximal common divider of the frequencies
%           in freqv. If N is not given, either, fs=df*2*(max(freqv)/df+1)
%       dev = device for which the series is prepared. Possibilities:
%           'screen': for plotting etc, no modification
%           'DAC': for D/A converter, the amplitudes will be predistorted by
%                the inverse tf of the zero order hold
%           default: 'DAC'
%
%       Usage: [xtim,df]=msinprep(freqv,cx,N,fs,dev);
%       Example: [cx,crestx]=msinclip([1:15]'/256,ones(15,1));
%                arbitgen=msinprep([1:15]'/256,cx,512,1,'DAC');
%
%       See also: MSINCLIP, OPTEXCIT.

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1991-2003
%       All rights reserved.
%       $Revision: $
%       Last modified: 31-Aug-2003

Fdat=varargin{1};
if isa(Fdat,'fiddata')|isstruct(Fdat) %new call
  error(nargchk(1,4,nargin))
  ch=Fdat.InputCharacter;
  if any(strmatch(ch,{'BL','Samples','Discrete'}))
    dev='screen'; %no predistrotion
  elseif strcmp(ch,'ZOH')|strcmp(ch,'FOH')
    dev='DAC';
  else
    dev='';
  end
  if (nargin>=4)&~isempty(varargin{4}), dev=varargin{4}; end
  if isempty(dev), dev='screen'; end
  if nargin<3, fs=[]; else fs=varargin{3}; end
  if nargin<2, N=[]; else N=varargin{2}; end
  cx=Fdat.Input;
  freqv=Fdat.FreqPoints;
else
  error(nargchk(2,5,nargin))
  if nargin<5, dev=''; else dev=varargin{5}; end
  if nargin<4, fs=[]; else fs=varargin{4}; end
  if nargin<3, N=[]; else N=varargin{3}; end
  cx=varargin{2};
  freqv=varargin{1};
  if isempty(dev), dev='DAC'; end
end
%
if length(freqv)~=length(cx), error('freqv and cx have different lengths'), end
[freqv,ind]=sort(freqv); freqv=freqv(:);
cx=cx(ind); cx=cx(:);
if any(freqv<0), error('freqv has negative element(s)'), end
freqvnz=freqv;
if freqvnz(1)==0, freqvnz(1)=[]; end
dfv=diff([0;freqvnz]);
if any(dfv==0), error('freqv must not contain equal elements'), end
%find common divider df
if isempty(N), remfnegl=max(freqv)/min(diff([0;freqvnz]))*eps*max(freqv);
elseif ~isempty(fs), remfnegl=fs/N/2;
else remfnegl=2*max(freqv)/N;
end
df=freqvnz(1);
if any(dfv<=0), error('freqv is not monotonously increasing'), end
while any(dfv>remfnegl)
  df0=df; df=min([df0;dfv]);
  dfv=sort(rem([df0;dfv],df));
  if isempty(N), remfnegl=max(freqv)/df*eps*max(freqv); end
  ind=find(dfv<=remfnegl); dfv(ind)=[];
end
fi=round(freqv/df); %harmonic numbers
if max(fi)>1023,
  warning(sprintf(['Maximum harmonic index found in ''msinprep''\n',...
            '   is %.0f, with df = %.3g Hz, T = %.3g s'],max(fi),df,1/df))
  if max(fi)>1e4
    disp('Large indexes are often due to inaccurately given frequency values.')
    disp('If this is the case, before invoking msinprep do')
    disp('   freqv = round(freqv*T)/T;')
    disp('where T is the desired period length.')
    disp(' ')
  end
end
devii=fi*df-freqv; ddf=fi\devii;
if max(abs(fi*df-freqv))>max(abs(fi*(df+ddf)-freqv)), df=df+ddf; end
maxdev=max(abs(fi*df-freqv));
if maxdev>2*eps*max(freqv)*max(fi)
  warning(sprintf('Maximum deviation of i*df from fi is %.2e in msinprep',maxdev))
end
%
if isempty(N)&isempty(fs) %none of them is given
  fs=df*2*(max(freqv)/df+1); N=round(fs/df);
end
if isempty(fs)
  fs=N*df;
elseif max(freqv)>(1+N*eps)*fs/2
  error('max of freqv exceeds fs/2')
end
if isempty(N)
  N=round(fs/df);
elseif N<(1-N*eps)*round(fs/df)
  error(sprintf('N (%.0f) must be at least %.0f for one period',...
      N,round(fs/df)))
end
N=round(N);
if abs(rem(df/(fs/N)+0.5,1)-0.5)>1e3*eps*N
  pno=N/(fs/df); digno=ceil(-log10(abs(rem(pno+0.5,1)-0.5)))+1;
  warning(sprintf(['The N (%.0f) samples cover %.',int2str(digno),'f'...
       ' periods,\n   this is not an integer number'],N,N/(fs/df)))
end
%
if strcmp(dev,'DAC') %precompensation for zero order hold
  tf=exp(-j*pi*freqv/fs).*sin(pi*freqv/fs+eps)./(pi*freqv/fs+eps);
  cx=cx./tf;
elseif strcmp(dev,'screen')|isempty(dev)
  %nothing to do
else
  error(['dev ''',dev,''' unknown'])
end
%
hno=round(freqv/(fs/N)); %harmonic numbers
Xk=zeros(N,1); Xk(hno+1)=2*cx; Xk(1)=Xk(1)/2; xtim=real(ifft(Xk));
if isa(Fdat,'fiddata')
  xtim=tiddata([],xtim,1/fs);
  if strcmp(dev,'screen'), xtim.inputcharacter='BL'; end
end
%%%%%%%%%%%%%%%%%%%%%%%% end of msinprep %%%%%%%%%%%%%%%%%%%%%%%%
