function [pdat,fit,Cp,CR,cfv,J,w]=elis(Fdat,varargin)
%ELIS   Parameter estimation of linear systems.
%
%       pdat=ELIS(Fdat,domain,numord,denomord,runmod)
%
%       Output argument:
%       pdat = parameters in fidmodel object. Special properties set:
%          fitinfo = informative column vector about the fit
%             (request details by 'elis fitinfo' )
%          The covariance is set to the approximate Cramer-Rao bound of the
%             covariance matrix (inv(JTJ))
%       Input arguments:
%       Fdat = Fourier data (fiddata object). The variance needs to be filled in.
%          In the object, the variances of the complex Fourier amplitudes are stored.
%          WARNING! these values are the doubles of the variances of the real parts,
%          needed in the old call forms of the toolbox. The object generating
%          functions (e.g.varanal) generate the objects properly for this.
%          The function fiddata requires the variances of the complex amplitudes.
%       domain = 's', 'z', 'w', or 'r'
%       numord = order of the numerator
%       denomord = order of the denominator
%       runmod = structure of the run modifiers
%            (optional, request details by 'elis runmod' )
%
%       Usage:
%           pdat=elis(Fdat,domain,numord,denomord,runmod);
%       Examples:
%           load bandpass, bp1=bandpass{:,1}; bp1.SisoVariance=1.7e-4*[1,1];
%           elis(bp1,'s',4,6);
%           pv=elis('inpchan','s',11,12);
%           %z-domain, free delay:
%           elis('inpchan','z',14,14,struct('fs',51200,'delaytreat','variable'));
%
%       See also: ELISTPER.
%
%       Remark: the old-form command line call (backward compatible with
%           the earlier versions of the toolbox) is still usable, type
%           'oldhelp elis' to see this. The easier-to-use new form is 
%           recommended, and will be supported in the future.

%Old fdident help
%ELIS   Parameter estimation of linear systems.
%
%       [pdat,fit,Cp,CR,cfv]=ELIS(Fdat,vdat,rppar,fixp,rpalg,rppl,initp,rpfs,initw)
%
%       Output arguments:
%       pdat = parameters in vector form (for imppar)
%       fit = informative column vector about the fit:
%           fit(1) = cost function
%           fit(2) = theoretical value of the cost function (chi^2/2)
%           fit(3) = 95% point of the theoretical distribution of the cf
%           fit(4) = number of frequencies
%           fit(5) = number of free parameters
%           fit(6) = performed iterations
%           fit(7) = last change of the cost function
%           fit(8) = last maximum of the relative changes of parameters
%           fit(9) = last lambda value (for Levenberg-Marquardt, otherwise NaN)
%           fit(10) = approximate mean model error
%           fit(11) = mean absolute value of the transfer function
%           fit(12) = Akaike information criterion (AIC), modified for this problem
%           fit(13) = condition number of the matrix actually decomposed
%               or inverted in the last cycle
%           fit(14) = condition number of J in the last cycle
%           fit(15) = condition number of Q=d^2C/dP^2, inverted when calculating
%              the approximate covariance matrix (see Eq. (2.23) in the manual).
%           fit(16) = scaling frequency for internal calculations
%           fit(17) = cause of termination: 1 for iteration number limit,
%              2 for small dcf, 3 for small dp, 4 for GUI request
%           fit(18) = Total running time in seconds
%           fit(19) = MDL, Rissanen's information criterion
%       Cp = approximate covariance matrix of the parameters: indentical to CR
%            in this version
%       CR = approximate Cramer-Rao bound of the covariance matrix (inv(JTJ))
%       cfv = vector of the cost function values in the iteration steps; for
%            the LM algorithm, the second column contains the lambda values.
%
%       Input arguments:
%       Among the input arguments, there are a few run parameter vectors.
%       These prescribe the running of the algorithm, by elements that are
%       either letters or numbers. It does not matter whether the vectors are
%       strings or a numerical vectors, ELIS will take care of the proper
%       meanings. The vectors may be shorter than their maximum length, or
%       may contain NaN elements; in such a case the default values will be
%       assigned to the corresponding run parameters.
%       Fdat = Fourier data ([freq,x,y], Fx3 array), or Fourier vector (see
%           expfou), or name of Fourier file
%       vdat = variance of the real parts of the Forier amplitudes:
%           Fx2 variance array or Fx3 covariance array ([vary,varx,covxy]),
%           (note that in ancient times, the order was different:
%           [varx,vary,covxy], but this has been changed)
%           or row vector of variances (and maybe of covariance),
%           or variance vector (see expvar) or variance file name
%           For either Fdat or vdat the name of a run parameter file may also
%           be given: this must be a string which ends by '.ebn', and may not
%           contain any other period, and the run parameter file must define
%           the Fourier or the variance data, respectively.
%       rppar = vector of the basic model descriptors:
%           1 - domain ('z'/'q' or 's'/'p', or 'w', or 'r')
%           2 - order of the numerator
%           3 - order of the denominator
%           4 - sampling frequency or internal scaling frequency
%           5 - 'a' if allpass design is required, 'n' if not
%           6 - 't' if transient signal estimation is required
%           7 - order of the numerator of the transient model
%           8 - delayscale: additional "scaling time" for the delay
%           9 - stabilization: 'r' for reflection, 'c' for contraction, 
%               'l' for step limitation to limit
%           10 - stability limit, default: 0 for s-domain, 1 for z-domain
%           Default: ['s',1,2,1,'n','n',denomord-1,1,'n',0/1]
%       fixp = definition of fixed parameters.
%           nx2 array: fixed parameters, indices and values of fixed
%           parameters in the vector [num,denom,delay]'; e.g. if num=[1,1] and
%           denom=[4,3,2,1], the delay and the last coefficient of the
%           denominator can be fixed with fixp=[6,1;7,0]
%           Other possibilities:
%           '0': fixed delay and zero-order coeff of denominator set to 1
%           'v' for variable, 'f' for fixed delay; other parameters remain
%             as defined in a run parameter file.
%           'n' for no fixed parameters at all
%           if fixp is empty, the previously defined fixed parameters remain
%           For allpass design, the numerator coefficients may be fixed.
%           Default: no fixed coefficient, fixed delay
%       rpalg = vector of basic algorithm descriptors
%           1 - algorithm: 'g' (Newton-Gauss), 'l' (Levenberg-Marquardt),
%              'm' (LM with svd), 's' (singular value decomposition),
%              'r' (Newton-Raphson)
%           2 - setting of starting (initial) values: 'l' (least squares),
%              'w' (weighted least squares, weight: vary), 'v' (outputrvar-WLS),
%              'r' (eqvar-WLS), 'u' (inputrvar-WLS), 's' (svd), 'y' (vary-WLS with svd),
%              'a' (approximate-ML WLS), 't' (try different algorithms),
%              'f' (file, see initp), 'x' (external weight vector),
%              'e' (equation error method)
%           3 - maximum number of iterations
%           4 - stop if rel. variation of cost function smaller than this value
%           5 - stop if rel. variation of parameters smaller than this value
%           Levenberg-Marquardt settings:
%           6 - number of consecutive decreases of the cost function before
%              trial with lambda=0. A negative value means that this is never
%              done, with the absolute value used as a lower bound for lambda
%           7 - starting value of lambda
%           8 - maximum value of lambda which allows stopping of iteration
%           Other parameters:
%           9 - 'l' for line search in calculated direction
%           10 - way of basis update in orthopol:
%              'n': calculate new basis before initial setting
%                   (using previous results if initp contains parameters),
%              'f' to use basis given in initp
%           Default: ['g','w',50,1e-6,0,10,0.1,1e-10,'f']
%       rppl = vector of plot modifiers
%           1 - n: plot for every nth iteration; inf for no plot,
%              negative for no 0th
%           2:5 - axis vector for magnitude plots
%           6 - 'i' for linear, 'o' for logarithmic frequency axis
%           7 - 'c' to calculate zeros, 'n' for don't calculate
%           8 - 'c' to calculate poles, 'n' for don't calculate
%           9:12 - axis vector for pole/zero plot (or 'p   ' for the same
%              vertical and horizontal scaling, all poles/zeros shown;
%              'a   ' for all poles/zeros just shown; [n,n,n,n] for all zeros
%              and poles with limit n*fvmax in s-domain, limit 2 in z-domain)
%           13 - number of consecutive zero/pole sets shown at the same time
%           14 - display messages during run, 'y', 'n', 'b' (burst),
%              'c' (collect only), or 's' (short)
%           15 - frequency axis in frequency 'f' or radian frequency 'r'
%           16 - start counting plots to screen (rppl(1)) at rppl(16)+1
%           17 - call drawnow & check iterctrl: 'd', 'i', 'b' (both), 'n'
%           18 - minimum time between serial plots in seconds
%           19 - 'b' means basic plot only (less info)
%           Default: [1,0,NaN,NaN,NaN,'i','c','c','a   ',1,'yf']
%       initp = vector or string: parameter vector (see exppar) or file name
%           scalar value: new value of the delay in s (s-domain) or
%           in samples (z-domain)
%           Default: delay=0
%       rpfs = names of files to be generated (shorter lines extended by
%           spaces, the order is not bound):
%           - report file (*.rep)
%           - parameter file (*.par or *.pbn or *.pnt)
%           - covariance file (*.cov or *.cbn or *.cnt)
%       initw = initial weight (linear, denominator) vector, length same as
%               of frequencies
%       WARNING! In Matlab5, strings are represented as 2-byte numbers, and
%       concatenation of strings with numbers results strings. Therefore, in
%       the setting the run parameters as rppar, rpalg, rppl, the strings
%       used in them must be converted first to numbers.
%       E.g. rppar=['s'+0,4,6,1e6]
%
%       Usage:
%         [pdat,cfv]=elis(Fdat,vdat,rppar,fixp,rpalg,rppl,initp,rpf);
%       Examples:
%         load bandpold %load variable Fvect1
%         elis(Fvect1,3.4e-4*[1,1],['s',4,6]);
%         load inpchold %load variable Fdat
%         elis(Fdat,[9.61e-10,9.61e-12],['s',11,12]);
%         %z-domain, free delay:
%         elis(Fdat,[9.61e-10,9.61e-12],['z'+0,14,14,51200],'v');
%
%       See also: ELISQA, ELRPF2V, ELRPV2F.

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1991-2005
%       All rights reserved.
%       $Revision: $
%       Last modified: 22-Apr-2005

%Return immediately if preload or help only:
if nargin==0, help elis, return, end
if nargin==1
  if isstr(Fdat)&strcmp(Fdat,'preload')
    return %loading only for one argument 'preload'
  elseif isstr(Fdat)&strcmp(Fdat,'runmod')
    type elisrmt
    return
  elseif isstr(Fdat)&strncmpi(Fdat,'object',3)
    type elisobj
    return
  elseif isstr(Fdat)&strcmp(Fdat,'devrunmod')
    type elisdrmt
    return
  elseif isstr(Fdat)&strncmp(Fdat,'fitinfo',4)
    type elisfiti
    return
  elseif isstr(Fdat)&strncmp(Fdat,'date',3)
    fid=fopen('elis.m','r');
    e=setstr(fread(fid,20000)'); fclose(fid);
    ind=findstr('Last modified',e);
    if isempty(ind), error('String ''Last modified'' not found'), end
    ee=e(ind(1)+5+[1:50]);
    ind=find(isspace(ee)); d=ee(ind(1)+1:ind(2)-1);
    if (length(d)~=11)|(sum(~isletter(d))~=8)|~strcmp(d([3,7:8]),'--2')
      %06-Jan-2001
      error('Date is not found')
    end
    if nargout>0
      pdat=d;
    else
      disp(['Date of elis: ',d])
      w=which('elis.m');
      dd=dir(w);
      fprintf(['Date of the file: ',dd.date,', length: %.0f bytes\n'],dd.bytes)
      crlf=length(findstr(setstr([13,10]),e));
      cr=length(findstr(setstr([13]),e))-crlf;
      lf=length(findstr(setstr([10]),e))-crlf;
      if (crlf>cr)&(crlf>lf), eol='CR-LF (PC)';
      elseif cr>lf, eol='CR (Mac)';
      else eol='LF (Sun)';
      end
      disp(['End-of-line character: ',eol])
      %crlf, cr, lf
    end
    return
  elseif isstr(Fdat)&strcmp(Fdat,'secret')
    fprintf(['  ''Secret'' way of setting the number of iterations:\n',...
        '    global elis_max_iteration_number\n    elis_max_iteration_number=<iterations>;\n'])
    return
  elseif isequal(Fdat,-1)
    warning('Obsolete preload argument: -1')
    return
  else
    if ischar(Fdat)
      disp(['Input argument: ''',Fdat,''''])
    end
    fprintf(['Allowable single input arguments:\n',...
       '  ''runmod'',''fitinfo'',''objects'',''preload'',''devrunmod''\n'])
    error('This single input argument is not allowed')
  end
elseif (nargin==2)&isstr(Fdat)&isstr(varargin{1})&...
    (strcmp(Fdat,'runmod')|strcmp(Fdat,'devrunmod')|strcmp(Fdat,'fitinfo'))
  %individual property help, e.g. elis runmod initset
  if strcmp(Fdat,'runmod'), fn=which('private/elisrmt');
  elseif strcmp(Fdat,'devrunmod'), fn=which('private/elisdrmt');
  elseif strcmp(Fdat,'fitinfo'), fn=which('private/elisfiti');
  end
  [fid,message]=fopen(fn,'r');
  if fid==-1, error(message), end
  s=fread(fid); fclose(fid); s=setstr(s');
  inds=sort([findstr([setstr(13),varargin{1}],s),findstr([setstr(10),varargin{1}],s)]);
  if length(inds)>1, error(['Ambiguous ',Fdat,' field ''',varargin{1},''''])
  elseif isempty(inds), error([Fdat,' field ''',varargin{1},''' not found'])
  end
  ind=sort([find(s==setstr(13)),find(s==setstr(10))]);
  fi=find(inds==ind)+1; s=[s,'%'];
  while strcmp(s(ind(fi)+[0:2]),setstr([13,10,32]))|strcmp(s(ind(fi)+1),' ')
    if (fi+1>length(ind))|(ind(fi+1)+1>length(s)), break, end
    fi=fi+1;
    if ind(fi)+3>length(s), break, end
  end
  sd=s(inds+1:ind(fi)-1);
  indcrlf=findstr(sd,setstr([13,10]));
  for ii=1:length(indcrlf), sd(indcrlf(ii)+[0,1])='\n'; end
  fprintf([sd,'\n\n'])
  return
end
cfn=setstr([102, 100, 99, 102, 115, 53, 95, 50]);
if isa(Fdat,'fidmodel')
  initmodel=Fdat; Fdat=initmodel.data;
  if isempty(varargin)|isstruct(varargin{1})
    variable=initmodel.variable; if any(findstr('z',variable)), variable='z'; end
    varargin=[{variable,length(initmodel.num)-1,length(initmodel.denom)-1},varargin];
    if length(varargin)>=4
      runmod=varargin{4};
      repr=initmodel.representation;
      if isfield(runmod,'representation')
        if ~strcmp(repr,runmod.representation), error('representation inconsistent'), end
      else
        runmod.representation=repr;
      end
      if isfield(runmod,'initset')
        if ~strcmp('object',runmod.initset), error('initset inconsistent'), end
      else
        runmod.initset='object';
      end
      if isfield(runmod,'initmodel')
        if ~isequal(initmodel,runmod.initmodel), error('initmodel is inconsistent'), end
      else
        runmod.initmodel=initmodel;
      end
    else
      runmod.representation=repr;
      runmod.initset='object';
      runmod.initmodel=initmodel;
    end
    varargin{4}=runmod; 
  end
end
Fdat=getobjf(Fdat,'fiddata');
if isnumeric(Fdat)|~isempty(Fdat.covariance), errorweighting='Linear';
elseif ~isempty(Fdat.nonlincovariance), errorweighting='NonLinear';
elseif israndomized(Fdat), errorweighting='Interpolated';
else errorweighting='Linear';
end
if isa(Fdat,'fiddata') %object input
  Fdatin='object';
  narginelis=max(length(varargin)+1,2);
  %
  if (length(varargin)+1>=4)&(length(varargin{2})==1) %rppar would be longer
    if (length(varargin)>=4)&isfield(varargin{4},'errorweighting')
      errorweighting=varargin{4}.errorweighting;
      if any(strmatch(errorweighting,{'linear','nonlinear','Linear','Nonlinear','Interpolated nonlinear','interpolated nonlinear'}))
      else error(['errorweighting as ''',errorweighting,''' is not legal'])
      end
    end
    call='new';
    vdat=[]; fixp=[];
    error(nargchk(4,6,length(varargin)+1))
    if nargout>2, error('Too many output arguments'), end
  else
    call='obsolete';
    if ~exist('fdguidev.mat')
      warning(sprintf(['Warning: This call (objects + parameter vectors) in elis is obsolete,\n',...
          '  and will be eliminated in future versions. Type ''help elis''']))
    end
    %Temporary fix
    if nargin>=2, vdat=varargin{1}; else vdat=[]; end
    if nargin>=3, rppar=varargin{2}; else rppar=[]; end
    if nargin>=4, fixp=varargin{3}; else fixp=[]; end
    if nargin>=5, rpalg=varargin{4}; else rpalg=[]; end
    if nargin>=6, rppl=varargin{5}; else rppl=[]; end
    if nargin>=7, initp=varargin{6}; else initp=[]; end
    if nargin>=8, rpfs=varargin{7}; else rpfs=[]; end
    if nargin>=9, initw=varargin{8}; else initw=[]; end
  end
  %
  if Fdat.expn>1
    if isequal(Fdat.freqn,1), Fdat.synchronization=''; end
    Fdat=collapse(Fdat);
  end
  expno=Fdat.expn; pn=Fdat.chn;
  if pn~=2, error('Two ports are obligatory'), end
  X=Fdat.input; Y=Fdat.output;
  if iscell(X), lX=prod(size(X)); X=cat(1,X{:}); Y=cat(1,Y{:}); expno=1;
  else lX=length(X);
  end
  if isempty(X)|isempty(Y), error('Input or output is empty'), end
  F=size(Y,1); 
  %
  linM=Fdat.M; nonlinM=Fdat.nonlinM;
  M=[];
  if any(findstr('onlin',lower(errorweighting)))|any(findstr('terpol',lower(errorweighting)))
    M=nonlinM;
  elseif any(findstr('linear',lower(errorweighting))), 
    if ~isempty(nonlinM)
      %M=linM*nonlinM;
      M=linM;
    else
      M=linM;
    end
  else
    error(['Cannot identify errorweighting as ''',errorweighting,''''])
  end
  if isempty(vdat)
    if any(findstr('onlin',lower(errorweighting)))|any(findstr('nterpol',lower(errorweighting)))
      vdat=sisononlinerror(Fdat)/2;
      nonlinM=Fdat.nonlinM; 
      if any(strmatch('nonlin',lower(errorweighting)))&~isempty(nonlinM)
        vdat=vdat/nonlinM;
      end
    elseif strcmpi('linear',lower(errorweighting))
      vdat=Fdat.OldTBSisoVariance;
    else
      error(['Cannot identify errorweighting as ''',errorweighting,''''])
    end
    if isempty(vdat)
      warning('Fdat does not contain variances')
      vdat=[1,1];
    end
  end
  if iscell(vdat), vdat=cat(1,vdat{:}); end
  %
  %Correct if M<6
  if ~isempty(M)&(M<7)
    Msave=M;
    [vdat,M]=smoothvar(vdat,M);
    if (length(varargin)>=5)&isstruct(varargin{5})&isfield(varargin{5},'displaymessages')
      dmin=getfield(varargin{5},'displaymessages');
      if all(~isnan(dmin(:)))
        if strcmp(dmin,'on'), dispmesg='y';
        elseif strcmp(dmin,'off'), dispmesg='n';
        else dispmesg=dmin(1);
        end
      end
    else
      dispmesg='y';
    end
    if exist('dispmesg')&~any(dispmesg=='ncs')
      fprintf('Warning: variances were smoothed, from %.0f experiments to equivalent to %.0f experiments\n',Msave,M)
    end
  end
  if any(findstr('nterpo',lower(errorweighting)))
    vartmp=get(Fdat,'OddOutputNonlinError');
    vartmp=smoothvar(vartmp,Msave);
    set(Fdat,'OddOutputNonlinError',vartmp,'noconsistency');
    vartmp=get(Fdat,'EvenOutputNonlinError');
    vartmp=smoothvar(vartmp,Msave);
    set(Fdat,'EvenOutputNonlinError',vartmp,'noconsistency');
    set(Fdat,'nonlinM',M);
    vdat=sisononlinvariance(Fdat)/2;
    while (size(vdat,1)>0)&(size(vdat,2)<3), vdat=[vdat,zeros(size(vdat,1),1)]; end
    while (size(vdat,2)>0)&(size(vdat,1)==1), vdat=vdat(ones(size(vy,1),1),:); end
    if isempty(vdat), vdat=0*[vy,vx,cxy]; end
  elseif any(findstr('onlin',lower(errorweighting)))
    vy=vdat(:,1); vx=vdat(:,2); cxy=[]; if size(vdat,2)==3, cxy=vdat(:,3); end
    set(Fdat,'outputnonlinvariance',vy*2,'inputnonlinvariance',vx*2,'nonlincovvector',cxy*2,...
      'nonlinM',M);
  elseif strcmpi('linear',lower(errorweighting))
    set(Fdat,'sisovariance',vdat*2,'M',M)  
  else
    error(['Cannot identify errorweighting as ''',errorweighting,''''])
  end
  %
  if isnumeric(vdat)&(size(vdat,2)>1)
    vdl=size(vdat,1);
    if (vdl>1)&(vdl<lX), vdat=kron(ones(lX/vdl,1),vdat); end
  end
elseif isnumeric(Fdat)|isstr(Fdat) %non-object input
  call='old';
  warning(sprintf(['This is an old-form call of elis. We strongly advise to switch to the new, \n',...
      'object-based call. See ''help elis''']))
  Fdatin='array';
  narginelis=nargin;
  if nargin>=2, vdat=varargin{1}; else vdat=[]; end
  if isnumeric(vdat)&~isempty(vdat)
    warning(sprintf(['When the variance data are given in an array, the necessary order of the vectors is\n',...
        '[vy,vu,cuy], different from the earlier used order [vx,vy,vxy]']))  
  end
  if nargin>=3, rppar=varargin{2}; else rppar=[]; end
  if nargin>=4, fixp=varargin{3}; else fixp=[]; end
  if nargin>=5, rpalg=varargin{4}; else rpalg=[]; end
  if nargin>=6, rppl=varargin{5}; else rppl=[]; end
  if nargin>=7, initp=varargin{6}; else initp=[]; end
  if nargin>=8, rpfs=varargin{7}; else rpfs=[]; end
  if nargin>=9, initw=varargin{8}; else initw=[]; end
  M=[];
else
  error(['Class of Fdat is ''',class(Fdat),''''])
end
if strcmp(call,'old')|strcmp(call,'obsolete')
  if isstruct(rpalg)|isstruct(rppl)
    error('Structure among arguments in an old call')
  elseif length(rppar)==1
    error('Length of rppar is 1 in old call')
  end
end
%
c=computer; MatlV=version; isPC=strncmp(c,'PC',2);
if ~strncmp(MatlV,'5.2',3), cfn=cfn(1:5); end
if MatlV(1)<'5', error('At least Matlab 4.0 is needed'), end
%
% Initializing variables  to [] because this happened automatically in V4.
Df=[]; Nf=[]; Nft=[]; unknownvect=[];
vartf=[]; J=[]; d2ExE=[]; pvtr=[]; w=[];
%
%Global statement for saving orthopol data
saveorthopoldata='no';
%global saveorthopoldata
%saveorthopoldata='yes';
%
nargoutelis=nargout;
%elisqa-based calls
if narginelis==0
  Fdat=[];
  disp('No input argument given in elis, question/answer procedure follows')
  feval('elisqa','elisrpar.ebn'); %eval to load elisqa only if necessary
  rpf='elisrpar.ebn';
elseif narginelis==1
  if isstr(Fdat)
    [fnam,fnamsh,ext]=fnamanal(Fdat,'ebn');
    if (strcmp(ext,'ebn')|strcmp(ext,'EBN'))&exist(fnam)
      rpf=fnam; Fdat=[];
    end
  end
else
  rpf='';
end
%
dispmesg='m'; %display messages
if ~exist('fdguidev.mat'), dispmesg='n'; end %do not display messages
msgsall='';
expimpmessagessave='';
if (nargin>=6)&~strcmp(call,'new')
  if length(rppl)>=14
    if ~isnan(rppl(14)), dispmesg=setstr(rppl(14)); end
    if any(dispmesg=='nc')
      global expimpmessages
      expimpmessagessave=expimpmessages; expimpmessages='no';
    end
  end
end
if narginelis<2, vdat=[]; end
if isstr(vdat)&(length(vdat)>4) %check for *.ebn file
  [fnam,fnamsh,ext]=fnamanal(vdat);
  if strcmp(ext,'ebn'), rpf=vdat; vdat=[]; end
end
%
%set default values
ffile=''; %name of Fourier file (maybe without extension)
vfile=''; %name of variance file (maybe without extension)
pfile=''; %parameter file to be generated (maybe empty)
cfile=''; %covariance file to be generated
rfile=''; %report file to be generated
initpfile=''; %model object or name of parameter file, containing the starting
              %values (maybe without extension)
algtype='NG'; %type of the iteration algorithm (NG,LM,LMsvd,NR,svd)
algstr='Newton-Gauss';
amin=NaN; %axisvect(3) for magnitude plot
amax=NaN; %axisvect(4) for magnitude plot
basistreat='n'; %way of treating basis for orthopol, 'n' for new, 'f' for file
cp=[]; cb=[]; %linear constraints in the form cp*[num,denom]'=cb
calcrnum='c'; %calculate roots of numerator (c,n)
calcrdenom='c'; %calculate roots of denominator (c,n)
coeffs='real'; %coefficients
covxy=[]; %covariance between input and output amplitudes
cps=''; %compensate for different cases during initial value setting
delay=0; %(starting) value of the delay
delayscale=1; %additional scaling time for the delay
delaytreat='f'; %delay fix (f) or variable (v)
denomord=2; %order of the denominator
denomfix=[]; %fix denominator coefficients
denomfixind=[]; %fix denominator coefficient indices (at least for z-domain)
%dispmesg='y'; %display messages during run, y, n, b (burst), c (collect only)
domain='s'; %domain of the model to be fitted (s,z,p,q,w,r)
expi=0; %number of the experiment to be used
fmin=NaN; %lower bound of displayed frequencies
fmax=NaN; %maximal displayed frequency in the s-domain
freqax='f'; %frequancy axis on plots in frequency ('f') or radian freq ('r')
fpfile=''; %file of fixed parameters (optional)
fs=NaN; %sampling (normalising) frequency
initset='t'; %way of setting the starting values (l,v,w,y,s,a,f,e,x,i,r,u,t,g,b)
itmax=50; %maximal number of iteration cycles (also defined in sme.m!)
itctrldrawnow='b'; %check iterctrl, call drawnow: 'i', 'd', 'b' (both), 'n'
lambda0=.1; %starting value for the Levenberg-Marquardt iteration
lambdadecr=10; %after lambdadecr consecutive decreases of lambda, 0 is tried
lambdalim=1e-10; %above this value of lambda the iteration will continue
lambdamin=0; %minimum allowable value of lambda
numord=1; %order of the numerator
numfix=[]; %fix numerator coefficients
numfixind=[]; %fix numerator coefficient indices (at least for z-domain)
paramtreat='n'; %no params fixed (n) some predefined (0, d or r) allpass (a)
plotdens=1; %cycles of iteration to plot (1,2...inf, negative for no 0th plot)
plotdensoffs=0; %offset for plotdens
plotdenstime=0; %minimum time between plots
plotlevel='n'; %normal plots
plotmode='i'; %frequency axis, i (lin) or o (log)
pzfollown=1; %pole/zero sets to be plotted on same plot
pzlimit='y'; %limit p/z on plot to pzlimitv*2*pi*fmax (s) or 2 (z) (n,y)
pzlimitv=10;
r=1; %relaxation factor in iqml
rcostvar=1e-6; %minimal relative variation of the cost function
reprn='p'; %polynomial, orthopol or vector orthopol
rparvar=0; %minimal relative variation of parameters
searchmode='n'; %mode of extra minimum search, 'l' for line search
stabmode='n'; %model stabilization, 'r' for reflections, 'c' for contraction
              %'l' for limitation
mphmode='n'; %model minimum phase, 'r' for reflections, 'c' for contraction
             %'l' for limitation
tauR=[]; %for r-domain
transtreat='n'; %consider transients, 't' or 'n'
trord=[]; %order of transient numerator
vardef='u'; %variance values: numbers or from file (u,f)
varx=1; %uniform variance value of the input coefficients
vary=1; %uniform variance value of the output coefficients
calculateCR=1;
%
%New call: set run parameters from structure
if strcmp(call,'new')
  if length(varargin)+1>=2, domain=varargin{1}; end
  if length(varargin)+1>=3, numord=varargin{2}; end
  if length(varargin)+1>=4, denomord=varargin{3}; end
  if any(domain=='zq')
    stablimit=1; %lower limit of abs. value of unstable poles in z-domain
    fs=Fdat.fs;
  else %s-domain or r-domain or w-domain
    stablimit=0; %lower limit of real part of unstable poles in s-domain
  end
  varx=vdat(:,2); vary=vdat(:,1);
  if length(vdat(1,:))==3 %covxy
    covxy=vdat(:,3);
  end
end
%
if strcmp(call,'new')
  if strcmp(domain,'p')
    %error('Obsolete domain: use ''s'' and runmod.representation instead')
  end
  runmod=[]; devrunmod=[];
  if length(varargin)+1>=5
    runmod=varargin{4};
    if ~isstruct(runmod)&~isempty(runmod)
      warning('runmod is not a structure, it will be discarded')
      runmod=[];
    end
  end
  if isstruct(runmod)
    if length(runmod)>1, error('runmod is a structure array, not just a structure'), end
    if isfield(runmod,'constraints')
      if ischar(runmod.constraints), runmod.constraints={runmod.constraints}; end
    end
    f=fieldnames(runmod);
    c=struct2cell(runmod);
    ind=strmatch('fs',f,'exact');
    if any(domain=='zq')&(isempty(ind)|isempty(c{ind})|isnan(c{ind}))
      %In z-domain, the default sampling frequency is that of the data
      fstmp=Fdat.fs;
      if ~isempty(fstmp), fs=fstmp; end
    end
    plot0='';
    for ii=1:length(f)
      if strncmp(f{ii},'representation',max(4,length(f{ii})))
        if all(~isnan(c{ii}(:)))
          if strcmp(c{ii},'orthopol')
            if strcmp(domain,'s'), domain='p';
            elseif strcmp(domain,'z'), domain='q';
            elseif any(domain=='pq')
            else error(['Orthopol is not yet implemented for domain ''',domain,''''])
            end
            if any([numfix(:);denomfix(:)])
              error('Cannot fix parameters with orthopol yet')
            end
          elseif strncmp(c{ii},'vector orthopol',6)
            if strcmp(domain,'s'), domain='u';
            elseif strcmp(domain,'z'), domain='v';
            else error(['Orthopol is not implemented for domain ''',domain,''''])
            end
            if any([numfix(:);denomfix(:)])
              error('Cannot fix parameters with orthopol yet')
            end
          elseif strcmp(c{ii},'polynomial')
            if strcmp(domain,'p')
              error('Contradictory demand: p-domain and polynomial representation')
              domain='s';
            elseif strcmp(domain,'q')
              error('Contradictory demand: q-domain and polynomial representation')
              domain='z';
            end
          else
            error(['representation ''',c{ii},''' is not allowed'])
          end
        end
      elseif strcmp(f{ii},'tauR')
        if all(~isnan(c{ii}(:))), tauR=c{ii}; end        
      elseif strncmpi(f{ii},'coeff',5)
        if isstr(c{ii}), coeffs=c{ii}; end
      elseif strcmp(f{ii},'fs')
        if any(domain=='zq')&~isempty(c{ii})&all(~isnan(c{ii}(:))), fs=c{ii}; end
      elseif strcmp(f{ii},'fscale')
        if ~isempty(c{ii})&all(~isnan(c{ii}(:)))&~any(domain=='zq'), fs=c{ii}; end
      elseif strcmp(f{ii},'allpass')
          if strcmp(c{ii},'on'), paramtreat='a'; end
      elseif strcmp(f{ii},'transients')
        if strcmp(c{ii},'on')|strcmp(c{ii},'t'), transtreat='t'; end
      elseif strcmp(f{ii},'trnumord')
        if all(~isnan(c{ii}(:))), trord=c{ii}; end
      elseif strcmp(f{ii},'delay')
        if all(~isnan(c{ii}(:))), delay=c{ii}; end
      elseif strcmp(f{ii},'delayscale')
        if all(~isnan(c{ii}(:))), delayscale=c{ii}; end
      elseif strcmp(f{ii},'stabilization')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), stabmode=c{ii}(1); end %'reflection' or 'contraction'
        if ~any(findstr(stabmode,'nrcl')), error('Illegal field ''stabilization'''), end
      elseif strcmp(f{ii},'forceminimumphase')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), mphmode=c{ii}(1); end %'reflection' or 'contraction'
        if ~any(findstr(mphmode,'nrcl')), error('Illegal field ''forceminimumphase'''), end
      elseif strcmp(f{ii},'stablimit')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), stablimit=c{ii}; end
      elseif strcmp(f{ii},'errorweighting')
        if strncmpi(c{ii},'linear',3)|strcmp(c{ii},''), errorweighting='Linear';
        elseif strncmpi(c{ii},'nonlin',4), errorweighting='Nonlinear';
        elseif strncmpi(c{ii},'interpolated',6), errorweighting='Interpolated nonlinear';
        else error(['errorweighting ',c{ii},' not allowed'])
        end
      %
      elseif strcmp(f{ii},'delaytreat')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), delaytreat=c{ii}(1); end %'variable', 'v', 'fixed', 'f'
      elseif strcmp(f{ii},'fixedpars')
        if isa(c{ii},'idmodel'), c{ii}=fidmodel(c{ii}); end
        if isa(c{ii},'fidmodel')
          fixmod=c{ii};
          nfix=fixmod.num;
          numfixind=find(~isnan(nfix))-1; numfix=nfix(numfixind+1);
          %if any(domain=='swrp'), numfixind=numord-numfixind; end
          dfix=fixmod.denom;
          denomfixind=find(~isnan(dfix))-1; denomfix=dfix(denomfixind+1);
          %if any(domain=='swrp'), denomfixind=denomord-denomfixind; end
          if any([numfix(:);denomfix(:)])&(domain=='pq')
            error('Cannot fix parameters with orthopol yet')
          end
        elseif isequal(c{ii},'a_0')|isequal(c{ii},'a_n')
          numfix=[]; numfixind=[];
          denomfix=1;
          if (isequal(c{ii},'a_0')&any(domain=='spw'))|...
              (isequal(c{ii},'a_n')&any(domain=='z'))
            denomfixind=[denomord];
          else
            denomfixind=[0];
          end
        elseif all(~isnan(c{ii}(:)))&~isempty(c{ii})
          error('Value of field fixedpars is not allowed')
        end
      elseif strncmp(f{ii},'algorithm',max(3,length(f{ii})))
        if strcmpi(c{ii},'Newton-Gauss')|strcmpi(c{ii},'NG')
          algtype='NG'; algstr='Newton-Gauss';
        elseif strcmpi(c{ii},'Levenberg-Marquardt')|strcmpi(c{ii},'LM')
          algtype='LM'; algstr='Levenberg-Marquardt';
        elseif strcmpi(c{ii},'Levenberg-Marquardt with svd')|...
            strcmpi(c{ii},'LMsvd')|strcmpi(c{ii},'LM with svd')
          algtype='LMsvd'; algstr='Levenberg-Marquardt with svd';
        elseif strcmpi(c{ii},'singular value decomposition')|strcmpi(c{ii},'svd')|...
            strcmpi(c{ii},'tls')
          algtype='svd'; algstr='singular value decomposition';
        elseif strcmpi(c{ii},'Newton-Raphson')|strcmpi(c{ii},'NR')
          algtype='NR'; algstr='Newton-Raphson';
        else
          error(['Unknown algorithm: ',c{ii}])
        end
      elseif strcmp(f{ii},'initset')
        if strcmpi(c{ii},'least squares')|strcmpi(c{ii},'LS')|strcmpi(c{ii},'l')
          initset='l';
        elseif strcmpi(c{ii},'output-weighted least squares')|...
            strcmpi(c{ii},'vary-WLS')|strcmpi(c{ii},'w')
          initset='w';
        elseif strcmpi(c{ii},'y-weighted svd')|strcmpi(c{ii},'ysvd')|...
            strcmpi(c{ii},'v')
          initset='v';
        elseif strcmpi(c{ii},'u-weighted svd')|strcmpi(c{ii},'usvd')|...
            strcmpi(c{ii},'u')
          initset='u';
        elseif strcmpi(c{ii},'singular value decomposition')|strcmpi(c{ii},'svd')|...
            strcmpi(c{ii},'s')|strcmpi(c{ii},'tls')
          initset='s';
        elseif strcmpi(c{ii},'output-weighted singular value decomposition')|...
            strcmpi(c{ii},'vary-WLS with svd')|strcmpi(c{ii},'y')
          initset='y';
        elseif strcmpi(c{ii},'frequency-weighted svd')|strcmpi(c{ii},'freq-w svd')|...
            strcmpi(c{ii},'AML')|strcmpi(c{ii},'approximate maximum likelihood')|...
            strcmpi(c{ii},'a')
          initset='a';
        elseif strcmpi(c{ii},'equation-error variance WLS')|...
            strcmpi(c{ii},'eevWLS')|strcmpi(c{ii},'r')
          initset='r';
        elseif strcmpi(c{ii},'iqml')|strcmpi(c{ii},'i')
          initset='i';
        elseif strcmpi(c{ii},'object')|strcmpi(c{ii},'o')
          initset='f';
        elseif strcmpi(c{ii},'file')|strcmpi(c{ii},'f')
          initset='f';
          warning('initset field value ''file'' is obsolete, use ''object'' instead')
        elseif strcmpi(c{ii},'external weight')|strcmpi(c{ii},'eW')|strcmpi(c{ii},'x')
          initset='x';
        elseif strncmpi(c{ii},'equation error method',8)|strcmpi(c{ii},'eem')|...
            strcmpi(c{ii},'e')
          initset='e';
        elseif strncmpi(c{ii},'generalized total least squares',5)|strcmpi(c{ii},'GTLS,')|...
            strcmpi(c{ii},'g')
          initset='g';
        elseif strncmpi(c{ii},'bootstrapped TLS',7)|strcmpi(c{ii},'BTLS,')|...
            strcmpi(c{ii},'b')
          initset='b';
        elseif strncmpi(c{ii},'try different algorithms',4)|strcmpi(c{ii},'trials')|...
            strncmpi(c{ii},'scan different algorithms',4)|...
            strcmpi(c{ii},'t')
          initset='t';
        else
          error(['Value of initset: ''',c{ii},''' is not allowed'])
        end
      elseif strcmp(f{ii},'initweight')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), initw=c{ii}; end
      elseif strncmp(f{ii},'relaxation_factor',5)
        r=c{ii};
      elseif strncmp(f{ii},'constraints',6)
        cc=c{ii};
      elseif strcmp(f{ii},'itmax')
        if all(~isnan(c{ii}(:))), itmax=c{ii}; end
      elseif strcmp(f{ii},'dcfmax')
        if all(~isnan(c{ii}(:))), rcostvar=c{ii}; end
      elseif strcmp(f{ii},'dprelmax')
        if all(~isnan(c{ii}(:))), rparvar=c{ii}; end
      elseif strcmp(f{ii},'lambdadecrease')
        if all(~isnan(c{ii}(:))), lambdadecr=c{ii}; end
      elseif strcmp(f{ii},'lambdamin')
        if all(~isnan(c{ii}(:))), lambdamin=c{ii}; end
      elseif strcmp(f{ii},'lambda0')
        if all(~isnan(c{ii}(:))), lambda0=c{ii}; end
      elseif strcmp(f{ii},'lambdalim')
        if all(~isnan(c{ii}(:))), lambdalim=c{ii}; end
        %
      elseif strcmp(f{ii},'plotdens')
        if all(~isnan(c{ii}(:)))
          plotdens=c{ii};
          if ~finite(plotdens)&~isnan(plotdens), plot0='off'; end
        end
      elseif strcmp(f{ii},'plot0')
        if all(~isnan(c{ii}(:))), plot0=c{ii}; end
      elseif strcmp(f{ii},'plotoffset')
        if all(~isnan(c{ii}(:))), plotdensoffs=c{ii}; end
      elseif strcmp(f{ii},'plotdenstime')
        if all(~isnan(c{ii}(:))), plotdenstime=c{ii}; end
      elseif strcmp(f{ii},'plotlevel')
        if strncmp(c{ii},'basic',1) plotlevel='b';
        elseif strncmp(c{ii},'advanced',1) plotlevel='n';
        end
      elseif strcmp(f{ii},'magnitudeaxis')
        if ~isempty(c{ii})&all(~isnan(c{ii}(:)))
          if length(c{ii})>=1, fmin=c{ii}(1); end
          if length(c{ii})>=2, fmax=c{ii}(2); end
          if length(c{ii})>=3, amin=c{ii}(3); end
          if length(c{ii})>=4, amax=c{ii}(4); end
        end
      elseif strcmp(f{ii},'plotfreq')
        if all(~isnan(c{ii}(:))), plotmode=c{ii}(2);   end
      elseif strcmp(f{ii},'freqdim')
        if strncmp(c{ii},'frequency',4)|strcmpi(c{ii},'Hz'), freqax='f';
        elseif strncmpi(c{ii},'radians',3), freqax='r';
        end
      elseif strcmp(f{ii},'pzaxis')
        if all(~isnan(c{ii}(:)))
          if length(c{ii})==4,
          elseif isstr(c{ii})&(strncmp(c{ii},'prop',4)|strncmp(c{ii},'all',3))
            pzaxis=c{ii}(1);
          elseif isnumeric(c{ii})&(length(c{ii})==1)
            pzlimit='y'; pzlimitv=c{ii};
          else error('Illegal pzaxis')
          end
        end
      elseif strcmp(f{ii},'initmodel')
        if ~isempty(c{ii})
          if isa(c{ii},'idmodel'), c{ii}=fidmodel(c{ii}); end
          if isa(c{ii},'fidmodel')
            initp=c{ii}; initpfile=initp;
            if initset=='t', initset='f'; end
          %elseif isnumeric(c{ii})&(length(c{ii})==1)
          %  if all(~isnan(c{ii}(:))), delay=c{ii}; end
          else
            error('Value of field initmodel is not allowed')
          end
        end
      elseif strcmp(f{ii},'reportfile')
        if all(~isnan(c{ii}(:))), rfile=c{ii}; end
      else
        error(['Field ''',f{ii},''' is not allowed in runmod. Type ''elis runmod'''])
      end
    end %for ii
    if strcmp(plot0,'off')|strcmp(plot0,'no'), plotdens=-abs(plotdens); end
  end
  %
  if length(varargin)+1>=6
    devrunmod=varargin{5};
    if ~isempty(devrunmod)&~isstruct(devrunmod), error('devrunmod is not a structure'), end
    if ~isstruct(devrunmod), devrunmod=[]; end
  end
  if isstruct(devrunmod)
    f=fieldnames(devrunmod);
    c=struct2cell(devrunmod);
    for ii=1:length(f)
      if strcmp(f{ii},'initweight')
        error('Field ''initweight'' needs to be moved to runmod from devrunmod')
        if all(~isnan(c{ii}(:)))&~isempty(c{ii}), initw=c{ii}; end
      elseif strcmp(f{ii},'newbasis')
        if strcmp(c{ii},'off') basistreat='f';
        elseif strcmp(c{ii},'on') basistreat='n';
        else error('basistreat value not allowed')
        end
      elseif strcmp(f{ii},'linesearch')
        if strcmp(c{ii},'on'), searchmode='l'; end
      elseif strcmp(f{ii},'displaymessages')
        if all(~isnan(c{ii}(:)))
          if strcmp(c{ii},'on'), dispmesg='y';
          elseif strcmp(c{ii},'off'), dispmesg='n';
          else dispmesg=c{ii}(1);
          end
        end
      elseif strcmp(f{ii},'calculatezeros')
        if strcmp(c{ii},'off')|strncmp(c{ii},'n',1)
          calcrnum='n';
        end
      elseif strcmp(f{ii},'calculatepoles')
        if strcmp(c{ii},'off')|strncmp(c{ii},'n',1)
          calcrdenom='n';
        end
      elseif strcmp(f{ii},'followroots')
        if isnumeric(c{ii})
          if length(c{ii})==1
            if c{ii}>=1
              pzfollown=round(c{ii});
            elseif isnan(c{ii})
            else error('Value of followroots in less than 1')
            end
          elseif isempty(c{ii})
          else
            error('Value of followroots in not scalar')
          end
        else error('Value of followroots in not a number')
        end
      elseif strcmp(f{ii},'checkiterctrl')
        if strcmp(c{ii},'off')|strncmp(c{ii},'n',1)
          if strcmp(itctrldrawnow,'b'), itctrldrawnow='d';
          elseif strcmp(itctrldrawnow,'i'), itctrldrawnow='n';
          end
        elseif strcmp(c{ii},'on')|strncmp(c{ii},'y',1)
          if strcmp(itctrldrawnow,'d'), itctrldrawnow='b';
          elseif strcmp(itctrldrawnow,'n'), itctrldrawnow='i';
          end
        end
      elseif strcmp(f{ii},'calldrawnow')
        if strcmp(c{ii},'off')|strncmp(c{ii},'n',1)
          if strcmp(itctrldrawnow,'b'), itctrldrawnow='i';
          elseif strcmp(itctrldrawnow,'d'), itctrldrawnow='n';
          end
        elseif strcmp(c{ii},'on')|strncmp(c{ii},'y',1)
          if strcmp(itctrldrawnow,'i'), itctrldrawnow='b';
          elseif strcmp(itctrldrawnow,'n'), itctrldrawnow='d';
          end
        end
      elseif strcmp(f{ii},'calculateCR')
        if strcmp(c{ii},'off')|strncmp(c{ii},'n',1)
          calculateCR=0;
        elseif strcmp(c{ii},'on')|strncmp(c{ii},'y',1)
          calculateCR=1;
        end
      elseif strncmp(f{ii},'compensate',9)
        compval=c{ii};
        if any(findstr('freqscal',compval)) | (any(findstr('freq',compval))&any(findstr('scal',compval)))
          cps=[cps,'f'];
        elseif any(findstr('represent',compval))
          cps=[cps,'r'];
        elseif any(findstr('fix',compval))
          cps=[cps,'x'];
        elseif any(findstr('all',compval))
          cps='frx';
        end
      else
        error(['Field ''',f{ii},''' is not allowed in devrunmod'])
      end
    end %for ii
  end
  if ~isempty(initpfile)&(~any(initset=='foi')&~(basistreat=='f'))
    error('Starting model can only be defined with initial setting from object or file')
  end
  %
  ffile=Fdat; pzaxis='';
  if any(domain=='zq')&isempty(fs)
    error('fs is empty for z-domain')
  end
  if any(dispmesg=='bym')
    fprintf('elis is running ...\n')
  end
  if exist('cc')
    [cp,cb]=fdident('private','constr2arr',cc,numord,denomord,domain);
    error('Handling of constraints is not yet ready')  
  end %transform constraints
else %old or obsolete call
  if ~isempty(rpf) %rpf not empty
    if ~any(dispmesg=='nc')
      fprintf(['elis is loading run parameters from file ''',rpf,''''])
    end
    load(rpf,'-mat'), if exist('x1m'), uv=x1m; end
    if ~exist('elisqadate'), elisqadate=''; end
    if ~any(dispmesg=='nc')
      if ~isempty(elisqadate), fprintf([', date: ',elisqadate]), end
      disp (' '),
    end
    if strcmp(paramtreat,'f')
      paramtreat='0'; %Correction for old files
    end
    if any(domain=='zq'), pzlimitv=2; end
    if length(plotmode)==3, plotmode=plotmode(2); end %Correction for old files
    if ~exist('Ffile'), Ffile=ffile; end
    if exist('Ffile'), ffile=Ffile; end %ffile remains
    if ~exist('lambdadlim'), lambdadlim=lambdadecr; %for compiler
    else lambdadecr=lambdadlim; %notation change
    end
  else %no run parameter file given
    if length(rppar)==0, NaNv=NaN;
      rppar=[rppar+0,NaNv(:,ones(1,3-length(rppar)))];
    end
    if ~all(finite(rppar(2:3)))
      error('Numerator and/or denominator orders are not properly given')
    end
    if any(rem(rppar(2:3),1)~=0)|any(rppar(2:3)<0)
      error('Numerator or denominator order is not nonnegative integer')
    end
  end
  %
  %Definition of run parameters from input arguments
  if narginelis>0 %Fdat
    if ~isempty(Fdat)
      ffile=Fdat;
    end
  end
  if isempty(ffile), error('No Fourier data are given'), end
  %
  %treat vdat
  if narginelis>1 %vdat
    if ~isempty(vdat)
      if isstr(vdat), vardef='f'; vfile=vdat;
      else vardef='u';
        if isnumeric(vdat)&((length(vdat(1,:))==2)|(length(vdat(1,:))==3))
          varx=vdat(:,2); vary=vdat(:,1);
          if length(vdat(1,:))==3 %covxy
            covxy=vdat(:,3);
          end
        elseif length(vdat(1,:))==1 %vector
          [varx,vary,covxy]=impvar(vdat);
        elseif iscell(vdat)
          varx=cell(size(vdat)); vary=varx;
          covxy=varx;
          for ii=1:length(vdat)
            varx{ii}=vdat{ii}(:,2); vary{ii}=vdat{ii}(:,1);
            if size(vdat{1},2)==3, covxy{ii}=covxy{ii}(:,3); end
          end
        else
          vsiz=size(vdat);
          error(sprintf('improper vdat with sizes (%.0f,%.0f)',vsiz(1),vsiz(2)))
        end
        if ~iscell(varx)
          if any(varx<0)|any(vary<0), error('Negative variance given'), end
          if any(imag(varx))|any(imag(vary)), error('Complex variance given'), end
        end
      end
    end
  end
  if isempty(vdat) & (strcmp(vardef,'f')&isempty(vfile))
    error('No variance data are given')
  end
  %
  %treat rppar
  if narginelis<=2, rppar=[]; end
  if length(rppar)>0
    if isstr(rppar)&(length(rppar)>3)
      if strcmp(MatlV(1:2),'4.')
        disp('Warning: rppar is a string, error in Matlab5')
      else %Matlab5
        error('rppar is a string')
      end
    end
    rppari=['s'+0,NaN,NaN,NaN,'fn'+0,NaN,1,'n'+0,NaN]; rppar=rppar(:)';
    if length(rppar)>length(rppari), error('rppar is too long'), end
    if isstr(rppar)&(length(rppar)>3)&(MatlV(1)>='5')
      error('rppar is a string')
    end
    rppari(1:length(rppar))=rppar;
    domainn=setstr(rppar(1));
    if ~strcmp(domain,domainn)&~isnan(domainn)&(domainn~=0)
      domain=domainn;
      numfixind=[]; numfixind=[]; denomfix=[]; denomfixind=[];
      if any(domain=='qz'), pzlimitv=2;
      elseif ~any(domain=='spwr')
        error('rppar(1) is not allowed')
      end
    end
    if ~isnan(rppari(2)), numord=rppari(2)+0; end
    if (numord<0)|(rem(numord,1)~=0)
      error(sprintf('Illegal value of numord: %.15g',numord))
    end
    if ~isnan(rppari(3)), denomord=rppari(3)+0; end
    if (denomord<0)|(rem(denomord,1)~=0)
      error(sprintf('Illegal value of denomord: %.15g',denomord))
    end
    if ~isnan(rppari(4)), fs=rppari(4)+0; end
    if any(domain=='zq')&isnan(fs), fs=1; end
    if (length(rppari)>4)&~isnan(rppari(5))&strcmp(setstr(rppari(5)),'a')
      if ~strcmp(paramtreat,'a')
        numfixind=[]; numfixind=[]; denomfix=[]; denomfixind=[];
      end
      paramtreat='a';
    end
  end
  if any(domain=='zq')
    stablimit=1; %lower limit of abs. value of unstable poles in z-domain
  else %s-domain or w-domain
    stablimit=0; %lower limit of real part of unstable poles in s-domain
  end
  if length(rppar)>0
    if ~isnan(rppari(6))&strcmp(setstr(rppari(6)),'t'), transtreat='t';
    elseif any(transtreat=='rc'), error(['transtreat=''',transtreat,''''])
    else transtreat='n';
    end
    if finite(rppari(7)), trord=rppari(7)+0; end
    if finite(rppari(8)), delayscale=rppari(8)+0; end
    %
    if strcmp(setstr(rppari(9)),'r'), stabmode='r';
    elseif strcmp(setstr(rppari(9)),'c'), stabmode='c';
    elseif strcmp(setstr(rppari(9)),'l'), stabmode='l';
    elseif ~strcmp(setstr(rppari(9)),'n')&finite(rppari(9))
      error('rppar(9) is not valid for stabmode')
    end
    if finite(rppari(10)), stablimit=rppari(10)+0; end
    if any(domain=='zq')
      if (stablimit<=0)|(stablimit>1)|(imag(stablimit)~=0)
        error('Stability limit given by rppar(10) is illegal for z-domain')
      end
    else %s-domain or w-domain
      if (stablimit>0)|(imag(stablimit)~=0)
        error(['Stability limit given by rppar(9) is illegal for ',domain,...
            '-domain'])
      end
    end
  end
  if (delayscale~=1)&~any(dispmesg=='ncs')
    fprintf('   delayscale: %.4g\n',delayscale)
  end
  %
  %treat fixp
  if narginelis>3 %fixp
    if ~isempty(fixp)
      if length(fixp)==1
        if strcmp(fixp,'f')|strcmp(fixp,'v')
          delaytreat=fixp;
        elseif strcmp(fixp,'n')
          numfix=[]; numfixind=[]; denomfix=[]; denomfixind=[]; paramtreat='n';
          delaytreat='v';
        elseif strcmp(fixp,'0')
          paramtreat='0';
          delaytreat='f'; numfix=[]; numfixind=[]; denomfix=1;
          if any(domain=='spwr'), denomfixind=[denomord];
          else denomfixind=0;
          end
        elseif fixp==0, error('fixp=0 not allowed, use fixp=''0'' instead')
        else error('fixp not allowed')
        end
      else %fixed parameters given in array or file
        numfix=[]; numfixind=[]; denomfix=[]; denomfixind=[];
        if isstr(fixp)&all(rem(fixp,1)==0)&all((fixp>0)&(fixp<256))
          if ~any(dispmesg=='nc')
            disp(['elis loads fixed parameters from file ',fixp])
          end
          fixp=loadasc(fixp,'flat');
        end
        [h,s]=size(fixp); if s~=2, error('fixp is not an nx2 array'), end
        ind=find(fixp(:,1)<=numord+1);
        if ~isempty(ind)
          numfixind=fixp(ind,1)'-1; numfix=fixp(ind,2)'; fixp(ind,:)=[];
          if any(numfixind<0)|any(rem(numfixind,1)~=0)
            error('Indices in fixp are not positive integers')
          end
        end
        if ~isempty(fixp)
          ind=find(fixp(:,1)<=numord+1+denomord+1);
          if ~isempty(ind)
            denomfixind=fixp(ind,1)'-numord-2;
            denomfix=fixp(ind,2)'; fixp(ind,:)=[];
          end
        end
        if ~strcmp(paramtreat,'a')
          if isempty([numfix(:);denomfix(:)]), paramtreat='n';
          else paramtreat='d';
          end
        else
          if ~any(domain=='zq')
            error('Allpass design is only possible in z-domain')
          end
        end
        if strcmp(paramtreat,'a')&~isempty(denomfix)
          error('Denominator coefficients may not be fixed for allpass design')
        end
        if ~isempty(fixp)
          if fixp(1)~=numord+denomord+3, error('Invalid last index in fixp'), end
          delay=fixp(1,2); delaytreat='f';
        else
          delaytreat='v';
        end
      end
    end %~isempty(fixp)
  end
  if ~isempty([numfix,denomfix])&any(domain=='qp')
    error('Parameter fixing is not yet inplemented for orthopol')
  end
  %
  if narginelis<5, rpalg=[]; end
  if ~isempty(rpalg)
    if isstr(rpalg)&(length(rpalg)>3)
      error('rpalg is a string')
    end
    rpalgi=['gt'+0,50,1e-6,0,10,0.1,1e-10,'n'+0,basistreat+0]; rpalg=rpalg(:)';
    rpalgind=find(~isnan(rpalg));
    if ~isempty(rpalgind), rpalgi(rpalgind)=rpalg(rpalgind); end
    if ~isnan(rpalgi(1)), algtn=setstr(rpalgi(1));
      if strcmp(algtn,'g'), algtype='NG'; algstr='Newton-Gauss';
      elseif strcmp(algtn,'r'), algtype='NR'; algstr='Newton-Raphson';
      elseif strcmp(algtn,'l'), algtype='LM'; algstr='Levenberg-Marquardt';
      elseif strcmp(algtn,'m'), algtype='LMsvd'; algstr='Levenberg-Marquardt with svd';
      elseif strcmp(algtn,'s'), algtype='svd'; algstr='singular value decomposition';
      else error(['Invalid algorithm type ',algtn,' in rpalg'])
      end
    end
    if ~isnan(rpalgi(2)), initsn=setstr(rpalgi(2));
      if any(initsn=='lawyvexisfrtgb')
        initset=initsn;
      else error(['Invalid initset ',initsn,' in rpalg'])
      end
    end
    if ~isnan(rpalgi(3)), itmax=rpalgi(3)+0; end
    if ~isnan(rpalgi(4)), rcostvar=rpalgi(4)+0; end
    if ~isnan(rpalgi(5)), rparvar=rpalgi(5)+0; end
    if ~isnan(rpalgi(6)), lambdadecr=rpalgi(6)+0; end
    if lambdadecr<0, lambdadecr=inf; lambdamin=abs(lambdadecr); end
    %if lambdadecr<0, error('lambdadecr<0'), end
    if ~isnan(rpalgi(7)), lambda0=rpalgi(7)+0; end
    if lambda0<0, error('lambda0 is negative'), end
    if ~isnan(rpalgi(8)), lambdalim=rpalgi(8)+0; end
    if ~isnan(rpalgi(9))&strcmp(setstr(rpalgi(9)),'l'), searchmode='l'; end
    if ~isnan(rpalgi(10)), basistreat=setstr(rpalgi(10));
    elseif initset=='f', basistreat='f';
    end
    if ~strcmp(basistreat,'f')&~strcmp(basistreat,'n')
      error(sprintf('basistreat=%.3g is not valid',basistreat))
    end
  end
  %
  if narginelis<6, rppl=[]; end
  pzaxis=[];
  if ~isempty(rppl)
    if isstr(rppl)
      if strcmp(MatlV(1:2),'4.')
        disp('Warning: rppl is a string, error in Matlab5')
      else %Matlab5
        error('rppl is a string')
      end
    end
    rppli=[1,0,NaN,NaN,NaN,'icc'+0,NaN,NaN,NaN,NaN,1,'yf'+0,0,'b'+0,NaN,NaN];
    rppl=rppl(:)'; rppli(1:length(rppl))=rppl;
    if ~isnan(rppli(1)), plotdens=rppli(1)+0; end
    if ~isnan(rppli(2)), fmin=rppli(2)+0; end
    if ~isnan(rppli(3)), fmax=rppli(3)+0; end
    if ~isnan(rppli(4)), amin=rppli(4)+0; end
    if ~isnan(rppli(5)), amax=rppli(5)+0; end
    if ~isnan(rppli(6)), plotmode=setstr(rppli(6)); end
    if length(plotmode)==3, plotmode=plotmode(2); end %earlier: lin or log
    if ~isnan(rppli(7))
      calcrnum=setstr(rppli(7));
      if ~strcmp(calcrnum,'c')&~strcmp(calcrnum,'n')
        error('Invalid rppli(7)')
      end
    end
    if ~isnan(rppli(8)), calcrdenom=setstr(rppli(8)); end
    if all(~isnan(rppli(9:12)))
      pzlimit='n';
      if all(rppli(9:12)==('p   '+0)), pzaxis='p';
      elseif all(rppli(9:12)==('a   '+0)), pzaxis='a';
      elseif ~any(diff(rppli(9:12)+0)), pzlimit='y'; pzlimitv=rppl(9)+0;
        pzaxis=[];
      else pzaxis=rppl(9:12)+0;
      end
    elseif any(~isnan(rppli(9:12)))
      pzlimit='y'; pzaxis=rppl(9:12)+0;
    end
    if ~isnan(rppli(13)), pzfollown=rppli(13)+0; end
    %if ~isnan(rppli(14)), dispmesg=setstr(rppli(14)); end %already done
    if ~isnan(rppli(15)), freqax=setstr(rppli(15)); end
    if ~isnan(rppli(16)), plotdensoffs=rppli(16)+0; end
    if ~isnan(rppli(17)), itctrldrawnow=rppli(17)+0; end
    if ~isnan(rppli(18)), plotdenstime=rppli(18)+0; end
    if ~isnan(rppli(19)), plotlevel=setstr(rppli(19)); end
    if ~any(dispmesg=='ymncbs')
      error('dispmesg is not ''m'', ''y'', ''n'', ''c'', ''s'', or ''b''')
    end
  end
  if dispmesg=='m', dispmesg='y'; end
  %
  if narginelis<7, initp=[]; end
  if ~isempty(initp)
    if isa(initp,'idmodel'), initp=fidmodel(initp); end
    if (length(initp)==1)&isa(initp,'double')&finite(initp) %delay value
      delay=initp;
    elseif isa(initp,'fidmodel')|...
        ((isa(initp,'double')|isa(initp,'char'))&all(finite(initp)))
      %parameter vector or file name
      initpfile=initp;
      if ~any(initset=='fi')&~any(domain=='pq')
        error(['Initial parameter vector is given while initset=''',initset,'''']);
      end
    else
      error('initp not allowed')
    end
  end
  if strcmp(initset,'f')&isempty(initpfile)
    error('No starting values are given, though initset=''f'' is defined')
  elseif strcmp(initset,'i')&isempty(initpfile)
    error('No starting values are given, though initset=''i'' is defined')
  end
  if any(domain=='pq')&(basistreat=='f')&(initset=='f')&isempty(initpfile)
    error('initp is not given, though required by orthopol')
  end
  %
  if narginelis>=8 %rpfs
    if ~isstr(rpfs), error('rpfs is not a string array'), end
    while ~isempty(rpfs)
      fnam=rpfs(1,:); rpfs(1,:)=[];
      ind=find(fnam==' '); if ~isempty(ind), fnam(ind)=''; end
      [fnam,dummy,ext]=fnamanal(fnam);
      if isempty(ext), error(['No extension in the file name ',fnam,' in rpfs'])
      end
      if strcmp(ext,'rep'), rfile=fnam;
      elseif strcmp(ext,'par')|strcmp(ext,'pbn')|strcmp(ext,'pnt'), pfile=fnam;
      elseif strcmp(ext,'cov')|strcmp(ext,'cbn')|strcmp(ext,'cnt'), cfile=fnam;
      else error(['Invalid file name extension ',ext,' in rpfs'])
      end
    end %while
  end
  %
  %End of run parameter handling
  if any(dispmesg=='bym')
    fprintf('elis is running ...\n')
  end
  %
end
global elis_max_iteration_number
if ~isempty(elis_max_iteration_number)
  if isnumeric(elis_max_iteration_number)&(length(elis_max_iteration_number)==1)&...
      finite(elis_max_iteration_number)&(elis_max_iteration_number>=0)
    itmax=min(itmax,elis_max_iteration_number);
    if ~any(dispmesg=='ncs')
      disp(['Maximum number of iterations is set by force to ',num2str(itmax),' in elis'])
    end
  end
end
%
if ~any(domain=='spzqwr'), error(['Invalid domain ''',domain,'''']), end
if domain=='r'
  if ~exist('tauR'), error('tauR is not given'), end
  if length(tauR)~=1, error('TauR is not a scalar'), end
  if ~finite(tauR)|~isreal(tauR)|(tauR<=0)
    error('tauR is not a positive scalar')
  end
end
algorithm.initdelay=delay;
if delayscale~=1, delay=delay/delayscale; end %internal delay representation
existIteration=1;
hmenu=feval('findobj',0,'Type','uimenu');
if ~exist([cfn,setstr([46, 112])])&~exist([cfn,setstr([46, 109])])
  error(sprintf(['This is not a supported fdident toolbox copy:\n',...
    '  please register or email to the distributors at fdident@vub.ac.be']))
end
for ih=length(hmenu):-1:1
  hIt1=feval('findobj',hmenu(ih),'tag','iteration_menu');
  if ~isempty(hIt1), existIteration=1; end
end
if ~existIteration
  calldrawnow=0; calliterctrl=0;
elseif isPC %Matlab5 under Win95 still greys out
  calldrawnow=0; calliterctrl=1;
else %regular case, e.g. Matlab5 on any platform, still may grey out
  calldrawnow=1; calliterctrl=1;
end
%calldrawnow
if any(itctrldrawnow=='ni'), calldrawnow=0; end
if any(itctrldrawnow=='nd'), calliterctrl=0; end
%
%calldrawnow=0; calliterctrl=0; %Force to eliminate drawnows
%calldrawnow=1; calliterctrl=1; %Force to call drawnows
%
itctrllastchecked='Continue';
if calldrawnow, drawnow, end
%
if strcmp(freqax,'f'), faxsc=1; else faxsc=2*pi; end
%
if ~any([numfix,denomfix])
  if ~any(dispmesg=='ncs')&...
       ( strcmpi(algtype,'ng')|strcmpi(algtype,'nr')|...
         strcmpi(algtype,'lm')|any(initset=='lw') )
    disp('No fixed nonzero parameters')
  end
  if strcmpi(algtype,'ng')|strcmpi(algtype,'nr')
    if ~any(dispmesg=='ncs')
      disp('  algtype changed to svd')
    end
    algtype='svd'; algstr='singular value decomposition';
  elseif strcmpi(algtype,'lm')
    if ~any(dispmesg=='ncs')
      disp('  algtype changed to lmsvd')
    end
    algtype='LMsvd'; algstr='Levenberg-Marquardt with svd';
  end
  if any(initset=='lw')
    if initset=='l', initset='s';
    elseif initset=='w', initset='y';
    end
    if ~any(dispmesg=='ncs')
      disp(['  setting of starting values changed to ''',initset,''''])
    end
  end
end
if transtreat=='t'
  if ~exist('trord'), trord=[]; end
  if any(domain=='zq'), trordsugg=max(numord,denomord)-1;
  elseif any(domain=='sp'), trordsugg=max(numord,denomord)-1;
  else error(['Transients cannot be estimated for ',domain,'-domain'])
  end
  if isempty(trord)
    trord=trordsugg;
  elseif trordsugg>trord
    error('Transient order is too low')
  end
else trord=-1;
end
%if trord<0, transtreat='n'; end
trno=trord+1;
%--------------------------------------------
%1a. Get the input data and set the values of some constants
[expno,expi,freq,uv,y1m]=feval(cfn,ffile,expi);
if calldrawnow, drawnow, end
%
if (itmax==0)&all(initset~='f')
  %disp('Warning! Requesting Cp is dubious after the initial step only')
end
if ~isempty(Fdat)&~isstr(Fdat)&(expno>1)&(length(expi)==1)&~any(dispmesg=='ncs')
  fprintf(['WARNING: Fourier vector contains more than one experiment\n',...
          '    only one of them will be used\n'])
end
if ~finite(fmin)
  %if any(domain=='zq'), fmin=0;
  fmin=(min(freq)); %-0.05*max(freq))*faxsc; 
end
if ~finite(fmax)
  fmax=max(freq)*faxsc;
  %fmax=1.05*fmax;
end
uv=uv(:,1); y1m=y1m(:,1); %First input, first output
%Recognition and handling of stopband frequencies
indsb=find((uv==0)&(y1m==0));
if length(indsb)>0
  uv(indsb)=ones(length(indsb),1);
  if ~any(dispmesg=='ncs')
    fprintf(['WARNING: %0.f stopband frequencies (0/0) found\n',...
      '   stopband input amplitudes are set to value 1\n'],length(indsb)/expno)
  end
end
tfm=y1m./uv;
F=length(freq);
%
if ~any(dispmesg=='ncs')
  if expno==1, fprintf('   Number of frequencies: %.0f\n',F)
  else fprintf('   Number of frequencies: %.0f x %.0f\n',expno,F/expno)
  end
  fprintf(['   Maximum of the frequency vector: ',num2str(max(freq)),' Hz\n'])
end
if any(domain=='spwr'),
  if min(freq)>-max(abs(freq))/2
    fsn=(min(freq)+max(freq))/2*2*pi; %scaling (angular) frequency
  else
    fsn=max(abs(freq))/2*2*pi; %scaling (angular) frequency
  end
  if fsn<=0, fsn=1; end
  if ~finite(fs), fs=fsn; end
  if ~any(dispmesg=='ncs')
    if (fs/fsn>2)|(fs/fsn<0.5)
      fprintf(['WARNING: scaling angular frequency %.2g rad*Hz, ',...
                'suggested: %.2g rad*Hz\n'],fs,fsn)
    else
      fprintf(['   Scaling angular frequency for internal calculations: ',...
              num2str(fs),' rad*Hz\n']);
    end
    if max(freq)>fmax/faxsc*(1.0001),
      disp(['freqmax=',num2str(max(freq)),...
        ' exceeds the displayed frequency range fmax=',num2str(fmax/faxsc)])
    end
  end
  freq=freq/fs; %scaling
  delay=delay*fs; %spwr domain
  if any(domain=='sp')
    numfix=numfix.*(fs.^(numord-numfixind));
    denomfix=denomfix.*(fs.^(denomord-denomfixind));
    pzsc=fs; %scale factor for poles and zeros
  elseif strcmp(domain,'w') %Warburg
    numfix=numfix.*(sqrt(fs).^(numord-numfixind));
    denomfix=denomfix.*(sqrt(fs).^(denomord-denomfixind));
    pzsc=sqrt(fs); %scale factor for poles and zeros
  elseif domain=='r'
    tauR=tauR*fs; %scaling
    pzsc=fs; %scale factor for poles and zeros
  end
elseif any(domain=='zq')
  pzsc=1;
  freq=freq/fs; %normalization
  if max(freq)>0.5+100*eps
    if max(freq)>1.0001*0.5, etxt='';
    else etxt=sprintf('\nalthough by a small amount only');
    end
    error(['freqmax=',num2str(max(freq*fs)),...
      ' exceeds the half of the sampling frequency fs/2=',num2str(fs/2),etxt])
  end
end
%
if strcmp(vardef,'f')
  [varx,vary,covxy]=impvar(vfile);
else %vectors varx and vary given
  if length(varx)==1
    varx=varx*ones(length(uv),1);
  end
  if length(vary)==1
    vary=vary*ones(length(y1m),1);
  end
  if ~isempty(covxy)&(length(covxy)==1)
    covxy=covxy*ones(length(uv),1);
  end
end
if length(varx)<length(uv)
  if rem(length(uv)/length(varx),1)==0
    expno=length(uv)/length(varx);
    fvarr=zeros(length(varx),expno);
    if length(freq)~=length(varx), fvarr(:)=freq; end
    if ~any(any(diff(fvarr')))
      expi=1:expno;
      varx0=varx; vary0=vary; covxy0=covxy;
      for ii=2:expno
        varx=[varx;varx0]; vary=[vary;vary0]; covxy=[covxy;covxy0];
      end
    else
      error(sprintf(['Variance vector is shorter than amplitude vector,'...
          ' but contents of frequency\nvector is not repeated',...
          ' with supposed experiment length %.0f'],length(varx)))
    end
  end
  if length(varx)==length(uv)
    if ~any(dispmesg=='ncs')
      fprintf(['\nWarning! %.0f experiments are used simultaneously, ',...
        'with %.0f frequencies each\n\n'],expno,length(varx)/expno)
    end
  end
end
if length(varx)~=length(uv)
  error(sprintf(['Lengths of amplitude and variance vectors',...
                ', %.0f and %.0f, are incompatible'],length(uv),length(varx)))
end
if length(varx(1,:))>1 %i1 and o1 from MIMO variance
  varx=varx(:,1); vary=vary(:,1);
  if ~isempty(covxy), covxy=covxy(:,1); end
end
if any(varx<0)|any(vary<0), error('Negative variance value'), end
if any( (varx==0)&(vary==0) )
  error('Zero input and output variances given at a frequency point')
end
if ~isempty(covxy)
  ind=find(abs(covxy)>(1-10*eps)*sqrt(varx.*vary));
  if ~isempty(ind)
    error(sprintf(['abs(covxy) is too large compared to ',...
        'sqrt(varx*vary), e.g. for k=%.0f'...
        '\nelis cannot perform calculations.'],ind(1)))
  end
end
%
%-------------------------------------------
%1b. Initialization of variables
if strcmp(paramtreat,'a')
  if any(domain=='wp')
    error(['Cannot design allpass transfer function for domain ''',domain,''''])
  end
  if numord~=denomord, error('Numerator and denominator orders differ'), end
  if domain=='z'
    denomfixind=numord-numfixind;
    denomfix=numfix(length(numfix):-1:1);
  else %'s' or 'r'
    denomfixind=numfixind;
    denomfix=numfix;
  end
end
numnotfixind=[0:numord]; numnotfixind(numfixind+1)=[];
denomnotfixind=[0:denomord]; denomnotfixind(denomfixind+1)=[];
lnnf=length(numnotfixind); ldnf=length(denomnotfixind);
Nfp=lnnf+ldnf+trno;
nfn0=sum([numfix,denomfix,0]~=0); %Fixed nonzero coeffs
Nfpm=Nfp; %Nfpm for calculation of d2ExE only
if strcmp(paramtreat,'a'), Nfp=Nfp-ldnf; end
if strcmp(coeffs,'complex'), Nfp=Nfp*2; Nfpm=Nfpm*2; end
if delaytreat=='v', Nfp=Nfp+1; Nfpm=Nfpm+1; lvd=1; rv=1;
else lvd=0; rv=[];
end
%N: degrees of freedom
N=2*F-Nfp; %2*freqno - number of free parameters
if nfn0==0, N=N+1; end %correction because of the norm=1 constraint
if any(freq==0), N=N-1; end %correction because dc is included
%
if ~any(dispmesg=='ncs')
  fprintf('   Number of estimated parameters: %.0f',Nfp)
  fprintf([', model: %.0f/%.0f, ',domain,'-domain\n'],numord,denomord)
end
if N<0, error('Degrees of freedom is not positive: not enough frequencies'), end
delayinit=delay; %Starting value of the delay
%
if strcmp(stabmode,'l')|strcmp(mphmode,'l')
  if strcmp(algtype,'NG')|strcmp(algtype,'NR')
    algtype='LM'; algstr='Levenberg-Marquardt';
    warning(sprintf(['Pole stabilization or forcing of zeros to minimum phase by step limiting:\n',...
      'algorithm type is changed for  to Levenberg-Marquardt']))
  elseif strcmp(algtype,'svd')
    algtype='LMsvd'; algstr='Levenberg-Marquardt with svd';
    warning(sprintf(['Pole stabilization or forcing of zeros to minimum phase by step limiting:\n',...
      'algorithm type is changed for  to Levenberg-Marquardt']))
  elseif strncmpi(algtype,'LM',2)
  else
    error(['algtype ''',algtype,''' is not recognized'])
  end
end
%
lambdadec=inf; lambdanew=lambda0;
if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
  lambda=lambda0; lambdaold=lambda0;
  if lambdadecr==0, lambda=0; end
  if lambdamin>lambda, lambda=lambdamin; end
end
%
t0=clock;  %start time of initial settings
clv=fix(t0);
if ~any(dispmesg=='nc')
  fprintf('Start time: %.0f:%02.0f:%02.0f\n',clv(4),clv(5),clv(6))
end
plottime=0; pztime=0; %reset plotting and pole/zero calculation times
exd=exp(-j*pi*freq*delay*delayscale); %delay term
y1mcexd=y1m.*conj(exd); uvexd=uv.*exd;
%negligible complex number for preallocation of complex arrays:
complmin=j*realmin;
if nargoutelis>=2, cfv=[];
  if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd'), cfv(50,2)=0;
  else cfv(50,1)=0;
  end
end
%---------------------------------------------
%---------------------------------------------
if calldrawnow, drawnow, end
%2. SETTING THE START VALUES
txti=issset(initset,domain);
if initset=='f'
  if isstr(initpfile), txti=['file ''',initpfile,'''']; end
end
if ~any(dispmesg=='ncs')
  itxt=['   Setting the starting values, mode: ',txti];
  disp([itxt,' ...'])
  if any(domain=='pq')&(basistreat=='f')&(initset=='f')
    fprintf('   Using orthonormal basis from ''initp''\n')
  end
end
fixbatxt='';
if length(numfix)>0
  for k=1:length(numfix)
    if ~isempty(fixbatxt), fixbatxt=[fixbatxt,',']; end
    if any(domain=='spwr')
      fixbatxt=[fixbatxt,sprintf('b%.0f',numord-numfixind(k))];
    else
      fixbatxt=[fixbatxt,sprintf('b%.0f',numfixind(k))];
    end
  end
end
if length(denomfix)>0
  for k=1:length(denomfix)
    if ~isempty(fixbatxt), fixbatxt=[fixbatxt,',']; end
    if any(domain=='spwr')
      fixbatxt=[fixbatxt,sprintf('a%.0f',denomord-denomfixind(k))];
    else
      fixbatxt=[fixbatxt,sprintf('a%.0f',denomfixind(k))];
    end
  end
end
if ~isempty(fixbatxt), fixbatxt=[fixbatxt,' fixed']; end
if paramtreat=='0', fixtxt=fixbatxt; fixbatxt='';
  %fixtxt=['Coeff of ',domain,'^0 fixed'];
elseif paramtreat=='a', fixtxt='Allpass';
  if nfn0==0, fixtxt=[fixtxt,', norm=1'];
  else fixtxt=[fixtxt,', fixed par'];
  end
elseif nfn0==0, fixtxt='Norm=1';
elseif ~isempty('fixbatxt')&(length(fixbatxt)<=18)
  fixtxt=fixbatxt; fixbatxt='';
else fixtxt='Fixed parameters';
end
if any(initset=='lwsyvuaxirt')
  if ~any(dispmesg=='ncs')
    if transtreat=='t', trtxt=sprintf(' + transients (%.0f/%.0f)',trno-1,denomord);
    else trtxt='';
    end
    if ~isempty(fixbatxt), fixbatxt=[' ',fixbatxt]; end
    disp(['   ',fixtxt,fixbatxt,trtxt])
  end
end
if ~any(dispmesg=='ncs')
  if any(stabmode=='rc')
    stbtxt='   Poles are stabilized by ';
    if stabmode=='r', stbtxt=[stbtxt,'reflection'];
    elseif stabmode=='c', stbtxt=[stbtxt,'contraction'];
    end
    disp(stbtxt)
  elseif any(stabmode=='l')
    disp('Solution not accepted because of unstable poles')
  end
  if any(mphmode=='rc')
    mphtxt='   Phases of zeros are minimized by ';
    if mphmode=='r', mphtxt=[mphtxt,'reflection'];
    elseif mphmode=='c', mphtxt=[mphtxt,'contraction'];
    end
    disp(mphtxt)
  elseif any(stabmode=='l')
    disp('Solution not accepted because of non-minimum phase zeros')
  end
end
%
condinum=NaN; condinumJ=NaN;
condinumd2Cdp2=NaN; %condition number of Q=d^2C/dp^2
cycle=0;
%
%INITIAL SETTINGS
if ~isempty(initpfile)
  if (transtreat=='t')&any(domain=='pq')&(initset=='f')
    error('transtreat==''t'', initial values cannot be loaded')
  end
  if isstr(initpfile)&~any(dispmesg=='ncs')
    if initset=='f'
      disp(['   Reading starting values from file ''',initpfile,''''])
    elseif initset=='i' %get model
      disp(['   Reading model from file ''',initpfile,''''])
    else %get orthopol basis
      disp(['   Reading earlier basis from file ''',initpfile,''''])
    end
  end
  [domainf,numf,denomf,delayf,fsf,Znum,Zdenom,comments,fdate,trnumf,Znumt]=...
      imppar(initpfile,fs);
  %
  if initset=='f'
    num=numf; denom=denomf;
    if trno==length(trnumf), trnum=trnumf; else trnum=zeros(1,trno); end
    if (transtreat=='t')&isempty(trnum)
      error('In initial setting from data trnum is empty')
    end
    %trord=length(trnum)-1; trno=trord+1;
    if delayscale~=1, delayf=delayf/delayscale; end
    if any(domain=='pq') %orthopol
      if basistreat=='f' %take orthogonal basis from file
        vnum=orthopol(numord,domain,freq,[],Znum);
        vdenom=orthopol(denomord,domain,freq,[],Zdenom);
        if transtreat=='t'
          vnumt=orthopol(trord,domain,freq,[],Znumt);
        end
      end
    else
      Znumt=[];
    end
    if ~strcmp(domainf,domain)&~(any(domainf=='sp')&any(domain=='sp'))
      error(['Domain of file is ''',domainf,''', not ''',domain,''''])
    end
    if fsf~=fs
      error(sprintf('Sampling freqs fs=%.3g and fsf=%.3g are inconsistent',...
        fs,fsf))
    end
    if delayf~=delay,
      if ((delay~=0)|(length(fixp)==2))&~any(dispmesg=='ncs')
        fprintf(['WARNING: Delay given by initp: %.3g differs from ',...
            'previously defined %.3g.\n'],delayf*delayscale,delay*delayscale)
        fprintf('Delay=%.3g will be used.\n',delayf*delayscale)
      end
      delay=delayf;
    end
    if (length(num)~=(numord+1))|(length(denom)~=(denomord+1)),
      error('parameter file inconsistent with given orders')
    end
    if any(abs(num(numfixind+1)-numfix)>1000*eps)
      error('parameter file inconsistent with fixed numerator coefficients')
    end
    if ~strcmp(paramtreat,'a') %not allpass
      if any(abs(denom(denomfixind+1)-denomfix)>10*eps)
        error('parameter file inconsistent with fixed denominator coefficients')
      end
    else %allpass
      if any(denom~=num(numord+1:-1:1))
        error('parameter file does not define allpass transfer function')
      end
    end
    if any(domain=='pq')&(basistreat=='n')
      if any(domainf=='pq')
        %nums=num*pnumarr; denoms=denom*pdenomarr;
        %The expressions would be OK, but they are numerically badly conditioned
        Nf=orthpval(num,Znum,freq);
        Df=orthpval(denom,Zdenom,freq);
      else %initial fit given in different domain
        Nf=tfcalc(exppar(domainf,num,1,0,fsf,[],[],[],[],[],[],[],coeffs,tauR),freq);
        Df=tfcalc(exppar(domainf,denom,1,0,fsf,[],[],[],[],[],[],[],coeffs,tauR),freq);
      end
      cfdenom=varx.*abs(Nf).^2+vary.*abs(Df).^2;
      if ~isempty(covxy)
        cfdenom=cfdenom-2*real(covxy.*(conj(exd).^2).*conj(Nf).*Df);
      end
      scfdenom=sqrt(cfdenom);
      %Basis of orthopol from previous parameters, maybe not orthopol
      [vnum,Znum]=orthopol(numord,domain,freq,uv./scfdenom);
      num=svdsolve(vnum./scfdenom(:,ones(1,size(vnum,2))),Nf./scfdenom,coeffs);
      [vdenom,Zdenom]=orthopol(denomord,domain,freq,y1m./scfdenom);
      denom=svdsolve(vdenom./scfdenom(:,ones(1,size(vdenom,2))),Df./scfdenom,coeffs);
      if transtreat=='t'
        [vnumt,Znumt]=orthopol(trord,domain,freq,ones(size(uv))./scfdenom);
        if any(domainf=='pq')
          Nft=orthpval(numt,Znumt,freq);
        else %initial fit given in different domain
          Nft=tfcalc(exppar(domainf,numt,1,0,fsf,[],[],[],[],[],[],[],coeffs,tauR),freq);
        end
        numt=svdsolve(vnumt./scfdenom(:,ones(1,size(vnumt,2))),Nft./scfdenom,coeffs);
      end
    end %(domain=='p')&(basistreat=='n')
  end %initset=='f'
else %initpfile empty
  Znum=[]; Zdenom=[]; Znumt=[]; %allow common saving
  if any(initset=='fi')
    error(['No initial model is given with initset==''',initset,''''])
  end
end %initial setting from file
if initset=='i'
  warning('This call of iqml is obsolete, and will be eliminated')
end
%---------------------------------------------
if strcmp(initset,'t'), initsetv='asvi';
else initsetv=initset;
end
cf=inf; iis=0;
pvsave.initset='s'; %for compiler
for initset=initsetv
  ndval=0;
  iis=iis+1;
  if length(initsetv)>1
    if initset=='i', istxt=['''iqml'', from ''',issset(pvsave.initset,domain),''''];
    else istxt=['''',issset(initset,domain),'''',' for initial setting ...'];
    end
    if ~any(dispmesg=='nc')
      disp(['   Trying ',istxt])
    end
  end
  if strcmp(initset,'e') %equation error method of the signal processing toolbox
    if any(domain=='wr')
      error(['Equation error method cannot be used with domain ''',domain,''''])
    elseif transtreat=='t'
      error('Transients cannot be treated with the eem method')
    end
    if exist(['invfreq',domain])==0
      if any(domain=='sz')
        error(sprintf(['The function m-file ''invfreq',domain,''' of the ',...
            'Signal Processing Toolbox\n      is required for the execution of',...
            ' the equation error method']))
      else
        if any(domain=='pq')
          error('eem cannot be used with orthopol')
        else %rw
          error(['eem cannot be used in domain ''',domain,''''])
        end
      end
    end
    %weightf=ones(length(vary),1)./(vary);
    if isempty(covxy)
      weightf=abs(uv.^2)./(2*(vary+varx.*abs(tfm.*tfm)));
    else
      weightf=abs(uv.^2)./(2*(vary+varx.*abs(tfm.*tfm)...
        -2*real(covxy.*conj(tfm)) ));
    end
    [num,denom]=feval(['invfreq',domain],(conj(exd).^2).*tfm,...
      freq*2*pi,numord,denomord,weightf);
    trnum=[];
    lfix=length([numfix,denomfix]);
    if paramtreat=='a', lfix=lfix/2; end
    if lfix>1
      error('Cannot fix more than 1 coefficient with the eem method')
    elseif lfix==1 %one fixed parameter
      if paramtreat=='a' %allpass
        if domain=='z'
          num=0.5*(num+denom(length(denom):-1:1)); denom=num(length(num):-1:1);
        else %s-domain and r-domain
          wm1=(-1).^[length(num)-1:-1:0];
          num=0.5*(num+wm1.*denom); denom=wm1.*num;
        end
      end
      if ~isempty(numfix)
        if abs(numfix)<100*eps
          error('Fixed parameter must not be very small')
        end
        if abs(num(numfixind+1))<100*eps
          error('Parameter to be fixed is too small with the eem method')
        end
        mgain=numfix/num(numfixind+1);
      end
      if ~isempty(denomfix)
        if abs(denomfix)<100*eps
          error('Fixed parameter must not be very small')
        end
        if abs(denom(denomfixind+1))<100*eps
          error('Parameter to be fixed is too small with the eem method')
        end
        mgain=denomfix/denom(denomfixind+1);
      end
      num=mgain*num; denom=mgain*denom;
    else %no fixed parameter
      num=num/norm([num,denom]); denom=denom/norm([num,denom]);
    end
  elseif any(initset=='lwsyvuaxir')
    %LS solution of xw*num'-yw*denom'=0
    %For transients, xw*num'-yw*denom'+tr=0
    %using the form  J*unknownvect=fixvect --> unknownvect=J\fixvect
    %first assign memory locations for variables
    p=0;
    num=zeros(1,numord+1); denom=zeros(1,denomord+1);
    if transtreat=='t', trnum=zeros(1,trno); else trnum=[]; end
    if ~any(domain=='pq') %not orthopol
      fixvect=[]; fixvect(length(uv),1)=complmin; %preallocate
      unknownvect=ones(length([numnotfixind,denomnotfixind]),1); %preallocate
      if transtreat=='t', unknownvect=[unknownvect;ones(trno,1)]; end
      J=[]; tJ=[];
      if length(unknownvect)>0
        J(length(uv),length(unknownvect))=complmin; %preallocate
      end
      if any(domain=='szwr')
        xwJ=[]; xwJ(F,numord+1)=complmin; %preallocate
        for p=0:numord,
          if any(domain=='zq'),
            xwJ(:,p+1)=uv.*exp(-p*j*2*pi*freq);
          elseif any(domain=='s'),
            xwJ(:,p+1)=uv.*((j*2*pi*freq).^(numord-p));
          elseif strcmp(domain,'w'),
            xwJ(:,p+1)=uv.*(sqrt(j*2*pi*freq).^(numord-p));
          elseif strcmp(domain,'r'),
            xwJ(:,p+1)=uv.*(tanh(j*2*pi*freq*tauR).^(numord-p));
          end
        end
        ywJ=[]; ywJ(F,denomord+1)=complmin; %preallocate
        if strcmp(transtreat,'t'), tJ=[]; 
          if trno>0, tJ(F,trno)=complmin; end
        end
        for p=0:max([denomord;trno])
          if any(domain=='zq'),
            if p<=denomord
              ywJ(:,p+1)=y1m.*exp(-p*j*2*pi*freq);
            end
            if (transtreat=='t')&(p<trno)
              tJ(:,p+1)=exp(-p*j*2*pi*freq);
            end
          elseif any(domain=='s'),
            if p<=denomord
              ywJ(:,p+1)=y1m.*((j*2*pi*freq).^(denomord-p));
            end
            if (transtreat=='t')&(p<trno)
              tJ(:,p+1)=(j*2*pi*freq).^(denomord-p);
            end
          elseif strcmp(domain,'w'),
            if p<=denomord
              ywJ(:,p+1)=y1m.*(sqrt(j*2*pi*freq).^(denomord-p));
            end
            if (transtreat=='t')&(p<trno)
              tJ(:,p+1)=sqrt(j*2*pi*freq).^(denomord-p);
            end
          elseif strcmp(domain,'r'),
            if p<=denomord
              ywJ(:,p+1)=y1m.*(tanh(j*2*pi*freq*tauR).^(denomord-p));
            end
            if (transtreat=='t')&(p<trno)
              tJ(:,p+1)=tanh(j*2*pi*freq*tauR).^(denomord-p);
            end
          end
        end
      end
      if (length(denomfixind)>0)
        fixvect=conj(exd).*(ywJ(:,denomfixind+1)*denomfix'); %fixvect was zero
      end
      if length(numfixind)>0,
        fixvect=fixvect-exd.*(xwJ(:,numfixind+1)*numfix');
      end
      J=[exd(:,ones(1,lnnf)).*xwJ(:,numnotfixind+1),...
          -conj(exd(:,ones(1,ldnf))).*ywJ(:,denomnotfixind+1)];
      if (transtreat=='t')&(trno>0), J=[J,exd(:,ones(1,trno)).*tJ]; end
      %if initset==initsetv(end), clear xwJ ywJ tJ, end
    end %domain~='pq'
    %
    vartf=ones(size(vary));
    %nonuniform weighting by variances or model:
    if any(initset=='wyvuaxir')
      vartf=vary;
      if strcmp(initset,'v')
        vartf=vary+varx.*abs(tfm.*tfm);
        if ~isempty(covxy)
          vartf=vartf-2*real(covxy.*conj(tfm));
        end
      elseif strcmp(initset,'u')
        vartf=vary./abs(tfm.*tfm)+varx;
        if ~isempty(covxy)
          vartf=vartf-2*real(covxy./tfm);
        end
      elseif strcmp(initset,'a')|strcmp(initset,'r') %AML
        %divide variance by transfer function where possible
        varm=vary./abs(tfm)+varx.*abs(tfm);
        if ~isempty(covxy)
          varm=varm-2*real(covxy.*exp(-j*angle(tfm)));
        end
        vartf=varm;
        %Set variances to infinity where tf is zero
        ind=find(abs(tfm)==0); %zero transfer function points
        if ~isempty(ind), infv=inf; varm(ind)=infv(ones(size(ind)),:); end
        %
        if strcmp(initset,'a')&any(domain=='spwr')
          ind=find( (2*pi*freq)==1 ); %Problem for omega==1
          if ~isempty(ind), freq(ind)=ones(size(ind)); end
          if any(domain=='spr')
            T=((2*pi*freq).^(numord+1)-1)./(2*pi*freq-1).*...
              ((2*pi*freq).^(denomord+1)-1)./(2*pi*freq-1);
          elseif strcmp(domain,'w')
            T=(sqrt(2*pi*freq).^(numord+1)-1)./(sqrt(2*pi*freq)-1).*...
              (sqrt(2*pi*freq).^(denomord+1)-1)./(sqrt(2*pi*freq)-1);
          end
          if ~isempty(ind)
            T(ind)=(numord+1)*(denomord+1)*ones(size(ind));
            freq(ind)=ones(size(ind))/(2*pi);
          end
          vartf=vartf.*T; %Se
        end
      elseif strcmp(initset,'i') %iqml
        %calculate denominator of Esm
        Nf=tfcalc(exppar(domainf,numf,1,delayf,fsf,Znum,1,[],[],[],[],[],coeffs,tauR),freq*fsf);
        Df=tfcalc(exppar(domainf,denomf,1,0,fsf,Zdenom,1,[],[],[],[],[],coeffs,tauR),freq*fsf);
        vartf=vary.*abs(Df).^2+varx.*abs(Nf).^2;
        if ~isempty(covxy), vartf=vartf-2*real(covxy.*conj(Nf).*Df); end
      end
      %
      if strcmp(initset,'x')
        if ~exist('initw'), error('Initial weight is not given'), end
        if min(size(initw))~=1, error('initw is not a vector'), end
        initw=initw(:);
        if any(imag(initw)), error('Complex element in initw'), end
        if length(initw)~=F, error('Length of initw is wrong'), end
        stdtf=1./abs(initw); %linear weight
      else
        %Keep linear weight for further calculations
        stdtf=sqrt(vartf);
      end
      indfin=find(finite(stdtf));
      ind=find(stdtf<10*eps*max([stdtf(indfin);eps]));
      if ~isempty(ind), stdtf(ind)=10*eps*max(stdtf)*ones(size(ind)); end
      if all(stdtf==0), stdtf=ones(size(stdtf)); end
    else
      stdtf=sqrt(vartf);
    end %nonuniform weighting
    %
    ind=find(isnan(stdtf)); if ~isempty(ind), stdtf(ind)=inf*ones(size(ind(:))); end
    w=1./stdtf; %prepare for export linear weight
    if any(domain=='pq') %orthopol
      if (basistreat=='n')|isempty(Znum)|(initset=='i') %determine initial basis
        [vnum,Znum]=orthopol(numord,domain,freq,uv./stdtf);
        [vdenom,Zdenom]=orthopol(denomord,domain,freq,y1m./stdtf);
        if transtreat=='t'
          [vnumt,Znumt]=orthopol(trord,domain,freq,ones(size(uv))./stdtf);
        end
      elseif (basistreat=='f')&(~exist('vnum')|isempty(vnum))
        %reconstruct only vnum and vdenom from Z's
        vnum=orthopol(numord,domain,freq,[],Znum);
        vdenom=orthopol(denomord,domain,freq,[],Zdenom);
        if transtreat=='t'
          vnumt=orthopol(trord,domain,freq,[],Znumt);
        end
      end
      J=[vnum.*uv(:,ones(1,numord+1)).*exd(:,ones(1,numord+1)),...
        -vdenom.*y1m(:,ones(1,denomord+1)).*conj(exd(:,ones(1,denomord+1)))];
      if transtreat=='t'
        J=[J,vnumt.*exd(:,ones(1,trord+1))];
      end
      fixvect=zeros(2*F,1);
    else %not orthopol
      Znum=[]; Zdenom=[]; Znumt=[]; %allow common export of data
      if ~strcmp(coeffs(1),'c') %real coeffs
        fixvect=[real(fixvect)./stdtf;imag(fixvect)./stdtf];
      else
        fixvect=fixvect./stdtf;
      end
    end
    %
    if ~strncmp(coeffs,'c',1) %real coeffs
      %Now make J real
      %J=[real(J);imag(J)];
      J=[real(J)./stdtf(:,ones(1,size(J,2)));...
          imag(J)./stdtf(:,ones(1,size(J,2)))];
    end
    %
    if strcmp(paramtreat,'a')
      if any(domain=='zq')
        J=[J(:,1:lnnf)+J(:,2*lnnf:-1:lnnf+1),J(:,2*lnnf+[1:trno])];
      else %s-domain and r-domain
        wm1=(-1).^[lnnf-1:-1:0]; wm1J=wm1(ones(size(J,1),1),:);
        J=[J(:,1:lnnf)+wm1J.*J(:,lnnf+1:2*lnnf),J(:,2*lnnf+[1:trno])];        
        %clear wm1J
      end
    end
    %
    %Calculate unknownvect and condition number
    if any(initset=='lw')
      unknownvect=J\fixvect;
      if itmax==0, condinum=cond(J); condinumJ=condinum; end
      Jdeficiency=NaN;
    elseif any(initset=='syvuaixr')
      [U,S,V]=svd(J,0);
      invS=abs(S);
      diaginvS=diag(invS);
      maxeig=max(diaginvS);
      [mineig,ind]=min(diaginvS);
      if nfn0==0 %all parameters free
        diaginvS(ind(length(ind)))=[]; mineig=min(diaginvS);
      end
      if mineig==0, disp('WARNING: J is infinitely badly conditioned')
        if maxeig==0, condinum=NaN; else condinum=inf; end
      else
        condinum=maxeig/mineig;
      end
      condinumJ=condinum;
      %clear diaginvS
      eigwarn=0; Jdeficiency=0;
      for kind=1:length(invS(1,:)),
        if invS(kind,kind)<maxeig*eps*length(invS(1,:)) %original
          %if invS(kind,kind)<maxeig*eps %Suggestion of Gyula
          invS(kind,kind)=0; %0 if eigenvalue is 0
          if (kind+sign(nfn0)<max(size(invS)))&(eigwarn==0)
            eigwarn=1;
            Jdeficiency=max(size(invS))-kind-sign(nfn0);
            fprintf(['WARNING: Rank of J is deficient:',...
                ' svd has found %.0f small eigenvalue(s)\n'],...
              max(size(invS))-kind-sign(nfn0))
          end
        else
          %proper reciprocal can be calculated
          invS(kind,kind)=1/S(kind,kind);
        end
      end %for kind
      if nfn0==0
        unknownvect=V(:,end);
        %
        %Make zeros true zeros
        %ind=find(abs(unknownvect)<median(abs(unknownvect))*eps);
        %if isempty(ind), unknownvect(ind)=0*ind; end
        %
        unknownvect=unknownvect/norm(unknownvect);
      else
        unknownvect=V*invS'*U'*fixvect;
      end
      %
      %clear U S V invS kind
      %clear maxeig mineig ind
    end
    %unknownvect calculated
    %
    if any(domain=='pq')
      num=unknownvect(1:numord+1)';
      denom=unknownvect(numord+1+[1:denomord+1])';
      %nums=num*pnumarr; denoms=denom*pdenomarr; %conventional polynomials
      %
    else %regular case
      num(numfixind+1)=numfix;
      num(numnotfixind+1)=unknownvect(1:numord+1-length(numfix));
      if strcmp(paramtreat,'a')
        if domain=='z'
          denom=num(numord+1:-1:1);
        else %s-domain and r-domain
          wm1=(-1).^[lnnf-1:-1:0];
          denom=wm1.*num;
        end
      else
        denom(denomfixind+1)=denomfix;
        denom(denomnotfixind+1)=...
          unknownvect(numord+1-length(numfix)+1:length(unknownvect)-trno);
      end
    end
    if transtreat=='t'
      trnum(1:length(trnum))=unknownvect(length(unknownvect)+[-trno+1:0]);
    end
  end
  %
  if exist('unknownvect'), clear unknownvect, end
  if (~strncmp(coeffs,'c',1)&any(imag([num,denom])))|...
      any(imag(delay))|any(~finite([num,denom,delay]))
    warning('Complex, inf or NaN value of estimated parameter')
    disp('Possible cause: badly conditioned normal equations')
    disp('All estimated parameters are set to NaN by elis')
    denom(denomnotfixind+1)=0*denom(denomnotfixind+1)+NaN;
    num(numnotfixind+1)=0*num(numnotfixind+1)+NaN;
    if delaytreat=='v', delay=NaN; end
    ndval=NaN;
  end
  %---------------------------------------------
  if calldrawnow, drawnow, end
  rnum=[]; rdenom=[];
  if ~isnan(ndval)
    pzt0=clock; %begin time of pole/zero calculation
    if strcmp(calcrnum,'c')
      if ~any(domain=='pq')&all(finite(num)), rnum=roots(num).'; ngain=[];
      elseif any(domain=='pq')&all(finite(num))
        [rnum,ngain]=ortroots(num,Znum); rnum=rnum.';
      end
    end
    if strcmp(calcrdenom,'c')
      if ~any(domain=='pq')&all(finite(denom)), rdenom=roots(denom).'; dgain=[];
      elseif any(domain=='pq')&all(finite(denom))
        [rdenom,dgain]=ortroots(denom,Zdenom); rdenom=rdenom.';
      end
    end
    if any(domain=='zq'),
      unstab=find(abs(rdenom)>=stablimit);
      nmph=find(abs(rnum)>=stablimit);
    elseif any(domain=='sp'),
      unstab=find(real(pzsc*rdenom)>=stablimit);
      nmph=find(real(pzsc*rnum)>=stablimit);
    elseif strcmp(domain,'w'),
      unstab=find(real((pzsc*rdenom).^2)>=stablimit);
      nmph=find(real((pzsc*rnum).^2)>=stablimit);
    elseif strcmp(domain,'r'),
      unstab=find(real(atanh(rdenom)/tauR*fs)>=stablimit);
      nmph=find(real(atanh(rnum)/tauR*fs)>=stablimit);
    end
    if (length(unstab)>0) & (any(stabmode=='rc')|((cycle==0)&strcmp(stabmode,'l')))
      %stabilization
      [rdenom,denom,delay,num,unstabstays]=...
        stabmph(domain,denom,num,delay,fs,Zdenom,Znum,stdtf,rdenom,dgain,...
        pzsc,unstab,stabmode,stablimit,freq,delaytreat,delayscale,...
        numfix,numfixind,denomfix,denomfixind,paramtreat,'stab',coeffs,tauR);
      if ~any(dispmesg=='ncs')
        fprintf('   %.0f unstable pole(s) stabilized by ',length(unstab))
        if any(stabmode=='rl'), fprintf('reflection\n')
        elseif strcmp(stabmode,'c'), fprintf('contraction\n')
        end
      end
      unstab=unstabstays;
    end
    if (length(nmph)>0) & (any(mphmode=='rc')|((cycle==0)&strcmp(mphmode,'l')))
      %minimum phase
      [rnum,num,delay,denom,nmphstays]=...
        stabmph(domain,num,denom,delay,fs,Znum,Zdenom,stdtf,rnum,ngain,...
        pzsc,nmph,mphmode,stablimit,freq,delaytreat,delayscale,denomfix,...
        denomfixind,numfix,numfixind,paramtreat,'nmph',coeffs,tauR);
      if ~any(dispmesg=='ncs')
        fprintf('   %.0f nonminimum phase zero(s) adjusted by ',length(nmph))
        if any(mphmode=='rl'), fprintf('reflection\n')
        elseif strcmp(mphmode,'c'), fprintf('contraction\n')
        end
      end
      nmph=nmphstays;
    end
    if calldrawnow, drawnow, end
    if (length(unstab)>0)&~any(dispmesg=='ncs')%&(length(initsetv)==1)
      fprintf(['WARNING: cycle %.0f gives %.0f unstable pole(s)',...
        ' (stablimit=%.5g):\n'],cycle,length(unstab),stablimit)
      for i=1:length(unstab)
        if imag(rdenom(unstab(i)))~=0
          unstxt=sprintf('   %+6.4g %+6.4g*j',...
            real(pzsc*rdenom(unstab(i))),imag(pzsc*rdenom(unstab(i))) );
          fprintf([unstxt,setstr(32*ones(1,19-length(unstxt)))])
          if any(domain=='zq')
            fprintf('   |z%.0f|=%6.4g\n',i,pzsc*abs(rdenom(unstab(i))))
          elseif strcmp(domain,'w')
            fprintf('   w%.0f^2=%+6.4g %+6.4g*j\n',i,...
              real((pzsc*rdenom(unstab(i)))^2),...
              imag((pzsc*rdenom(unstab(i)))^2))
          elseif strcmp(domain,'r')
            fprintf('   atanh(r%.0f)/tauR=%+6.4g %+6.4g*j\n',i,...
              real(atanh(unstab(i))/tauR*fs),...
              imag(atanh(unstab(i))/tauR*fs))
          else fprintf('\n')
          end
        else %real unstable pole
          unstxt=sprintf('   %+6.4g',real(pzsc*rdenom(unstab(i))) );
          fprintf([unstxt,setstr(32*ones(1,19-length(unstxt)))])
          if any(domain=='zq'), fprintf('   |z%.0f|=%6.4g\n',i,...
            pzsc*abs(rdenom(unstab(i))))
          elseif strcmp(domain,'w')
            fprintf('   w%.0f^2=%+6.4g\n',i,(pzsc*rdenom(unstab(i)))^2)
          elseif strcmp(domain,'r')
            fprintf('   atanh(r%.0f)/tauR=%+6.4g\n',i,...
              (atanh(unstab(i)))/tauR*fs)
          else fprintf('\n')
          end
        end %if imag(...
      end %for i
    end %if length(unstab)
    pztime=pztime+etime(clock,pzt0); %collect pz calculation time
  end
  if (length(initsetv)>1)&~isnan(ndval)
    if domain=='z', variable='z^-1'; else variable=domain; end
    pvi=fidmodel(variable,num,denom,delay,fs,[],[],...
      trnum,Znum,Zdenom,Znumt,'Coefficients',coeffs,'tauR',tauR,'noconsistency');
    if ~exist('Fdatcf')
      if isa(Fdat,'fiddata')
        Fdatcf=Fdat;
      else
        Fdatcf=fiddata(y1m,uv,freq*fs,vary*2,varx*2,covxy*2,'noconsistency');
      end
    end
    if isempty(Fdatcf.SisoVariance)
      set(Fdatcf,'SisoVariance',[varx*2,vary*2,covxy*2],'noconsistency')
    end
    set(pvi,'data',Fdatcf,'noconsistency');
    set(pvi,'algorithm',struct('initset',issset(initset,domain)),'noconsistency')
    cfi=eliscost(pvi);
    if ~any(dispmesg=='nc')
      fprintf('     Cost function: %.5g',cfi)
    end
    if cfi<cf
      if iis>1, if ~any(dispmesg=='nc'), fprintf(', better than %.5g',cf), end, end
      cf=cfi;
      if initset~=initsetv(end)
        pvsave.initset=initset;
        pvsave.num=num; pvsave.denom=denom; pvsave.trnum=trnum;
        pvsave.Jdeficiency=Jdeficiency;
        pvsave.condinum=condinum; pvsave.condinumJ=condinumJ;
        pvsave.rnum=rnum; pvsave.rdenom=rdenom;
        pvsave.unstab=unstab; psave.nmph=nmph;
        pvsave.pvi=pvi;
      end
    elseif initset==initsetv(end)
      %this was the last cycle, and this was not the best:
      %restore the best one
      initset=pvsave.initset;
      num=pvsave.num; denom=pvsave.denom; trnum=pvsave.trnum;
      Jdeficiency=pvsave.Jdeficiency;
      condinum=pvsave.condinum; condinumJ=pvsave.condinumJ;
      rnum=pvsave.rnum; rdenom=pvsave.rdenom;
      unstab=pvsave.unstab; nmph=psave.nmph;
    end
    if ~any(dispmesg=='nc'), disp(' '), end
    if (iis<length(initsetv))&(initsetv(iis+1)=='i')
      pvi=pvsave.pvi; %prepare for iqml
      %pvi.algorithm
      [domainf,numf,denomf,delayf,fsf,Znum,Zdenom,comments,fdate,trnumf,Znumt]=...
         imppar(pvi,fs);
    end
  end %initset='t'
end %for initset
if length(initsetv)>1
  if initset=='i', istxt=['iqml, from ',issset(pvsave.initset,domain)];
  else istxt=issset(initset,domain);
  end
  if ~any(dispmesg=='nc')
    disp(['Selected setting: ''',istxt,''''])
  end
end
%---------------------------------------------
%---------------------------------------------
%3. MAIN PROGRAM
%---------------------------------------------
%First some settings ...
%initialization of arrays to follow movement of poles and zeros:
polesarray=rdenom; zerosarray=rnum;
tc0=clock;
plotinst=clock; plotinst(4)=plotinst(4)-1; %early time
if isnan(plotdenstime), plotdenstime=0; end
cycleend=0;
termcond=0; %code of cause of termination
if calldrawnow, drawnow, end
plotdone='n'; %no plot is done yet
Cpreq=0; CRreq=0;
itcdone=0; %iterctrl menu is put up
if calculateCR==0 %do not calculate
elseif ~isempty(cfile)|~isempty(rfile)|(nargoutelis>=1)|isa(Fdat,'fiddata')
  CRreq=1;
end
if ~isempty(M)&finite(M)
  if M<=2, Mm=NaN;
  elseif finite(M), Mm=(M-1)/(M-2);
  end
else
  Mm=1;
end
helis=NaN; %elis plot handle
hamp=NaN;
cl=1;
if N>0
  %Normal distribution 95%: 1.645, 97.5%: 1.96
  meanch=Mm*N;
  chi05pr=Mm*N*(1-2/9/N-1.645*sqrt(2/9/N))^3;
  chi95pr=Mm*N*(1-2/9/N+1.645*sqrt(2/9/N))^3;
  if ~isempty(M)&finite(M)
    chi05pr=meanch+sqrt((M-1)/(M-3))*(chi05pr-meanch);
    chi95pr=meanch+sqrt((M-1)/(M-3))*(chi95pr-meanch);
  end
else
  chi05pr=0; chi95pr=0;
end
%---------------------------------------------
if (itmax>0)|Cpreq|CRreq
  dndp=zeros(F,0);
  if ~any(domain=='pq')
    if lnnf>0
      dndp(F,lnnf)=complmin; %preallocate
    end
    pcyc=0;
    for p=numnotfixind(:)'
      pcyc=pcyc+1;
      if any(domain=='zq'),
        dndp(:,pcyc)=exp(-p*j*2*pi*freq);
      elseif any(domain=='s'),
        dndp(:,pcyc)=(j*2*pi*freq).^(numord-p);
      elseif strcmp(domain,'w'),
        dndp(:,pcyc)=sqrt(j*2*pi*freq).^(numord-p);
      elseif strcmp(domain,'r'),
        dndp(:,pcyc)=tanh(j*2*pi*freq*tauR).^(numord-p);
      end
    end %for p
  else %orthopol
    dndp=vnum;
  end
  rn=ones(1,lnnf);
  dddp=zeros(F,0);
  if ~any(domain=='pq')
    if ldnf>0
      dddp(F,ldnf)=complmin; %preallocate
    end
    pcyc=0;
    for p=denomnotfixind(:)'
      pcyc=pcyc+1;
      if any(domain=='zq'),
        dddp(:,pcyc)=exp(-p*j*2*pi*freq);
      elseif any(domain=='s'),
        dddp(:,pcyc)=(j*2*pi*freq).^(denomord-p);
      elseif strcmp(domain,'w'),
        dddp(:,pcyc)=sqrt(j*2*pi*freq).^(denomord-p);
      elseif strcmp(domain,'r'),
        dddp(:,pcyc)=tanh(j*2*pi*freq*tauR).^(denomord-p);
      end
    end
  else %orthopol
    dddp=vdenom;
  end
  rd=ones(1,ldnf);
  %
  if transtreat=='t'
    dntdp=zeros(F,0);
    if ~any(domain=='pq')
      if trno>0, dntdp(F,trno)=complmin; end %preallocate
      pcyc=0;
      for p=0:trord
        pcyc=pcyc+1;
        if any(domain=='zq'),
          dntdp(:,pcyc)=exp(-p*j*2*pi*freq);
        elseif any(domain=='s'),
          dntdp(:,pcyc)=(j*2*pi*freq).^(trord-p);
        elseif strcmp(domain,'w'),
          dntdp(:,pcyc)=sqrt(j*2*pi*freq).^(trord-p);
        elseif strcmp(domain,'w'),
          dntdp(:,pcyc)=tanh(j*2*pi*freq*tauR).^(trord-p);
        end
      end %for p
    else %orthopol
      dntdp=vnumt;
    end
  else
    dntdp=[];
  end %transtreat
  rnt=ones(1,trno);
  %
end %itmax>0
if dispmesg=='c', msgsall=''; end
Cpint=[];
CRint=[]; %temporary storage of Cramer-Rao bound
messnoC=0; %counter of too small eigenvalues of d^2C/dp^2
plotno=0; cycnoplot=0; costfold=inf;
%---------------------------------------------
for cycle=0:itmax+1 %main loop, last cycle for covariance calculation only
  if calldrawnow, drawnow, end
  %consecutive parameter calculation cycle
  if cycleend==1, calcpar=0; else calcpar=1; end
  if (cycleend==1)&~any(dispmesg=='nc')
    if cycle<=1, lasttxt='A'; else lasttxt='A last'; end
    ctxt='covariances';
    msgsall=[msgsall,lasttxt,' cycle follows for calculation of the ',...
        ctxt,sprintf('\n')];
    if (dispmesg=='y'), fprintf(msgsall), msgsall=''; end
  end
  %if (Cpreq==1) & (cycleend==1)
  %  d2ExEreq=1; %calculate second derivative
  %else
    d2ExEreq=0; %don't calculate
  %end
  if cycle==0
    if delaytreat=='v', fixtxt=[fixtxt,', delay free'];
    else fixtxt=[fixtxt,', delay fixed'];
    end
    if (transtreat=='t')&(trno>0)
      fixtxt=[fixtxt,sprintf(' + transients (%.0f/%.0f)',trno-1,denomord)];
    end
  end
  %
  if (cycle==1)&(itmax>0) %messages in first iteration cycle
    clv=clock;
    if ~any(dispmesg=='ns')
      msgsall=[msgsall,sprintf('------------------\n'),...
          sprintf('Time: %.0f:%02.0f:%02.0f\n',clv(4),clv(5),clv(6)),...
          '   Beginning the iteration algorithm: ',algtype];
      if strcmp(searchmode,'l')
        msgsall=[msgsall,sprintf(', with line search\n')];
      else
        msgsall=[msgsall,sprintf('\n')];
      end
      if any(domain=='pq')
        msgsall=[msgsall,sprintf('   using orthogonal polynomials\n')];
      end
      msgsall=[msgsall,sprintf(['   ',fixtxt,'\n'])];
      if ~isempty(covxy)
        msgsall=[msgsall,sprintf(...
            '   Input-output covariances taken into account\n')];
        if ~any(covxy~=0)
          msgsall=[msgsall,sprintf('   although all with zero values\n')];
        end
      end
      if (rcostvar<100*eps)&(rcostvar~=0),
        msgsall=[msgsall,...
            sprintf(['   Warning! relvar=%.3e<%.3e=100*eps is close to ',...
              'roundoff errors\n'],rcostvar,100*eps),...
            sprintf('   This stop criterion will probably be ineffective\n')];
      end
      msgsall=[msgsall,'------------------\n'];
      if any(dispmesg=='by'), fprintf(msgsall), msgsall='';
      elseif any(dispmesg=='cs'), msgsall='';
      end
    end %dispmesg
  end %if cycle==1
  %
  if ~any(domain=='pq')
    Nf=tfcalc(exppar(domain,num,1,0,1,[],[],[],[],[],[],[],coeffs,tauR),freq);
    Df=tfcalc(exppar(domain,denom,1,0,1,[],[],[],[],[],[],[],coeffs,tauR),freq);
    if transtreat=='t', Nft=tfcalc(exppar(domain,trnum,1,...
        [],1,[],[],[],[],[],[],[],coeffs,tauR),freq);
    end
  else %orthopol
    Nf=vnum*num'; Df=vdenom*denom';
    if transtreat=='t', Nft=vnumt*trnum'; end
  end
  if (cycle>0)
    if ~any(dispmesg=='n')&(calcpar==1)
      msgsall=[msgsall,sprintf('Iteration cycle: %.0f\n',cycle)];
      if any(dispmesg=='y'), fprintf(msgsall), msgsall=''; end
    end
    %p=p0-inv(J*J')*J'*Esmr=p0+dpnf
    %J=-dEsmr/dp
    %dpnf is the solution of J*dp=-Esmr
    %
    if strcmp(delaytreat,'v') %updates
      exd=exp(-j*pi*freq*delay*delayscale);
      uvexd=uv.*exd; y1mcexd=y1m.*conj(exd);
    end
    if (cycle==1)|strcmp(delaytreat,'v') %update if delay changes
      %derivative of numerator, real-imaginary representation
      daridp=...
        [real(uvexd(:,rn).*dndp),...
          -real(y1mcexd(:,rd).*dddp);...
          imag(uvexd(:,rn).*dndp),...
          -imag(y1mcexd(:,rd).*dddp)];
      if transtreat=='t'
        daridp=[daridp,[real(exd(:,ones(1,trno)).*dntdp);
            imag(exd(:,ones(1,trno)).*dntdp)]];
      end
    end %if cycle==1
    %
    Esm=uvexd.*Nf-y1mcexd.*Df; %This is yet the numerator only
    if transtreat=='t'
      Esm=Esm+exd.*Nft;
    end
    Esmrnum=[real(Esm);imag(Esm)];
    Esmdenom2=varx.*abs(Nf).^2+vary.*abs(Df).^2;
    if ~isempty(covxy)
      Esmdenom2=Esmdenom2-2*real(covxy.*(conj(exd).^2).*conj(Nf).*Df);
    end
    if cycleend==0
      if strcmp(call,'new'), w=sqrt(Esmdenom2);
      elseif nargout>6, w=sqrt(Esmdenom2);
      end
    end
    Esm=Esm./sqrt(Esmdenom2); %Now Esm is ready
    Esmrdenom=sqrt([Esmdenom2;Esmdenom2]);
    Esmr=Esmrnum./Esmrdenom; %Now Esmr is ready: Esmr=a/sqrt(b)
    %
    %d/dp(a/sqrt(b))=((da/dp)-db/dp*a/2/b)/sqrt(b)
    dbdp=[2*varx(:,rn).*real(dndp.*conj(Nf(:,rn))),...
          2*vary(:,rd).*real(dddp.*conj(Df(:,rd)))];
    if ~isempty(covxy)
      dbdp=dbdp-2*real([((covxy.*(conj(exd).^2).*Df)*rn).*conj(dndp),...
          ((covxy.*(conj(exd).^2).*conj(Nf))*rd).*dddp]);
    end
    if transtreat=='t', dbdp=[dbdp,zeros(F,trno)]; end
    J=-[dbdp;dbdp];
    %J=J.*(((Esmrnum/2)./Esmrdenom.^3)*[rn,rd,rnt]);
    J=J.*(((Esmrnum/2)./Esmrdenom./Esmrdenom./Esmrdenom)*[rn,rd,rnt]);
    J=J+daridp./Esmrdenom(:,[rn,rd,rnt]);
    if paramtreat=='a' %allpass
      if domain=='z'
        J=[J(:,1:lnnf)+J(:,2*lnnf:-1:lnnf+1),J(:,2*lnnf+1:size(J,2))];
      else %s or r
        wm1=(-1).^[lnnf-1:-1:0]; wm1J=wm1(ones(size(J,1),1),:);
        J=[J(:,1:lnnf)+wm1J.*J(:,lnnf+1:2*lnnf),J(:,2*lnnf+1:size(J,2))];        
      end
    end
    if delaytreat=='v', %variable delay
      %derivative of real and imaginary parts
      %Esmnum=exd.*Nf.*Xm-conj(exd).*Df.*Ym
      %exd=exp(-j*pi*freq*delayscale*delay)
      dEdd=-j*pi*delayscale*freq.*uvexd.*Nf...
        -j*pi*delayscale*freq.*y1mcexd.*Df;
      if transtreat=='t'
        dEdd=dEdd-j*pi*delayscale*freq.*exd.*Nft;
      end
      dEddr=[real(dEdd);imag(dEdd)];
      if ~isempty(covxy)
        %Esmdenom2=Esmdenom2-2*real(covxy.*(conj(exd).^2).*conj(Nf).*Df);
        dbdd=-2*real(covxy.*(j*2*pi*delayscale*freq)...
          .*(conj(exd).^2).*conj(Nf).*Df);
        dEddr=dEddr-[dbdd;dbdd].*(Esmrnum/2)./Esmrdenom.^2;
      else
        dbdd=zeros(F,1);
      end
      J=[J,dEddr./Esmrdenom];
    else
      dEddr=[]; dbdd=[];
    end
    if strcmpi(algtype,'nr')|(d2ExEreq==1) %Newton-Raphson or Cp matrix req
      %calculate 2nd term of d^2C/dp^2
      %
      %d^2/dp^2(En/sqrt(Ed)) =
      % + (d^2En/dp1dp2)/sqrt(Ed)
      % - (dEn/dp1*dEd/dp2) / (2Ed*sqrt(Ed))
      % - (dEn/dp2*dEd/dp1) / (2Ed*sqrt(Ed))
      % - d^2Ed/dp1dp2 * (En/(2Ed*sqrt(Ed)))
      % + dEd/dp1*dEd/dp2 * (3*En/(4*Ed^2*sqrt(Ed)))
      %
      %Esmnum=exd.*Nf.*Xm-conj(exd).*Df.*Ym
      %Esmrdenom=sqrt(sy^2.*abs(Df)^2+sx^2.*abs(Nf)^2
      %     -2real(cxy.*conj(exd).*conj(Nf).*conj(exd)*.Df)
      %exd=exp(-j*pi*freq*delayscale*delay)
      %Derivatives are calculated for variable parameters only
      %
      d2ExE=zeros(Nfpm,Nfpm);
      %
      %term 1 is zero for fixed delay (linear in coefficients):
      % + (d^2En/dp1dp2)/sqrt(Ed)
      if strcmp(delaytreat,'v')
        %Derivatives of terms in numerator of Esm, multiplied by conj(Esm)
        dndd1=-j*pi*delayscale*freq.*uvexd./Esmrdenom(1:F).*conj(Esm);
        dndd2=-j*pi*delayscale*freq.*y1mcexd./Esmrdenom(1:F).*conj(Esm);
        if transtreat=='t'
          dndd3=-j*pi*delayscale*freq.*exd./Esmrdenom(1:F).*conj(Esm);
        end
        %Mixed (variable-parameter) terms
        d2ExEv=[real(2*dndd1.'*dndp),real(2*dndd2.'*dddp)];
        if transtreat=='t'
          d2ExEv=[d2ExEv,real(2*dndd3.'*dntdp)];
        end
        %Derivative by d^2
        d2ExEd=real(((+((j*pi*delayscale*freq).^2).*Nf.*uvexd...
          -((j*pi*delayscale*freq).^2).*Df.*y1mcexd...
          +((j*pi*delayscale*freq).^2).*Nft.*exd...
          )./Esmrdenom(1:F)).'*conj(Esm));
        d2ExE(Nfpm,1:lnnf+ldnf+trno)=d2ExEv;
        d2ExE(1:lnnf+ldnf+trno,Nfpm)=d2ExEv.';
        d2ExE(Nfpm,Nfpm)=d2ExEd;
      end %delaytreat
      %
      %terms 2-3:
      % - (dEn/dp1*dEd/dp2) / (2Ed*sqrt(Ed))
      % - (dEn/dp2*dEd/dp1) / (2Ed*sqrt(Ed))
      multerm=Esmr./(2*Esmrdenom.^3); %include Esmr: multiplicative term
      term2=[daridp,dEddr]'*([dbdp,dbdd;dbdp,dbdd].*multerm(:,[rn,rd,rnt,rv]));
      d2ExE=d2ExE-(term2+term2');
      %
      %term 4:
      % - d^2Ed/dp1dp2 * (En/(2Ed*sqrt(Ed)))
      %second term:
      multerm=conj(Esm).*Esm./(2*Esmdenom2);
      dbdp2m=zeros(Nfpm,Nfpm);
      dbdp2m(1:lnnf+ldnf,1:lnnf+ldnf)=...
        real([2*dndp'*(dndp.*varx(:,rn).*multerm(:,rn)),...
          zeros(lnnf,ldnf);...
          zeros(ldnf,lnnf),2*dddp'*(dddp.*vary(:,rd).*multerm(:,rd))]);
      dbdp2m=(dbdp2m+dbdp2m')/2;
      if ~isempty(covxy)
        %New multerm
        multerm=multerm.*covxy.*conj(exd).^2;
        db2part=-2*real(2*dndp.'*(dddp.*multerm(:,rd).^2));
        if ~isempty(db2part)
          dbdp2m(1:lnnf,lnnf+[1:ldnf])=db2part;
          dbdp2m(lnnf+[1:ldnf],1:lnnf)=db2part';
        end
      end %covxy
      if strcmp(delaytreat,'v')
        if ~isempty(covxy)
          multerm=multerm.*(j*2*pi*delayscale*freq);
          db2ext=-2*real(2*multerm.'*[conj(dndp),dddp,zeros(size(dntdp))]);
          multerm=multerm.*(j*2*pi*delayscale*freq);
          dbddel=-2*real(multerm.'*(conj(Nf).*Df));
          dbdp2ms=size(dbdp2m,1);
          dbdp2m(:,dbdp2ms)=[db2ext';dbddel];
          dbdp2m(dbdp2ms,:)=[db2ext,dbddel];
        end %covxy
      end %delaytreat
      d2ExE=d2ExE-dbdp2m;
      %
      %term 5:
      % + dEd/dp1*dEd/dp2 * (3*En/(4*Ed^2*sqrt(Ed)))
      multerm=3*Esmr.*Esmrnum./(4*Esmrdenom.^5);
      d2ExE=d2ExE+[dbdp,dbdd;dbdp,dbdd]'...
        *([dbdp,dbdd;dbdp,dbdd].*multerm(:,[rn,rd,rnt,rv]));
      %
      %Final housekeeping
      if any(any(imag(d2ExE))), disp('WARNING: imaginary d2ExE in elis'), end
      if strcmp(paramtreat,'a') %reduce size of d2ExE for allpass
        if domain=='z'
          d2ExE(1:lnnf,1:2*lnnf)=d2ExE(1:lnnf,1:2*lnnf)...
            +d2ExE(2*lnnf:-1:lnnf+1,1:2*lnnf);
          d2ExE(1:2*lnnf,1:lnnf)=d2ExE(1:2*lnnf,1:lnnf)...
            +d2ExE(1:2*lnnf,2*lnnf:-1:lnnf+1);
          d2ExE(lnnf+1:2*lnnf,:)=[]; d2ExE(:,lnnf+1:2*lnnf)=[];
        else %s-domain
          %This needs fixing ***
          error('Newton-Raphson with s-domain allpass not yet ready')
        end
      end %allpass
      if any(any(d2ExE-d2ExE'))
        if any(any((abs(d2ExE-d2ExE')./d2ExE) > length(d2ExE)*eps))
          disp('WARNING! d2ExE (second derivative) is not symmetric')
        end
        d2ExE=(d2ExE+d2ExE')/2;
      end
    else %neither Newton-Raphson, nor Cp required
      d2ExE=[]; %empty matrix may be needed
    end %calculate 2nd term of d^2C/dp^2
    %
    if calldrawnow, drawnow, end
    if strcmpi(algtype,'ng')
      dpnf=-J\Esmr;
      condinum=cond(J); condinumJ=condinum;
      Jdeficiency=NaN;
    elseif strcmpi(algtype,'nr')
      if ~isempty(d2ExE)
        dpnf=-(J'*J+d2ExE)\(J'*Esmr);
        condinum=cond(J'*J+d2ExE); condinumJ=cond(J);
        Jdeficiency=NaN;
      else %true Newton-Gauss instead of Newton-Raphson
        error('Newton-Raphson has run into a software bug')
      end
    elseif strcmpi(algtype,'lm')
      if lambda==0
        dpnf=-J\Esmr;
        condinum=cond(J); condinumJ=condinum;
      else %lambda>0
        JTJ=J'*J; lJTJ=length(JTJ);
        dpnf=-( JTJ+lambda*100*eps*norm(J,'fro')^2*eye(lJTJ,lJTJ) )\(J'*Esmr);
        condinum=cond(JTJ+lambda*100*eps*norm(J,'fro')^2*eye(lJTJ,lJTJ));
        condnumJ=cond(J);
      end
      Jdeficiency=NaN;
    elseif strcmpi(algtype,'svd')|strcmpi(algtype,'lmsvd')
      [Uelis,S,Velis]=svd(J,0);
      invS=abs(S);
      diaginvS=diag(invS);
      maxeig=max(diag(invS));
      [mineig,ind]=min(diaginvS);
      if nfn0==0 %all parameters free, or only zero parameters fixed
        diaginvS(ind(1))=[]; mineig=min(diaginvS);
      end
      if mineig==0, disp('WARNING: J is infinitely badly conditioned')
        if maxeig==0, condinum=NaN; else condinum=inf; end
      else
        condinum=maxeig/mineig;
      end
      condinumJ=condinum;
      if strcmpi(algtype,'lmsvd'), froJ2e=100*eps*norm(J,'fro')^2;
      else froJ2e=0; lambda=0;
      end
      eigwarn=0; Jdeficiency=0;
      for kind=1:length(invS(1,:))
        sden=S(kind,kind)^2+lambda*froJ2e;
        if invS(kind,kind)*maxeig*eps*length(invS(1,:)) >= sden %bad conditioning
          invS(kind,kind)=0; %0 if eigenvalue is 0
          if (kind+sign(nfn0)<max(size(invS)))&(eigwarn==0)
            eigwarn=1;
            Jdeficiency=max(size(invS))-kind-sign(nfn0);
            msgsall=[msgsall,sprintf(['WARNING: Rank of J is deficient:',...
                  ' svd has found %.0f small eigenvalue(s)\n'],...
                max(size(invS))-kind-sign(nfn0))];
          end
        elseif (kind<length(invS(1,:)))|(lambda>0)|(nfn0>0)
          %proper singular value
          invS(kind,kind)=S(kind,kind)/sden;
        else %The last singular value is too large despite undeterminedness
          msgsall=[msgsall,sprintf(['WARNING! The condition number is ',...
                'smaller than 1/eps/freepar\n  for svd or lmsvd:',...
                ' %.3e < %.3e\n'],...
              maxeig/invS(kind,kind),1/eps/length(invS(1,:)))];
          fprintf(['  The reciprocal of the smallest singular value is set',...
              ' to zero\n'])
          invS(kind,kind)=0;
        end
      end %for kind
      %
      %First line for looking after svd/multiplication error
      if any(any(isnan(Velis)))|any(any(isnan(invS)))|any(any(isnan(Uelis)))|any(isnan(Esmr))
        error('NaN is found in Uelis or invS or Velis or Esmr')
      end
      if ~strncmp(version,'7.0.4',5)
        dpnf=-Velis*invS'*Uelis'*Esmr;
      else %7.0.4
        %disp('Evaluating products one by one ...')
        dpnf=Esmr; dpnf=Uelis'*dpnf; dpnf=invS'*dpnf; dpnf=-Velis*dpnf;
      end
      if any(isnan(dpnf)) %eliminate strange error, cannot localize cause
        if strncmp(version,'7.0.4',5)
          disp('Evaluating product in a serarate workspace ...')
          dpnf=matproduct(-Velis,invS',Uelis',Esmr);
        end
      end
      if any(isnan(dpnf))
        dbs=dbstack;
        Vmax=max(max(Velis)), maxS=max(max(invS)),
        maxU=max(max(Uelis)), maxE=max(Esmr),
        cl=clock; assignin('base','cl',cl), 
        try, evalin('base','elapsed_time=sprintf(''%.1f min'',etime(cl,cl0)/60)'), catch, end
        commands='dbtype elis 2821:2846';
        save(['eliserror-',date,sprintf('_%.0f-%.0f',cl(4),cl(5)),'.mat'])
        error('dpnf contains NaN')
      end %last line
      %
      if nfn0==0 %no fixed nonzero parameter
        if ~strcmp(paramtreat,'a') %not allpass
          dpnfm=zeros(numord+denomord+2+trno,1);
          nfixpind=[numnotfixind+1,denomnotfixind+numord+2,...
              numord+denomord+2+[1:trno]];
        else
          dpnfm=zeros(numord+1+trno,1);
          nfixpind=[numnotfixind+1,numord+1+[1:trno]];
        end
        if delaytreat=='v', dpnfm(nfixpind)=dpnf(1:length(dpnf)-1);
        else dpnfm(nfixpind)=dpnf;
        end
        if ~strcmp(paramtreat,'a') %not allpass
          paramv=[num,denom,trnum]';
          if any(abs(dpnfm)>1e-6) %sufficient modification
            dpnfm=(paramv+dpnfm)/norm(paramv+dpnfm)-paramv;
          end
        else %allpass
          paramv=[num,trnum]';
          if any(abs(dpnfm)>1e-6) %sufficient modification
            dpnfm=(paramv+dpnfm)/norm(paramv+dpnfm)-paramv;
          end
        end
        dpnf(1:length(nfixpind))=dpnfm(nfixpind);
      end %nfn0
    end %algtype
    %
    if calldrawnow, drawnow, end
    %
    if delaytreat=='v',
      ddelay=dpnf(length(dpnf));
      dpnf(length(dpnf))=[];
    else
      ddelay=0;
    end
    dparam=zeros(1,numord+denomord+2+trno);
    dparam(numnotfixind+1)=dpnf(1:numord+1-length(numfix));
    if strcmp(searchmode,'l') %prepare for line search
      num0=num; denom0=denom; trnum0=trnum; delay0=delay;
    end
    dnum=dparam(1:numord+1);
    if paramtreat~='a' %not allpass
      dparam(numord+1+denomnotfixind+1)=...
        dpnf(numord+1-length(numfix)+[1:length(denomnotfixind)]);
    else %allpass
      if domain=='z'
        dparam(numord+1+denomnotfixind+1)=dpnf(numord+1-length(numfix):-1:1);
      else %'s' or 'r'
        dparam(numord+1+denomnotfixind+1)=...
          ((-1).^(denomord-denomnotfixind')).*dpnf(1:numord+1-length(numfix));        
      end
    end
    ddenom=dparam(numord+1+[1:length(denom)]);
    if transtreat=='t'
      dparam(numord+denomord+2+[1:trno])=...
        dpnf(length(dpnf)+[-trno+1:0]);
      dtrnum=dparam(length(dparam)+[-trno+1:0]);
    end
    num=num+dnum; denom=denom+ddenom;
    if any(isnan(denom)), error('denom contains NaN'), end
    if transtreat=='t'
      trnum=trnum+dtrnum;
    end
    delay=delay+ddelay;
    if (~strncmp(coeffs,'c',1)&any(imag([num,denom,trnum])))|...
        any(imag(delay))|any(~finite([num,denom,trnum,delay]))
      disp('WARNING: Complex, inf or NaN value of an estimated parameter')
      disp('Possible cause: badly conditioned normal equations')
      disp('All estimated parameters are set to NaN by elis')
      denom(denomnotfixind+1)=0*denom(denomnotfixind+1)+NaN;
      num(numnotfixind+1)=0*num(numnotfixind+1)+NaN;
      trnum=0*trnum+NaN;
      if delaytreat=='v', delay=NaN; end
    elseif strcmp(searchmode,'l')&(calcpar==1)
      if any(domain=='pq'), error('line search not yet implemented for orthopol'), end
      if transtreat=='t'
        error('Line search is not implemented for transients')
      end
      cfevalno=0;
      %error('Line search is not yet implemented')
      %Triplet: p0, p1, p2, cost functions: c0, c1, c2
      c0=costfold; c1=c0; c2=c0*(1-101*eps); p0=0; p1=0; pcoeff=1; p2=pcoeff;
      while c2<c1*(1-100*eps)
        c1=c2;
        Nf=tfcalc(exppar(domain,num0+pcoeff*dnum,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
        Df=tfcalc(exppar(domain,denom0+pcoeff*ddenom,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
        Nf=exp(-j*2*pi*(delay0+pcoeff*ddelay)*delayscale*freq).*Nf;
        Emd=varx.*abs(Nf).^2+vary.*abs(Df).^2;
        if ~isempty(covxy), Emd=Emd-2*real(covxy.*conj(Nf).*Df); end
        c2=sum(abs(Nf.*uv-Df.*y1m).^2./(2*Emd));
        cfevalno=cfevalno+1;
        if c2<c1*(1-100*eps), p0=p1; p1=pcoeff; pcoeff=pcoeff*2; p2=pcoeff; end
      end
      if p1==0 %stretch of step unsuccessful
        p2=1; pcoeff=1;
        for i=1:5
          pcoeff=pcoeff/10;
          Nf=tfcalc(exppar(domain,num0+pcoeff*dnum,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
          Df=tfcalc(exppar(domain,denom0+pcoeff*ddenom,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
          Nf=exp(-j*2*pi*(delay0+pcoeff*ddelay)*delayscale*freq).*Nf;
          Emd=varx.*abs(Nf).^2+vary.*abs(Df).^2;
          if ~isempty(covxy), Emd=Emd-2*real(covxy.*conj(Nf).*Df); end
          c1=sum(abs(Nf.*uv-Df.*y1m).^2./(2*Emd));
          cfevalno=cfevalno+1;
          if c1<c0*(1-100*eps)
            p1=pcoeff;
            if ~any(dispmesg=='ncs')&~isempty(msgsall)
              fprintf(msgsall), msgsall='';
            end
            break
          else p2=pcoeff; c2=c1;
          end
        end %for i
      end %if p1==0
      if p1>1e3 %restrict maximum step size
        p1=1e3; p2=0;
        msgsall=[msgsall,...
            sprintf('   Step size is restricted to 1e3*dpcalc\n')];
      end
      if p1==0
        if ~strcmp(dispmesg,'n')
          msgsall=[msgsall,sprintf('   Line search unsuccessful\n')];
        end
      else %improvement found with line search
        while ((p2-p0) > 0.1*p2) & (p2 > 0.01)
          %Golden section search
          gold=(sqrt(5)-1)/2; %0.6180
          if (p1-p0)>(p2-p1), pcoeff=(1-gold)*p0+gold*p1;
          else pcoeff=(1-gold)*p2+gold*p1;
          end
          Nf=tfcalc(exppar(domain,num0+pcoeff*dnum,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
          Df=tfcalc(exppar(domain,denom0+pcoeff*ddenom,1,[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
          Nf=exp(-j*2*pi*(delay0+pcoeff*ddelay)*delayscale*freq).*Nf;
          Emd=varx.*abs(Nf).^2+vary.*abs(Df).^2;
          if ~isempty(covxy), Emd=Emd-2*real(covxy.*conj(Nf).*Df); end
          c1c=sum(abs(Nf.*uv-Df.*y1m).^2./(2*Emd));
          cfevalno=cfevalno+1;
          if c1c<c1, c1=c1c;
            if pcoeff<p1, p2=p1; p1=pcoeff; c1=c1c;
            else p0=p1; p1=pcoeff; c1=c1c;
            end
          else %worse
            if pcoeff<p1, p0=pcoeff; else p2=pcoeff; end
          end
        end %while
        num=num0+p1*dnum; denom=denom0+p1*ddenom;
        if any(isnan(denom)), error('denom contains NaN'), end
        delay=delay0+p1*ddelay;
        if ~any(dispmesg=='ns')&(p2~=0)
          msgsall=[msgsall,...
              sprintf('   Triplet in line search: (%.3g,%.3g,%.3g)*dpcalc',...
              p0,p1,p2)];
          msgsall=[msgsall,sprintf(', cf evaluations: %.0f\n',cfevalno)];
        end %dispmesg
      end %p1
      if any(dispmesg=='y'), fprintf(msgsall), msgsall=''; end
    end %line search
    %
    pzt0=clock;
    if strcmp(calcrnum,'c')
      if ~any(domain=='pq')&all(finite(num)), rnum=roots(num).';
      elseif any(domain=='pq')&all(finite(num))
        [rnum,ngain]=ortroots(num,Znum); rnum=rnum.';
      else rnum=[];
      end
    end
    if strcmp(calcrdenom,'c')
      if ~any(domain=='pq')&all(finite(denom)), rdenom=roots(denom).';
      elseif any(domain=='pq')&all(finite(denom))
        [rdenom,dgain]=ortroots(denom,Zdenom); rdenom=rdenom.';
      else rdenom=[];
      end
    end
    if any(domain=='zq'),
      unstab=find(abs(rdenom)>=stablimit);
      nmph=find(abs(rnum)>=stablimit);
    elseif any(domain=='sp'),
      unstab=find(real(pzsc*rdenom)>=stablimit);
      nmph=find(real(pzsc*rnum)>=stablimit);
    elseif strcmp(domain,'w'),
      unstab=find(real((pzsc*rdenom).^2)>=stablimit);
      nmph=find(real((pzsc*rnum).^2)>=stablimit);
    elseif strcmp(domain,'r'),
      unstab=find(real(atanh(rdenom)/tauR*fs)>=stablimit);
      nmph=find(real(atanh(rnum)/tauR*fs)>=stablimit);
    end
    if (length(unstab)>0)&any(stabmode=='rc')&(calcpar==1)
      %stabilization
      [rdenom,denom,delay,num,unstabstays]=...
        stabmph(domain,denom,num,delay,fs,Zdenom,Znum,Esmrdenom,rdenom,dgain,...
        pzsc,unstab,stabmode,stablimit,freq,delaytreat,delayscale,numfix,...
        numfixind,denomfix,denomfixind,paramtreat,'stab',coeffs,tauR);
      if ~any(dispmesg=='ncs')
        fprintf('   %.0f unstable pole(s) stabilized by ',length(unstab))
        if strcmp(stabmode,'r'), fprintf('reflection\n')
        elseif strcmp(stabmode,'c'), fprintf('contraction\n')
        end
      end
      unstab=unstabstays;
    end
    if (length(nmph)>0)&any(mphmode=='rc')&(calcpar==1)
      %minimum phase
      [rnum,num,delay,denom,nmphstays]=...
        stabmph(domain,num,denom,delay,fs,Znum,Zdenom,Esmrdenom,rnum,ngain,...
        pzsc,nmph,mphmode,stablimit,freq,delaytreat,delayscale,denomfix,...
        denomfixind,numfix,numfixind,paramtreat,'nmph',coeffs,tauR);
      if ~any(dispmesg=='ncs')
        fprintf('   %.0f nonminimum phase zero(s) adjusted by ',length(nmph))
        if strcmp(stabmode,'r'), fprintf('reflection\n')
        elseif strcmp(stabmode,'c'), fprintf('contraction\n')
        end
      end
      nmph=nmphstays;
    end
    if calldrawnow, drawnow, end
    if (length(unstab)>0)&~any(dispmesg=='ncs')&(calcpar==1)
      msgsall=[msgsall,...
          sprintf(['WARNING: cycle %.0f gives %.0f unstable pole(s)',...
            ' (stablimit=%.5g):\n'],cycle,length(unstab),stablimit)];
      for i=1:length(unstab)
        if imag(rdenom(unstab(i)))~=0
          unstxt=sprintf('  %+6.4g %+6.4g*j',...
            real(pzsc*rdenom(unstab(i))),imag(pzsc*rdenom(unstab(i))) );
          msgsall=[msgsall,...
              sprintf([unstxt,setstr(32*ones(1,19-length(unstxt)))])];
          if any(domain=='zq')
            msgsall=[msgsall,sprintf('   |z%.0f|=%6.4g\n',i,...
                pzsc*abs(rdenom(unstab(i))))];
          elseif strcmp(domain,'w')
            msgsall=[msgsall,sprintf('   w%.0f^2=%+6.4g %+6.4g*j\n',i,...
                real((pzsc*rdenom(unstab(i)))^2),...
                imag((pzsc*rdenom(unstab(i)))^2))];
          elseif strcmp(domain,'r')
            msgsall=[msgsall,sprintf('   atanh(r%.0f)/tauR=%+6.4g %+6.4g*j\n',i,...
                real(atanh(rdenom(unstab(i)))*tauR*fs),...
                imag(atanh(rdenom(unstab(i)))*tauR*fs))];
          else msgsall=[msgsall,sprintf('\n')];
          end
        else %real unstable pole
          unstxt=sprintf('  %+6.4g          ',real(pzsc*rdenom(unstab(i))) );
          msgsall=[msgsall,...
              sprintf([unstxt,setstr(32*ones(1,19-length(unstxt)))])];
          if any(domain=='zq')
            msgsall=[msgsall,sprintf('   |z%.0f|=%6.4g\n',i,...
                pzsc*abs(rdenom(unstab(i))))];
          elseif strcmp(domain,'w')
            msgsall=[msgsall,...
                sprintf('   w%.0f^2=%+6.4g\n',i,(pzsc*rdenom(unstab(i)))^2)];
          elseif strcmp(domain,'r')
            msgsall=[msgsall,...
                sprintf('   atanh(r%.0f)/tauR=%+6.4g\n',i,atanh(rdenom(unstab(i)))/tauR*fs)];
          else msgsall=[msgsall,sprintf('\n')];
          end
        end %if imag(...
      end %for i
    end %if length(unstab)
    pztime=pztime+etime(clock,pzt0); %collect pole/zero calculation time
  end %cycle>0
  if any(dispmesg=='y')&~isempty(msgsall), fprintf(msgsall), msgsall=''; end
  %
  if calldrawnow, drawnow, end
  %
  %evaluation of the cost function
  Nf=0*freq; Df=0*freq; Nft=0*freq;
  if any(~finite([num,denom,trnum,delay])) %useless result
  else
    if ~any(domain=='pq')
      Nf=tfcalc(exppar(domain,num,1,[],1,[],[],[],[],[],[],[],coeffs,tauR),freq);
      Df=tfcalc(exppar(domain,denom,1,[],1,[],[],[],[],[],[],[],coeffs,tauR),freq);
      if transtreat=='t', Nft=tfcalc(exppar(domain,trnum,1,[],1,[],[],[],[],[],[],[],coeffs,tauR),freq); end
    else %orthopol: domain ='p'
      Nf=vnum*num'; Df=vdenom*denom';
      if transtreat=='t', Nft=vnumt*trnum'; end
    end
  end
  Nf=exp(-j*2*pi*delay*delayscale*freq).*Nf;
  Nft=exp(-j*2*pi*delay*delayscale*freq).*Nft;
  Emd=varx.*abs(Nf).^2+vary.*abs(Df).^2;
  if ~isempty(covxy)
    Emd=Emd-2*real(covxy.*conj(Nf).*Df);
  end
  Em=abs(Nf.*uv-Df.*y1m+Nft)./sqrt(2*Emd);
  if calcpar==1, costfnew=real(Em'*Em); end
  %
  %Orthopol saving, cycle>0
  if any(domain=='pq')&(cycle>0)&(1==2)
    if strcmp(saveorthopoldata,'yes')
      if ~exist('T'), T=[]; end
      condJ=condinum;
      if strcmp(computer,'VAXVMSD')|strcmp(computer,'VAXVMSG')
        if exist('orthpdn.mat'), unix(['delete orthpdn.mat;*']), end
      end
      disp('   Save data to orthpdn.mat ...')
      save orthpdn.mat covxy cycle delay delayscale ...
      denom denomord Df freq fs numord ...
      num Nf stdtf vartf varx vary vnum vdenom uv y1m Znum Zdenom condJ J ...
      initset basistreat ...
      ddelay ddenom dnum dparam
    end
  end
  %
  if cycle==0 %initialization
    numold=num; denomold=denom; delayold=delay; costfold=costfnew;
    maxdrp=NaN;
    trnumold=trnum; Nftold=Nft; Dfold=Df;
  elseif calcpar==1 %consecutive parameter calculation cycle
    parvo=[numold(numnotfixind+1),denomold(denomnotfixind+1),trnumold];
    parv=[num(numnotfixind+1),denom(denomnotfixind+1),trnum];
    if strcmp(delaytreat,'v') %delay also changes
      parvo=[parvo,delayold]; parv=[parv,delay];
    end
    id=find(parv==0); if ~isempty(id), parv(id)=[]; parvo(id)=[]; end
    maxdrp=max([abs((parv-parvo)./parv),0]);
  else
    %maxdrp=NaN;
  end
  if calcpar==1, dcostf=costfnew-costfold; end
  %
  if (cycle>0)&(cycleend==0) %administration of old and new values
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      if (costfnew<=costfold*(1+rcostvar)) & ...
          ~(strcmp(stabmode,'l')&~isempty(unstab)) & ...
          ~(strcmp(mphmode,'l')&~isempty(nmph))
        %at least as good as previous estimate
        % and for limitation, no violating poles/zeros are found
        lambda=lambda/2; %In any case, decrease lambda
        lambdanew=max(lambda,lambdamin);
        lambdadec=lambdadec+1;
        if (lambdadec>=lambdadecr)&(lambdanew>0)
          lambdaold=lambdanew; lambdanew=0;
        end
        numold=num; denomold=denom; delayold=delay;
        costfold=costfnew;
        trnumold=trnum;
      else %worse than previous
        lambdadec=0;
        if lambda>0, lambdanew=lambda*10; else lambdanew=lambdaold; end
        %restore previous solution
        num=numold; denom=denomold; delay=delayold; costfnew=costfold;
        trnum=trnumold;
        Nft=Nftold; Df=Dfold;
      end
    else %not LM
      numold=num; denomold=denom; delayold=delay; costfold=costfnew;
      trnumold=trnum;
    end
  else %only covariance calculation in cycle
    num=numold; denom=denomold; delay=delayold; costfnew=costfold;
    trnum=trnumold;
  end %administration of old and new values
  if nargoutelis>=2
    cfv(cycle+1,1)=costfnew;
    if (strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd'))&(cycle>0)
      cfv(cycle+1,2)=lambda;
    end
  end
  %---------------------------------------------
  %administration of the completed cycle
  %
  fis=5; %number of digits in showing delay in tabs
  delayscr=delay*delayscale;
  if ~any(findstr(domain,'z'))&~any(findstr(domain,'q'))
    delayscr=delayscr*fs;
  else
    dfr=abs(delayscr-round(delayscr));
    if dfr>1e-8
      fis=max(fis,ceil(-log10(dfr))+2);
    end
  end
  %Messages to the Command window
  if ~any(dispmesg=='n')&(calcpar==1)
    if (cycle>0)&(itmax>0)
      if finite(dcostf)
        msgsall=[msgsall,sprintf('   Costf: %.5g, cfth: %.1f, dcf: %+.5g',...
            costfold,N/2*Mm,dcostf)];
      else
        msgsall=[msgsall,sprintf('   Costf: %.5g, cfth: %.1f, dcf: %.5g',...
            costfold,N/2*Mm,dcostf)];
      end
      msgsall=[msgsall,sprintf(', stop for |dcf/cf| < %.2g\n',rcostvar)];
    elseif cycle==0
      msgsall=[msgsall,sprintf('   Cost function: %.5g, cfth: %.1f\n',...
          costfold,N/2*Mm)];
    end
    if (rparvar>0)&(cycle>0)&~any(dispmesg=='s')
      msgsall=[msgsall,sprintf(['   Max. rel. var. of parameters: %.5g',...
            ', stop for |dp/p| < %.2g\n'],maxdrp,rparvar)];
    end
    if (strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd'))&~any(dispmesg=='s')
      if cycle==0
        %msgsall=[msgsall,sprintf('   Lambda is set to %.4g\n',lambda)];
      elseif lambdadec==0
        msgsall=[msgsall,sprintf(['   Results of earlier iteration cycle ',...
              'have been restored\n']),...
            sprintf('   lambda=%.4g is increased to %.4g\n',lambda,lambdanew)];
      else %lambda decreased
        if lambdanew>0
          msgsall=[msgsall,sprintf(['   lambda=%.4g is set to %.4g, %.0f. ',...
                'consecutive improvement\n'],lambda,lambdanew,lambdadec)];
          if lambdamin==0
            msgsall=[msgsall,...
                sprintf(['      lambdalim=%.2e\n      improvements required',...
                  ' before trial with lambda=0: %.0f\n'],lambdalim,lambdadecr)];
          end
        else msgsall=[msgsall,sprintf('   lambda=%.4g, lambdanew=0\n',lambda)];
        end
      end
    elseif (dcostf>rcostvar*costfold)&~any(dispmesg=='s')
      msgsall=[msgsall,sprintf(['WARNING: there is an increase of +',...
            num2str(dcostf),' in the cost function\n'])];
    end
    if strcmp(delaytreat,'v')
      if any(domain=='zq')
        msgsall=[msgsall,sprintf(['   Delay: %.',num2str(fis),'g samples = %.5g s\n'],...
            delay*delayscale,delay*delayscale/fs)];
      elseif any(domain=='spwr')
        msgsall=[msgsall,sprintf('   Delay: %.5g s\n',delay*delayscale/fs)];
      end
    end
    if any(dispmesg=='y'), fprintf(msgsall), msgsall=''; end
  end %dispmesg
  %
  %update pole/zero arrays
  if calcpar==1
    if pzfollown==1
      polesarray=rdenom; zerosarray=rnum;
    else
      polesarray=[rdenom;polesarray]; zerosarray=[rnum;zerosarray];
      if length(polesarray)>0,
        if length(polesarray(:,1))>pzfollown,
          polesarray=polesarray(1:pzfollown,:);
        end
      end
      if length(zerosarray)>0,
        if length(zerosarray(:,1))>pzfollown,
          zerosarray=zerosarray(1:pzfollown,:);
        end
      end
    end
  end
  if calliterctrl&~isempty(get(0,'Children'))
    if calldrawnow==1, drawnow, end
    %Store value to minimize iterctrl calls, to bypass bug on PC:
    pause(0) %Read in button push if there was any
    itctrlchecked=iterctrl('checked');
    if ~strcmp(itctrlchecked,'Continue')
      if ~isempty(msgsall), fprintf(msgsall), msgsall=''; end
    end
    if strcmp(itctrlchecked,'Pause')|...
        strcmp(itctrlchecked,'Matlab prompt')
      if strcmp(itctrlchecked,'Pause'),
        disp('Iteration paused by GUI')
      else
        fprintf('Type your commands in the mini-window\n\n')
      end
      while strcmp(itctrlchecked,'Pause')|...
          strcmp(itctrlchecked,'Matlab prompt')
        drawnow, pause(0.1) %wait and draw
        if isPC
          %Such commands that allow to accept Enter in mini-window,
          %to bypass bug on PC
          clv=clock; drawnow, pause(0)
          while etime(clock,clv)<0.1, drawnow, pause(0), end
        end
        itctrlchecked=iterctrl('checked'); drawnow
      end %while
      disp('Continue...')
    end %pause or Matlab prompt
    if strcmp(itctrlchecked,'Keyboard')
      disp('You may now enter commands within the workspace of elis')
      disp('Modify GUI selection and type ''return'' to continue')
      keyboard
      itctrlchecked=iterctrl('checked');
    end
    if strcmp(itctrlchecked,'Finish')
      cycleend=1; termcond=max(termcond,4);
      itctrllastchecked='Finish';
    elseif strcmp(itctrlchecked,'Cancel')
      iterctrl('Continue');
      disp('elis run canceled through GUI')
      pdat=[]; fit=[]; Cp=[]; CR=[]; cfv=[]; pvtr=[];
      return
    elseif strcmp(itctrlchecked,'Abort')
      iterctrl('Continue'); error('Abort requested through GUI')
    end
    isPlot=strcmp(itctrlchecked,'Plot');
  else
    isPlot=0;
  end
  %
  %evaluate stop criterion
  if (cycle>=itmax), cycleend=1; termcond=max(termcond,1); end
  if (cycle>0) & ...
      ( (costfnew==0) | (abs(dcostf)<(rcostvar)*costfold) | (maxdrp<rparvar) )
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      if lambda<lambdalim, cycleend=1; end
    else %other algorithms
      cycleend=1;
    end
    if cycleend==1
      if (costfnew==0)|(abs(dcostf)<(rcostvar)*costfold)
        termcond=max(termcond,2);
      end
      if maxdrp<rparvar, termcond=max(termcond,3); end
    end
  end
  %
  %PLOTS
  if (calcpar==1) & (  isPlot | ( finite(plotdens) & ...
      ((cycle>0)|(plotdens>=0)) & ... 
      ( ...
      ( (cycnoplot+1>=plotdens) & (cycle>=plotdensoffs) & (plotdenstime<=etime(clock,plotinst)) ) |...
      (cycleend==1) ) ...
      ) ) & (lambdadec>0)
    %Plot results
    plotno=plotno+1; cycnoplot=0;
    if itcdone==0 %try to put up iterctrl
      if ~any(findobj(gcf,'type','uimenu','tag','iteration_menu'))&exist('iterctrl')
        iterctrl;
      end
      itcdone==1;
    end
    if calliterctrl
      itctrlchecked=iterctrl('checked');
      if calldrawnow==1, drawnow, end
      if strcmp(itctrlchecked,'Hold graph')&strcmp(plotdone,'y')
        disp('Last graph of elis held by GUI')
        while strcmp(itctrlchecked,'Hold graph')
          drawnow
          itctrlchecked=iterctrl('checked');
        end %Wait
        disp('Continue...')
      end
      if strcmp(itctrlchecked,'Keyboard')
        disp('You may now enter commands within the workspace of elis')
        disp('Modify GUI selection and type ''return'' to continue')
        keyboard
        itctrlchecked=iterctrl('checked');
      end
      if strcmp(itctrlchecked,'Abort')
        iterctrl('Continue'); error('Abort requested through GUI')
      end
    end
    plott0=clock;
    %plots
    if isnan(fmin)|((min(freq)<0)&(fmin==0)), fmin=faxsc*min(freq)*fs; end
    if strcmp(plotmode,'o')
      if fmin<=0
        ftmp=sort(freq); ind=find(ftmp<=0); ftmp(ind)=[]; fmin=min(faxsc*ftmp)*fs;
        %clear ftmp, clear ind
      end
    end
    if fmax==0, fmax=0.001; fmin=-1e-4; end
    resol=256;
    if any(domain=='zq')
      if 1|(numord<100)|(fmax/faxsc<=fs/4)|strcmp(plotmode,'o')
        if strcmp(plotmode,'i')
          %if fmax/faxsc<=fs/4
            freqd=[fmin/fs:(fmax-fmin)/fs/resol:fmax/fs]'/faxsc;
          %else
          %  freqd=[fmin/fs:(1/2-fmin/fs)/resol:0.5]'/faxsc;
          %end
        else %log
          freqd=logspace(log10(fmin/fs),log10(fmax/fs),resol)'/faxsc;
        end
        ind0=[];
        dfd=polyval(denom(denomord+1:-1:1),exp(-j*freqd*2*pi));
        ind0=find(dfd==0);
        if ~isempty(ind0)
          freqd(ind0)=[]; dfd(ind0)=[];
        end
        if any(domain=='z')
          tfd=polyval(num(numord+1:-1:1),exp(-j*freqd*2*pi))./dfd;
        else %orthopol
          tfd=exp(-j*freqd*2*pi).*orthpval(num,Znum,freqd)./orthpval(denom,Zdenom,freqd);
        end        
      else
        freqd=[0:1/2/resol:0.5]';
        tfd=fft([num,zeros(1,2*resol-numord-1)]')...
          ./fft([denom,zeros(1,2*resol-denomord-1)]');
        tfd=tfd(1:length(tfd)/2);
      end
      if abs(fs/2-fmax/faxsc)>fs/2, fmax=fs/2*faxsc; end
    elseif any(domain=='spwr')
      if strcmp(plotmode,'i')
        freqd=[fmin/fs:(fmax-fmin)/fs/(resol-1):fmax/fs]'/faxsc;
        if (freqd(1)==0)&( any(domain=='swpr')&(denom(length(denom))==0) )
          freqd(1)=[];
        end
      else
        freqd=logspace(log10(fmin/fs),log10(fmax/fs),resol)'/faxsc;
      end
      freqd=sort([freqd(:);freq(:)]);
      %tfd=polyval(num,j*2*pi*freqd)./polyval(denom,j*2*pi*freqd);
      if ~any(domain=='pq')
        tfd=tfcalc(exppar(domain,num,denom,[],[],[],[],[],[],[],[],[],coeffs,tauR),freqd);
      else %orthopol
        tfd=orthpval(num,Znum,freqd)./orthpval(denom,Zdenom,freqd);
      end
    end
    magd=abs(tfd); ind=find(finite(magd)); magdmax=max(magd(ind));
    i0=find(magd==0); magd(i0)=magd(i0)+realmin;
    i0=find(magd<1e-5*magdmax);
    %
    %Delete all axes objects in current figure
    %Delete all axes because findobj may be missing (Matlab 4.1 or earlier)
    %delete(findobj(gcf,'Type','axes'));
    if isnan(helis)
      flist=findobj(get(0,'children'),'Type','figure');
      for fi=flist(:)'
        if findstr([get(fi,'Name'),' '],'ELiS')
          if isempty(get(fi,'tag'))
          elseif strcmp(get(fi,'tag'),'elis_window')
            if ~isnan(helis), error('More than one ELiS windows'), end
            helis=fi;
          end
        end
      end
      if isnan(helis)
        gcf; drawnow;
        helis=gcf;
        while any(findstr(get(helis,'tag'),'fdtool_'))|any(findstr(get(helis,'tag'),'_main'))
          helis=figure(rand); drawnow;
        end
      end
    end
    if plotno==1
      if ~strncmp(version,'5.2',3) %In 5.2 gray boxes would remain
        set(helis,'nextplot','add')
        delete(findobj(helis,'Type','axes'))
        delete(findobj(helis,'Type','uicontrol'))
        pause(0)
      else %desperate solution for 5.2
        hiter=findobj(helis,'type','uimenu','tag','iteration_menu');
        if ~isempty(hiter)
          tag=get(helis,'tag'); name=get(helis,'name');
          nt=get(helis,'numbertitle');
          if ~isempty(name), nt='off'; end
          pos=get(helis,'position');
          if ~strncmp('5.2',version,3), toolbar=get(helis,'toolbar'); end
          %
          %set(helis,'nextplot','add')
          %delete(get(helis,'children')), pause(0) %gray boxes remain in 5.2
          %
          %delete(findobj(helis,'Type','axes'))
          %delete(findobj(helis,'Type','uicontrol')), pause(0) %gray boxes remain in 5.2
          %
          close(helis), pause(0)
          if rem(helis,1)~=0
            hall=get(0,'children'); helis=ceil(max([hall,0])+0.01);
          end
          helis=figure(helis);
          %
          set(helis,'name',name,'tag',tag,...
            'position',pos,'numbertitle',nt)
          iterctrl('Initialize',helis);
          %if ~strncmp('5.2',version,3), set(helis,'toolbar',toolbar); end
          if ~strncmp('5.2',version,3), set(helis,'toolbar','none'); end
          %drawnow, pause(0) %THIS IS THE KEY FOR ONE RUN?!
          %for the second run in the same window, probably only 'close'
          %helps to avoid that the menu Iterctrl disappears
        else
          clf
        end
      end
    end
    %
    st=dbstack;
    if (length(st)>=2)&...
        (any(findstr(st(2).name,'peeling.m'))|any(findstr(st(2).name,'autoorder.m')))
			mboffs=0.04;
		else mboffs=0;
		end
		%
    if mean(get(gcf,'Color'))<=0.5, white='w'; else white='k'; end
    if get(0,'ScreenDepth')<4, blue=white; red=white; green=white;
    else blue='b'; red='r'; green='g';
    end
    if get(0,'ScreenDepth')>4, cyan='c'; else cyan=red; end
    %
    if calldrawnow, drawnow, end
    %Deleting of uncontrols is commented out:
    %Matlab prompt window was deleted too
    %feval('delete',findobj(gcf,'Type','uicontrol'));
    %hc=get(gcf,'Children');
    %for i=1:length(hc)
    %if strcmp(get(hc(i),'Type'),'uicontrol'), delete(hc(i)), end
    %end
    aproc=0.65; %procent of amplitude plot
    %subplot(2,2,1): [0.1300,0.5825,0.3175,0.3375]
    %subplot(2,2,3): [0.1300,0.1100,0.3175,0.3375]
    %subplot(1,2,2): [0.5825,0.1100,0.3175,0.8100]
    %hph=axes('Position',[0.13,0.15,0.3175,0.2975-(aproc-0.5)*.68]);
    if isPC, amod=0.02; else amod=0; end
    %
    if ~isnan(hamp)&(1==2)
      %"Bypass" strange disappearance of menu on PC and Sun:
      %This is necessary to avoid occasional delete of plot in sme
      %if during iteration the sme is clicked on
      gcfh=gcf; set(0,'currentfigure',helis);
      %This is the actual fix
      %hxs=subplot(1,1,1); plot(5,5,'parent',hxs); delete(hxs)
      hxs=axes('position',[0,0,0.01,0.01]); plot(5,5,'parent',hxs); delete(hxs)
    end
    %
    if plotno==1
      hamp=axes('Position',...
        [0.13,0.5825-(aproc-0.5)*.68+amod,0.3175,0.3375+(aproc-0.5)*.68-amod],...
        'parent',helis);
    end
    magmeas=abs(tfm); ind=find(finite(magmeas)&(magmeas>0));
    magmeasmax=max(magmeas(ind));
    i0=find(magmeas<1e-5*magmeasmax);
    if ~isempty(i0)
      magmeas(i0)=magmeas(i0)*0+0.9e-5*magmeasmax;
    end
    minmag=20*log10(min(1e-5*magmeasmax,1e-5*magdmax));
    if length(freq)>1
      ind=find( (freqd>=min(freq))&(freqd<=max(freq)) );
    else %freq scalar
      ind=find( (freqd>=fmin/faxsc)&(freqd<=fmax/faxsc) );
    end
    indinf=find(~finite(magd(ind))|(magd(ind)<=0));
    if ~isempty(indinf), ind(indinf)=[]; end
    %
    am4=max( [20*log10(magd(ind));20*log10(magmeas)] );
    am3=max(minmag,min( [20*log10(magd(ind));20*log10(magmeas)] ));
    am4=am4+0.05*(am4-am3); am3=am3-0.05*(am4-am3);
    %indtoplot=find(20*log10(magd)>=am3);
    indtoplot=[1:length(magd)]';
    if ~((amin==0)&(amax==0))
      if finite(amin), am3=amin; end
      if finite(amax), am4=amax; end
    end
    %
    if ~any(freq*fs<fmax)
      if ~any(dispmesg=='nc'), fprintf(msgsall), msgsall=''; end
      disp('WARNING: No measured points can be shown in given frequency band')
      pause(3)
    end
    if am3==am4, am3=am3-2*abs(am3)*eps; am4=am4+2*abs(am4)*eps; end
    if abs(am4-am3)<2*eps*(am3+am4)/2
      am3=(1-1e-9)*(am3+am4)/2-eps; am4=am3/(1-1e-9)^2+2*eps;
      %am3=(am3+am4)/2-eps; am4=am3+2*eps;
    elseif abs(am4-am3)<200*realmin
      am3=-100*realmin; am4=100*realmin;
    end
    axv=[fmin,fmax,am3,am4];
    if strcmp(plotmode,'i')
      axv(1:2)=axv(1:2)+0.03*diff(axv(1:2))*[-1,1];  
    else %log
      axv(1:2)=axv(1:2).*(axv(2)/axv(1)).^(0.03*[-1,1]);  
    end
    axvrat=abs(diff(axv(1:2))/diff(axv(3:4)));
    %bypass bug in Matlab: badly conditioned matrix in plot
    axvratlim=2e6; %1e5???
    if axvrat>axvratlim
      axv(3:4)=mean(axv(3:4))+(axv(3:4)-mean(axv(3:4)))*axvrat/axvratlim;
    elseif axvrat<1/axvratlim
      axv(1:2)=mean(axv(1:2))+(axv(1:2)-mean(axv(1:2)))/(axvrat*axvratlim);
    end
    if transtreat=='t', mmod=abs(tfm-exd.*Nft./Df./uv); mcol=cyan;
    else
      mmod=magmeas;
      mcol=green;
    end
    if plotno==1
      if strcmp(plotmode,'i')
        plot(freq*faxsc*fs,20*log10(mmod),['+',mcol],'parent',hamp)
        set(hamp,'nextplot','add')
        hampy=plot(freqd(indtoplot)*faxsc*fs,20*log10(magd(indtoplot)),...
          ['-',red],'parent',hamp);
        set(hamp,'nextplot','replace')
      else
        semilogx(freq*faxsc*fs,20*log10(mmod),['+',mcol],'parent',hamp)
        set(hamp,'nextplot','add')
        hampy=semilogx(freqd(indtoplot)*faxsc*fs,20*log10(magd(indtoplot)),...
          ['-',red],'parent',hamp);
        set(hamp,'nextplot','replace')
      end
    else
      set(hampy,'xdata',freqd(indtoplot)*faxsc*fs,...
        'ydata',20*log10(magd(indtoplot)))
    end
    set(hamp,'xlim',axv(1:2),'ylim',axv(3:4)), pause(0)
    if plotno==1
      title('Magnitude','parent',hamp)
      ylabel('dB','parent',hamp)
      if strcmp(freqax,'f'), xlabel('Frequency, Hz','parent',hamp)
      elseif strcmp(freqax,'r'), xlabel('Radian frequency','parent',hamp)
      end
      if calldrawnow, drawnow, end
      %hph=axes('Position',[0.13,0.15+amod,0.3175,0.2975-(aproc-0.5)*.68-amod]);
      hph=axes('Position',[0.13,0.10+amod+mboffs,0.3175,0.3275-(aproc-0.5)*.68-amod],...
        'parent',helis);
      if strcmp(plotmode,'i')
        plot([fmin,fmax],[0,0],[':',white],'parent',hph)
      else
        semilogx([fmin,fmax],[0,0],[':',white],'parent',hph)
      end
      axv=[axv(1:2),-180,180];
      set(hph,'xlim',axv(1:2),'ylim',axv(3:4))
      if plotlevel=='b', title('Phase','parent',hph)
      else title('Phase error','parent',hph)
      end
      ylabel('Degrees','parent',hph)
      if strcmp(freqax,'f'), xlabel('Frequency, Hz','parent',hph)
      elseif strcmp(freqax,'r'), xlabel('Radian frequency','parent',hph)
      end
    end %plotno==1
    %
    if ~any(domain=='pq')
      tfe=exp(-j*2*pi*freq*delay*delayscale).*...
        tfcalc(exppar(domain,num,denom,[],1,[],[],[],[],[],[],[],coeffs,tauR),freq);
    else %orthopol
      Nf=vnum*num'; Df=vdenom*denom';
      tfe=exp(-j*2*pi*freq*delay*delayscale).*Nf./Df;
    end
    if transtreat=='t', tfmzmod=tfm-exd.*Nft./Df./uv; mcol=cyan;
    else
      tfmzmod=tfm;
      if plotlevel=='b', mcol=green; else mcol=cyan; end
    end
    indz=find(tfmzmod==0); indnz=find(tfmzmod~=0);
    tfmzmod(indz)=tfmzmod(indz)+10*eps;
    if plotlevel=='b'
      phasediff=unwrap(angle(tfmzmod))/pi;
      phe=angle(tfe)/pi;
      phmode=round((phe-phasediff)/2)*2;
      phe=phe-phmode;
    else phasediff=angle(tfe./tfmzmod)/pi;
    end
    %hold on
    if (plotno==1)|~ishandle(hphd)
      set(hph,'nextplot','add')
      if strcmp(plotmode,'i')
        hphd=plot(freq(indnz)*faxsc*fs,phasediff(indnz)*180,['+',mcol],'parent',hph);
        if plotlevel=='b'
          hphe=plot(freq(indnz)*faxsc*fs,phe*180,'-r','parent',hph);
        end
      else
        hphd=semilogx(freq(indnz)*faxsc*fs,phasediff(indnz)*180,['+',mcol],'parent',hph);
        if plotlevel=='b'
          hphe=semilogx(freq(indnz)*faxsc*fs,phe*180,'-r','parent',hph);
        end
      end
      %hold off
      set(hph,'nextplot','replace')
      %
    else %not first plot
      set(hphd,'Ydata',phasediff(indnz)*180)
      if plotlevel=='b'
        set(hphe,'xdata',freq(indnz)*faxsc*fs,'Ydata',phe*180);
      end
      pause(0)
    end
    if plotlevel=='b'
      yl(1)=min(min(phasediff)-1/6,-1)*180;
      yl(2)=max(max(phasediff)+1/6,1)*180;
      set(hph,'ylim',yl)
    end
    if plotno==1
      %Texts to plot:
      dcfinct='';
      if (strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd'))&(cycle>0)
        if lambdadec==0, dcfinct=' (not applied)'; end
      end
      if any([covxy;0]~=0), cxytxt='covxy';
      else cxytxt='';
      end
      if (searchmode=='l'), cxytxt=[cxytxt,', line search']; end
      if any(stabmode=='rc'), cxytxt=[cxytxt,', stab: ',stabmode]; end
      if any(mphmode=='rc'), cxytxt=[cxytxt,', nmph: ',mphmode]; end
      if ~isempty(cxytxt), if strcmp(cxytxt(1:2),', '), cxytxt(1:2)=''; end, end
      if calldrawnow, drawnow, end
      txthu=axes('Position',[0.5,0.80,0.5,0.20],'tag','elistexts','parent',helis,'visible','off');
      txthl=axes('Position',[0.5,0,0.5,0.20],'tag','elistexts','parent',helis,'visible','off');
      %Hide text axes from zoom:
      txthucov=axes('Position',[0.5,0.80,0.5,0.20],'parent',helis,'visible','off');
      txthlcov=axes('Position',[0.5,0,0.5,0.20],'parent',helis,'visible','off');
      cfpos=0.04; txtGUI='';
    else
      if ~exist('hcftext'), hcftext=0; elseif ishandle(hcftext), delete(hcftext), end
    end %plotno==1
    if termcond==4, txtGUI=', stopped by GUI'; end
    itertxt=sprintf(['cfth=%.1f,    iter=%.0f',txtGUI],N/2*Mm,cycle);
    if plotno==1
      hitertxt=text(cfpos,0+mboffs*7,itertxt,'VerticalAlignment','bottom','parent',txthl);
    else
      set(hitertxt,'string',itertxt)
    end
    if ~exist('ith')&isempty(txtGUI)
      ith=text(1,0,'Iterating...','tag','iteration message',...
        'parent',txthl,'Verticalalignment','bottom','horizontalalignment', 'right');
    elseif exist('ith')&~isempty(txtGUI)
      set(ith,'string','')
    end
    txtp=[' (',domain,')'];
    txtydiff=0.035;
    if cycle>0
      if finite(dcostf)
        hcftext=text(cfpos,txtydiff*5+mboffs*7,[sprintf('cf=%.5g, dcf=%+.3g',...
            costfnew,dcostf),dcfinct],'VerticalAlignment','bottom',...
          'parent',txthl);
      else
        hcftext=text(cfpos,txtydiff,[sprintf('cf=%.5g, dcf=%.3g',costfnew,dcostf),...
            dcfinct],'VerticalAlignment','bottom','parent',txthl);
      end
      atxt=['Alg: ',algtype,txtp,', init: ',txti];
      if (plotno<=2)&(plotlevel~='b')
        if ~exist('halgtext'), halgtext=0; elseif ishandle(halgtext), delete(halgtext), end
        halgtext=text(0.05,0.975-5*txtydiff,atxt,'VerticalAlignment','bottom',...
          'parent',txthu);
      end
    else %initial cycle
      hcftext=text(cfpos,5*txtydiff,sprintf('cf=%.5g',costfnew),...
        'VerticalAlignment','bottom','parent',txthl);
      if (plotlevel~='b')
        atxt=['Init: ',txti,txtp];
        halgtext=text(0.05,0.975-5*txtydiff,atxt,'VerticalAlignment','bottom',...
          'parent',txthu);
      end
    end
    if ~exist('Jdeficiency'), Jdeficiency=0; end
    if isempty(Jdeficiency), Jdeficiency=0; end
    hJtxt='';
    if (Jdeficiency==0)|isnan(Jdeficiency)
      if ~isnan(condinum)
        hJtxt=sprintf('Condition number: %.3e',condinum);
      end
    else
      hJtxt=sprintf('Deficiency of J: %.0f',Jdeficiency);
    end
    if plotlevel~='b'
      if ~exist('hJ'),
        hJ=text(cfpos,2*5*txtydiff+mboffs*7,hJtxt,'verticalalignment','bottom',...
          'parent',txthl);
      else set(hJ,'string',hJtxt)
      end
    end
    if (plotno==1)&(plotlevel~='b')
      text(0.05,0.975-2*5*txtydiff,fixtxt,'VerticalAlignment','bottom','parent',txthu)
      if ~isempty(cxytxt)
        text(0.05,0.975-3*5*txtydiff,cxytxt,'VerticalAlignment','bottom','parent',txthu)
        ldntpos=1-4*txtydiff;
      else ldntpos=0.975-3*txtydiff;
      end
    end
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      if lambdadec>0
        if lambdanew>0, ltxt=sprintf('(%.0f. decr.)',lambdadec);
        else ltxt='';
        end
      else ltxt='(to be increased)';
      end
      if cycle>0,
        %axes(txth)
        if ~exist('hlambda'), hlambda=[]; 
        elseif ishandle(hlambda)&~isequal(hlambda,0), delete(hlambda)
        end
        if plotlevel~='b'
          hlambda=text(0.05,0.975-(5*(1-ldntpos)),sprintf(['lambda=%.5g ',ltxt],lambda),...
            'VerticalAlignment','bottom','tag','lambdatext','parent',txthu);
        end
      end
    end
    if any(domain=='spwr'), dtxt='s';
    elseif any(domain=='zq'), dtxt='samples';
    end
    %hold on
    set(helis,'nextplot','add')
    %Put delay text to graph window
    if calldrawnow, drawnow, end
    if plotno==1
      txthdel=axes('Position',[0.5,0.15,1,0.2],'parent',helis,'visible','off');
    end
    if (plotno==1)|~exist('hd')
      if domain=='r', tauRtxt=sprintf(',  tauR=%.3g s',tauR/fs); else tauRtxt=''; end
      hd=text(0.25,0+mboffs*7,sprintf(['Delay: %.',num2str(fis),'g ',dtxt,'%s'],delayscr,tauRtxt),...
        'HorizontalAlignment','center',...
        'VerticalAlignment','bottom','parent',txthdel);
      set(hd,'Units','normalized')
    elseif delaytreat~='f'
      set(hd,'string',sprintf(['Delay: %.',num2str(fis),'g ',dtxt],delayscr))
    end
    %hold off
    set(helis,'nextplot','replace')
    if plotmode=='o'
       %Fix ticks on plots
       fixlgtck([hamp,hph],axv)
    end
    %
    %Pole/zero plot (put last to keep this current axes, without using axes(h))
    if strcmp(calcrnum,'c')|strcmp(calcrdenom,'c')
      if strcmp(pzlimit,'n')
        axvpz=pzaxis;
        if strcmp(axvpz,'a'), axvpz=''; end
      else %pzlimit='y'
        pzs=[rnum,rdenom,0]*pzsc; ind=find(~finite(pzsc)); pzsc(ind)=[];
        if any(domain=='spwr')
            ind=find(-pzlimitv*2*pi*fmax/1.15<real(pzs));
            if ~isempty(ind)
            axvpz(1)=max(-pzlimitv*2*pi*fmax/1.15,min(real(pzs(ind))));
          else axvpz(1)=max(real(pzs));
          end
          ind=find(pzlimitv*2*pi*fmax/1.15>real(pzs));
          if ~isempty(ind)
            axvpz(2)=min(pzlimitv*2*pi*fmax/1.15,max(real(pzs(ind))));
          else axvpz(2)=min(real(pzs));
          end
          ind=find(-pzlimitv*2*pi*fmax/1.15<imag(pzs));
          if ~isempty(ind)
            axvpz(3)=max(-pzlimitv*2*pi*fmax/1.15,min(imag(pzs(ind))));
          else axvpz(3)=max(imag(pzs));
          end
          ind=find(pzlimitv*2*pi*fmax/1.15>imag(pzs));
          if ~isempty(ind)
            axvpz(4)=min(pzlimitv*2*pi*fmax/1.15,max(imag(pzs(ind))));
          else axvpz(4)=min(imag(pzs));
          end
          %axvpz(1)=max(-pzlimitv*2*pi*fmax/1.15,min(real(pzs)));
          %axvpz(2)=min(pzlimitv*2*pi*fmax/1.15,max(real(pzs)));
          %axvpz(3)=max(-pzlimitv*2*pi*fmax/1.15,min(imag(pzs)));
          %axvpz(4)=min(pzlimitv*2*pi*fmax/1.15,max(imag(pzs)));
          if axvpz(1)>0, axvpz(1)=-0.1*axvpz(2); end
          if axvpz(2)<0, axvpz(2)=-0.1*axvpz(1); end
          if axvpz(3)>0, axvpz(3)=-0.1*axvpz(4); end
          if axvpz(4)<0, axvpz(4)=-0.1*axvpz(3); end
          dax=diff(axvpz);
          if dax(1)==0, dax(1)=0.1*abs(axvpz(1)); end
          if dax(3)==0, dax(3)=0.1*abs(axvpz(3)); end
          inca=0.15*[-dax(1),dax(1),-dax(3),dax(3)];
          axvpz=axvpz+inca; %increase borders
          if axvpz(1)>=axvpz(2),
            axvpz(1)=min(-10*eps,axvpz(1)*(1+eps));
            axvpz(2)=max(10*eps,axvpz(2)*(1+eps));
          end
          if axvpz(3)>=axvpz(4),
            axvpz(3)=min(-10*eps,axvpz(3)*(1+eps));
            axvpz(4)=max(10*eps,axvpz(4)*(1+eps));
          end
          if stablimit<0, axvpz(1)=min(axvpz(1),1.1*stablimit); end
          %axvpz=min(pzlimitv*2*pi*fmax,1.1*max(max(abs([rnum,rdenom]*pzsc))))...
          %                *[-1,1,-1,1];
        elseif any(domain=='zq')
          axvpz=max(1.5,min(pzlimitv,1.1*max(max(abs(pzs)))))*[-1,1,-1,1];
        end
        if length(pzaxis)==4
          ind=find(finite(pzaxis)); axvpz(ind)=pzaxis(ind);
        end
      end %pzlimit
      %subplot(1,2,2), hpz=gca; % [0.5825,0.1100,0.3175,0.8100]
      if calldrawnow, drawnow, end
      set(helis,'nextplot','add')
      if plotno==1
        hpz=axes('Position',[0.5825,0.3000,0.3175,0.5000],'parent',helis);
        %hpz=axes('Position',[0.5,0,0.5000,1000],'parent',helis);
        set(hpz,'nextplot','replace')
      else
        delete(allchild(hpz))
      end
      set(helis,'nextplot','add')
      if pzfollown>1
        plotelpz('','',1,axvpz,'notext',...
          zerosarray*pzsc,polesarray*pzsc,domain,'','','','',hpz);
        %hold on
      end
      set(hpz,'nextplot','add')
      plotelpz('','',1,[axvpz,stablimit],'nomsg',rnum*pzsc,rdenom*pzsc,domain,...
        '','','','',hpz,tauR);
      %hold off
      set(helis,'nextplot','replace')
      pause(0)
    end
    %
    %Set state of graphics at the end of the cycle
    %pop Graph window to front only if there is no control menu:
    %if strcmp(iterctrl('Iteration'),'Continue'), figure(gcf), end
    %axes(hpz) %leave axes of poles/zeros active
    %drawnow
    plottime=plottime+etime(clock,plott0); %collect plotting time
    plotdone='y'; %a plot was already done
    plotinst=clock;
    if plotmode=='o'
       %Fix ticks on plots
       fixlgtck([hamp,hph],axv)
    end
    pause(0), drawnow
    %
  else %do not plot
    cycnoplot=cycnoplot+1;
    %update lambda information:
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      if lambdadec>0
        if lambdanew>0, ltxt=sprintf('(%.0f. decr.)',lambdadec);
        else ltxt='';
        end
      else ltxt='(to be increased)';
      end
      if cycle>0,
        %axes(txth)
        if ~exist('hlambda'), hlambda=[]; 
        elseif ishandle(hlambda)&~isequal(hlambda,0), delete(hlambda)
        end
        if plotlevel~='b'
          if ishandle(helis)
            if ~exist('txthu')
              txthu=axes('Position',[0.5,0.80,0.5,0.20],'tag','elistexts',...
                'parent',helis,'visible','off');
            end
            hlambda=findobj(txthu,'tag','lambdatext');
            if isempty(hlambda)
              hlambda=text(0.05,0.975-(5*(1-ldntpos)),sprintf(['lambda=%.5g ',ltxt],lambda),...
                'VerticalAlignment','bottom','tag','lambdatext','parent',txthu);
            end
            set(hlambda,'string',sprintf(['lambda=%.5g ',ltxt],lambda))
            drawnow
          end
        end
      end
    end
  end %plots
  %
  tr=clock;
  trtime=etime(tr,t0);
  if trtime<0 %etime error or clock error on Macintosh
    if ~isempty(msgsall), fprintf(msgsall), msgsall=''; end
    msgsall=[msgsall,sprintf('Run time calculation error:\n'),...
        sprintf('   t0 = %.0f-%.2g-%.2g',t0(1),t0(2),t0(3)),...
        sprintf(', %.2g:%.2g:%.2g',t0(4),t0(5),t0(6))];
    msgsall=[msgsall,sprintf('  >  tr = %.0f-%.2g-%.2g',tr(1),tr(2),tr(3)),...
        sprintf(', %.2g:%.2g:%.2g\n',tr(4),tr(5),tr(6)),...
        sprintf('   trtime = etime(tr,t0) = %.4g s = %.2g min\n',...
            trtime,trtime/60)];
    fprintf(msgsall), msgsall='';
    earlyclock=tr;
  end
  %
  if 1==2 %This does not help in general
    trtneg=0;
    while trtime<=0
      %Attempt correction
      tr=clock; trtime=etime(tr,t0); trtneg=trtneg+1;
      if trtime<0, timeloss=etime(tr,earlyclock); end
    end
    if trtneg>0
      msgsall=[msgsall,...
          sprintf('   %.0f attempts to get positive time\n',trtneg)];
    end
    if trtneg>1, fprintf('   Time lost with waiting: %.1f s\n',timeloss), end
  end %if 1==2
  %
  if ((cycle>0)|(itmax>0))&~any(dispmesg=='n')
    msgsall=[msgsall,sprintf('   ')];
    if trtime>=0
      msgsall=[msgsall,sprintf('Total run time: %.2f min',trtime/60)];
    end
    if cycle>0
      tce=clock;
      msgsall=[msgsall,sprintf(', %.2f min/cycle',etime(tce,tc0)/cycle/60)];
    end
    if plottime>0.0051*60
      msgsall=[msgsall,sprintf(', plots: %.2f min',plottime/60)];
    end
    if (pztime>0.0051*60)&(strcmp(calcrnum,'c')|strcmp(calcrdenom,'c'))
      %msgsall=[msgsall,sprintf(', pole/zero calc: %.2f min',pztime/60)];
    end
    msgsall=[msgsall,sprintf('\n')];
    if calcpar==0
      %msgsall=[msgsall,...
      %    sprintf(['Warning! This was a covariance calculation cycle only.',...
      %    ' Parameters are not used.\n'])];
    end
  end
  if (dispmesg=='ys')&~isempty(msgsall), fprintf(msgsall), msgsall=''; end
  if calliterctrl&~isempty(get(0,'Children'))
    if calldrawnow, drawnow, end
    if strcmp(iterctrl('checked'),'Abort')
      iterctrl('Continue'); error('Abort requested through GUI')
    end
  end
  if calldrawnow, drawnow, end
  %
  if ~any(dispmesg=='n')&(cycleend==1)&(calcpar==1)
    if strcmp(itctrllastchecked,'Finish')
      msgsall=[msgsall,...
          sprintf('Iteration finishing requested through GUI menu\n')];
    elseif itmax~=0
      msgsall=[msgsall,sprintf('Iteration termination criterion is met\n')];
    end
    if (cycle>0) & (itmax>0) & (abs(dcostf)<(rcostvar)*costfnew) & ...
       ((~strcmpi(algtype,'lm')&~strcmpi(algtype,'lmsvd'))|(lambda<lambdalim))
      msgsall=[msgsall,...
          sprintf('Variation of the cost function is sufficiently small\n')];
    end
    if (cycle>0)&(itmax>0) & (maxdrp<rparvar) & ...
       ((~strcmpi(algtype,'lm')&~strcmpi(algtype,'lmsvd'))|(lambda<lambdalim))
      msgsall=[msgsall,...
        sprintf('Relative variations of parameters are sufficiently small\n')];
    end
    if cycle==itmax
      if itmax==0
        msgsall=[msgsall,...
           sprintf('itmax=0, no iteration cycle will be performed\n')];
      else
        msgsall=[msgsall,sprintf(['Number of cycles, %.0f, ',...
              'has reached allowed maximum\n'],cycle)];
      end
    end
    if (2*costfnew<chi05pr)&~any(dispmesg=='nsc')
      msgsall=[msgsall,...
          sprintf(['WARNING: too small cost function.\n   Expected: %.1f, ',...
             '5%%%%-95%%%% percentiles: (%.3g, %.3g)\n'],...
             N/2,chi05pr/2,chi95pr/2),...
      sprintf('   Possible cause: wrong (too large) variances\n')];
    end
  end %dispmesg
  if ~any(dispmesg=='nc')&~isempty(msgsall), fprintf(msgsall), msgsall='';
  elseif dispmesg=='c', msgsall='';
  end
  %
  if any(~finite([num,denom,delay])) | ...
         ( (cycleend==1) & ((calcpar==0)|(CRreq==0)) )
    %break follows
    %cycleend=1: iteration criterion met
    if ~strcmpi(algtype,'svd')&~strcmpi(algtype,'lmsvd')
      condinumJ=cond(J);
      if strcmpi(algtype,'ng')
        condinum=condinumJ;
      elseif strcmpi(algtype,'nr')
        if ~isempty(d2ExE)
          condinum=cond(J'*J+d2ExE);
        else %true Newton-Gauss instead of Newton-Raphson
          condinum=condinumJ;
        end
      elseif strcmpi(algtype,'lm')
        if lambda==0
          condinum=condinumJ;
        else
          condinum=cond(JTJ+lambdaold*norm(J,'fro')^2*eye(lJTJ,lJTJ));
        end
      end
      if condinum>1/(N+1)/eps
        msgsall=[msgsall,...
          sprintf('WARNING: Equations badly conditioned in elis: cn=%.5g\n',...
            condinum)];
      end
    end %svd or lmsvd
    %
    if ~any(dispmesg=='nc')&~isempty(msgsall), fprintf(msgsall), msgsall=''; end
    if termcond==4
      disp('elis run finished following command from GUI')
    end
    break %stop iteration
  else %don't stop yet
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      if lambda>0, lambdaold=lambda; end
      if (cycle>0)&(cycleend==0), lambda=lambdanew; end
    end %LM
  end %cycleend
  %
  pause(0) %try to allow button push in another window
end %for cycle, main loop
if calcpar==0, cycle=cycle-1; end
tce=clock;
%
%clear uvexd y1m y1mcexd daridp dbdp dndp dddp rn rd pcyc
%clear polesarray zerosarray
%clear dEdd dEddr dbdd dndd1 dndd2 d2ExEv d2ExEd multerm db2part dbdp2m
%clear db2ext dbddel JTJ U S V invS diaginvS Esmr
if nargoutelis>=2, cfv=cfv(1:cycle+1,:); end
%
%---------------------------------------------
%Prepare covariance data
if d2ExEreq==1 %covar matrix requested
  if ~isempty(d2ExE)
    [U,S,V]=svd(J'*J+d2ExE,0);
  else error('This is an obsolete branch') %[U,S,V]=svd(J,0);
  end
  invS=abs(S);
  diaginvS=diag(invS);
  maxeig=max(diag(invS));
  [mineig,ind]=min(diaginvS);
  if nfn0==0 %all parameters free
    diaginvS(ind(1))=[]; mineig=min(diaginvS);
  end
  condinumd2Cdp2=maxeig/mineig;
  messnoC=0;
  for kind=1:length(invS(1,:))
    if invS(kind,kind) < maxeig*eps*length(invS(1,:)) %bad conditioning
      if (nfn0>0)|(kind~=ind(1))
        invS(kind,kind)=0;
        messnoC=messnoC+1;
        if messnoC==1
          msgsall=[msgsall,...
              sprintf(['WARNING: d^2C/dP^2 is badly conditioned in elis:\n',...
                  '   covariance matrix may be unreliable\n']),...
              sprintf('   Condition number: %.3e, 1/eps=%.3e\n',...
                  condinumd2Cdp2,1/eps),...
              sprintf('   Deficiency: %.0f\n   You may have to ',...
                      length(invS(1,:))-kind-sign(nfn0))];
          if ~any(domain=='pq')
            msgsall=[msgsall,sprintf('use orthogonal polynomials\n   or ')];
          end
          msgsall=[msgsall,sprintf('use the Cramer-Rao matrix instead,\n')];
          if any(domain=='spw')
            msgsall=[msgsall,sprintf(['   or change the scaling frequency ',...
                     'to improve the cn,\n'])];
            fssug=[];
            if strcmp(calcrnum,'c')&strcmp(calcrdenom,'c')&...
                   ~isempty([rnum(:);rdenom(:)])
              if any(domain=='spr')
                fssug=mean(abs([rnum(:);rdenom(:)]))*fs;
              elseif strcmp(domain,'w')
                fssug=mean(abs([rnum(:);rdenom(:)]))^2*fs;
              end
            else
              if ~any(domain=='pq')
                pdat=exppar(domain,num,denom,[],[],[],[],[],[],[],[],[],coeffs,tauR);
                fssug=fs*pdat(2);
              end
            end
            if isempty(fssug), fssug=fs; end
            if abs(fssug-fs)/fs>0.1
              msgsall=[msgsall,...
                sprintf('     suggestion: fs=%.2e -> %.2e rad*Hz,\n',fs,fssug)];
            end
          end
          msgsall=[msgsall,...
              sprintf('   or collect new data in a broader band...\n')];
        end %messages
      else %The first singular value is zero for norm=1 solution
        invS(kind,kind)=0; %pseudoinverse
      end
    elseif (kind<length(invS(1,:)))|(nfn0>0) %proper reciprocal
      invS(kind,kind)=1/S(kind,kind);
    else %The last singular value is too large
      fprintf(['WARNING! The condition number is smaller than ',...
               '1/eps/freepar\n  in the calculation of Cp:',...
               ' %.3e < %.3e\n'],...
               maxeig/invS(kind,kind),1/eps/length(invS(1,:)))
      fprintf(['  The reciprocal of the smallest singular value is set',...
               ' to zero\n'])
      invS(kind,kind)=0;
    end %if invS
  end %for kind
  %
  Cpint=V*(invS*(U'*(J'*J)*U)*invS)*V';
  %
  nCpint=norm(Cpint,'fro');
  Qm1=V*invS*U';
  Cpint2=Qm1'*(J'*J)*Qm1;
  %clear Qm1
  Cptmp=J*U*invS*V'; Cpint3=Cptmp'*Cptmp;
  if any(abs(Cpint2-Cpint)>prod(size(J))*nCpint*eps)...
      |any(abs(Cpint3-Cpint)>prod(size(J))*nCpint*eps)
    fprintf('WARNING! The inverse of the appr. covariance matrix is unreliable\n')
    fprintf('  norm(Cpint) = %.3e, size(J) = (%.0f,%.0f)\n',nCpint,...
          size(J,1),size(J,2))
    fprintf('  max(abs(Cpinti-Cpint)) = %.3e, %.3e\n',...
          max(max(abs(Cpint2-Cpint))),max(max(abs(Cpint3-Cpint))) )
    eigCpint=eig(Cpint); eigCpint2=eig(Cpint2);  eigCpint3=eig(Cpint3);
    fprintf('  Eigenvalues of the internal Cp matrices (Cpint, Cpint2, Cpint3):\n')
    eigCp1=eig(Cpint); [dummy,ind1]=sort(-abs(eigCp1));
    eigCp2=eig(Cpint2); [dummy,ind2]=sort(-abs(eigCp2));
    eigCp3=eig(Cpint3); [dummy,ind3]=sort(-abs(eigCp3));
    fprintf('  %+.4e, %+.4e, %+.4e\n',[eigCp1(ind1),eigCp2(ind2),eigCp3(ind3)]')
  end
  %
  Cpint=(Cpint+Cpint')/2;
  eigCpint=eig(Cpint);
  if any(eigCpint<-max(abs(eigCpint))*eps*length(d2ExE))
    disp('WARNING! Cp is unusable (negative eigenvalue)')
    fprintf('  Max-min eigenvalues: %+.4e %+.4e\n',max(eigCpint),min(eigCpint))
    fprintf('  Eigenvalues:\n')
    fprintf('    % .4e\n',eigCpint)
    Cpint=NaN*ones(size(Cpint));
  end
  %
  if calldrawnow, drawnow, end
%%%%%%%%%%%%%%%%%%%%%%%%%%
end
%%%%%%%%%
%  if isempty(d2ExE)
%    CRint=Cpint;
%  elseif CRreq==1 %CR requested
%%%%%%%%%
if CRreq==1
  if CRreq==1
%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [U,S,V]=svd(J,0);
    %
    if (delaytreat=='v')%&(delayscale~=1)
    %test for delay scaling
      Js2=size(J,2);
      SJm1=svd(J(:,1:Js2-1),0);
      if nfn0>0, S0=0; else S0=1; end
      diagS=diag(S);
      %indS=pairs(log(SJm1(1:Js2-1-S0)+realmin),log(diagS(1:Js2-S0)+realmin),10);
      indS=pairs(SJm1(1:Js2-1-S0),diagS(1:Js2-S0),1);
      Sdind=find(diff([0,sort(indS'),Js2-S0+1])>1);
      Srest=diagS([1:Sdind-1,Sdind+1:Js2-1-S0]);
      dscdisptxt=0;
      if dscdisptxt %test display lines
        fprintf('size(J): (%.0f,%.0f)',size(J,1),size(J,2))
        fprintf(', cond(J) = %.3e, cond(Jm1) = %.3e\n',...
               diagS(1)/diagS(Js2-S0),SJm1(1)/SJm1(Js2-1-S0))
        fprintf('Singular value for delay: %.3e, extremes: %.3e, %.3e\n',...
            diagS(Sdind),min(Srest),max(Srest))
      end
      if dscdisptxt|( ((diagS(Sdind)<min(Srest))|(diagS(Sdind)>max(Srest)))&...
              ((delayscale~=1)|(max(diagS)*eps*100>diagS(length(diagS)-S0))) )
        fprintf('   Optimal delay scaling: %.3e, presently: %.3e\n',...
               delayscale*sqrt(max(Srest)*min(Srest))/diagS(Sdind),delayscale);
      end
    end
    %
    invS=abs(S);
    diaginvS=diag(invS);
    maxeig=max(diag(invS));
    [mineig,ind]=min(diaginvS);
    if nfn0==0 %all parameters free
      diaginvS(ind(1))=[]; mineig=min(diaginvS);
    end
    dst=dbstack; cl=1;
    for ii=1:length(dst)
      if any(findstr(dst(ii).name,'sme.m')), cl=0; end
    end
    if cl, try, denom=fixden(fidmodel,denom); end, end
    if any(isnan(denom)), error('denom contains NaN'), end
    condinumJ=maxeig/mineig;
    messnoJ=0;
    for kind=1:length(invS(1,:))
      if invS(kind,kind) < maxeig*eps*length(invS(1,:)) %bad conditioning
        if (nfn0>0)|(kind~=ind(1))
          invS(kind,kind)=1/S(kind,kind);
          messnoJ=messnoJ+1;
          if messnoJ==1
            if kind+sign(nfn0)<max(size(invS))
               msgsall=[msgsall,sprintf(['WARNING: Rank of J is deficient:',...
                   ' svd has found %.0f small eigenvalue(s)\n'],...
                   max(size(invS))-kind-sign(nfn0))];
            end
            msgsall=[msgsall,...
                  sprintf('   Condition number: %.3e, 1/eps=%.3e\n',...
                        condinumJ,1/eps),...
                  sprintf('   Deficiency: %.0f\n',...
                        length(invS(1,:))-kind-sign(nfn0))];
            if any(domain=='sp')
              msgsall=[msgsall,sprintf('   You may try to ')];
              if ~any(domain=='pq')
                msgsall=[msgsall,sprintf('use orthogonal polynomials\n   or ')];
              end
              msgsall=[msgsall,sprintf(['change the scaling frequency ',...
                  'to improve the cn,\n'])];
              fssug=[];
              if strcmp(calcrnum,'c')&strcmp(calcrdenom,'c')&...
                     ~isempty([rnum(:);rdenom(:)])
                if any(domain=='spr')
                    fssug=mean(abs([rnum(:);rdenom(:)]))*fs;
                elseif strcmp(domain,'w')
                    fssug=mean(abs([rnum(:);rdenom(:)]))^2*fs;
                end
              else
                if ~any(domain=='pq')
                  pdat=exppar(domain,num,denom,[],[],[],[],[],[],[],[],[],coeffs,tauR); fssug=fs*pdat(2);
                end
              end
              if isempty(fssug), fssug=fs; end
              if abs(fssug-fs)/fs>0.1
                msgsall=[msgsall,...
                    sprintf('     suggestion: fs=%.2e -> %.2e rad*Hz,\n',...
                      fs,fssug)];
              end
              msgsall=[msgsall,...
                  sprintf('   or collect new data in a broader band...\n')];
            else
              msgsall=[msgsall,...
                  sprintf(['   You may have to collect new data in a ',...
                    'broader band...\n'])];
            end
          end
        else %The first singular value is zero for norm=1 solution
          invS(kind,kind)=0; %pseudoinverse
        end
      elseif (kind<length(invS(1,:)))|(nfn0>0) %proper reciprocal
        invS(kind,kind)=1/S(kind,kind);
      else %The last singular value is too large
        fprintf(['WARNING! The condition number is smaller than ',...
               '1/eps/freepar\n  in the calculation of CR:',...
               ' %.3e < %.3e\n'],...
               maxeig/invS(kind,kind),1/eps/length(invS(1,:)))
        fprintf(['  The reciprocal of the smallest singular value is set',...
               ' to zero\n'])
        invS(kind,kind)=0;
      end %if invS
    end %for kind
    CRint=V*(invS.*invS)*V';
  end
  CRint=(CRint+CRint')/2;
  %%%%%%%
  Cpint=CRint;
  %%%%%%%
  %Cpint and CRint are the covariance matrices of the estimated parameters 
  %
  if any(diag(Cpint)<0)
    msgsall=[msgsall,sprintf(['WARNING: Covariance values are unreliable ',...
        'in elis (negative variance)\n'])];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
  end
  if any(any(imag(Cpint))) %complex value
    msgsall=[msgsall,...
        sprintf('WARNING: Complex element in covariance matrix\n')];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
    msgsall=[msgsall,sprintf('Elements of Cp changed to NaN-s by elis\n')];
    Cpint=0*Cpint+NaN;
  end
  if ~all(all(finite(Cpint)))
    msgsall=[msgsall,sprintf(['WARNING: Covariance values are unreliable ',...
        'in elis (infinite or NaN value)\n'])];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
  end
  if any(diag(CRint)<0)
    msgsall=[msgsall,...
        sprintf(['WARNING: Cramer-Rao values are unreliable in elis ',...
          '(negative variance)\n'])];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
  end
  if any(any(imag(CRint))) %complex value
    msgsall=[msgsall,...
        sprintf('WARNING: Complex element in Cramer-Rao matrix\n')];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
    msgsall=[msgsall,sprintf('Elements of CR changed to NaN-s by elis\n')];
    CRint=0*CRint+NaN;
  end
  if ~all(all(finite(CRint)))
    msgsall=[msgsall,...
        sprintf(['WARNING: Cramer-Rao values are unreliable in elis ',...
        '(infinite or NaN value)\n'])];
    msgsall=[msgsall,...
        sprintf('Probable cause: badly conditioned normal equations\n')];
  end
  %
  if paramtreat=='a' %allpass
    %First save delay related numbers
    if delaytreat=='v'
      lCpint=length(Cpint);
      cvd=Cpint(lCpint,:); Cpint(lCpint,:)=[]; Cpint(:,lCpint)=[];
      if ~isempty(CRint)
        cvdCR=CRint(lCpint,:); CRint(lCpint,:)=[]; CRint(:,lCpint)=[];
      end
    end %delaytreat
    %
    %Now generate full matrices
    if domain=='z'
      Cpint=[Cpint,fliplr(Cpint)]; Cpint=[Cpint;flipud(Cpint)];
      CRint=[CRint,fliplr(CRint)]; CRint=[CRint;flipud(CRint)];
    else
      Cpint=[Cpint,Cpint]; Cpint=[Cpint;Cpint];
      CRint=[CRint,CRint]; CRint=[CRint;CRint];
    end
    %
    %Add back delay-related info
    if delaytreat=='v'
      if domain=='z'
        cvd=[cvd(1:end-1),fliplr(cvd(1:end-1)),cvd(end)];
        Cpint=[Cpint,cvd(1:end-1)';cvd];
        if ~isempty(CRint)
          cvdCR=[cvdCR(1:end-1),fliplr(cvdCR(1:end-1)),cvdCR(end)];
          CRint=[CRint,cvdCR(1:end-1)';cvdCR];
        end
      else %'s' or 'r'
        cvd=[cvd(1:end-1),cvd(1:end-1),cvd(end)];
        Cpint=[Cpint,cvd(1:end-1)';cvd];
        if ~isempty(CRint)
          cvdCR=[cvdCR(1:end-1),cvdCR(1:end-1),cvdCR(end)];
          CRint=[CRint,cvdCR(1:end-1)';cvdCR];
        end
      end
    end
  end %allpass
  if any(domain=='sw') %frequency scaling is necessary
    if any(domain=='s')
      scv=[fs.^(numord-numnotfixind)';fs.^(denomord-denomnotfixind)'];
    elseif strcmp(domain,'w')
      scv=sqrt([fs.^(numord-numnotfixind)';fs.^(denomord-denomnotfixind)']);
    end
    if transtreat=='t'
      if any(domain=='s')
        scv=[scv;fs.^[trord:-1:0]'];
      elseif strcmp(domain,'w')
        scv=sqrt([scv;fs.^[trord:-1:0]']);
      end
    end
    if delaytreat=='v', scv=[scv;fs/delayscale]; end
    Cpint=Cpint./(scv*scv');
    if ~isempty(CRint), CRint=CRint./(scv*scv'); end
  end %any(domain=='sw')
end %d2ExEreq==1
if ~any(dispmesg=='nc')&~isempty(msgsall), fprintf(msgsall), msgsall='';
elseif dispmesg=='c', msgsall='';
end
%
if calldrawnow, drawnow, end
%-------------------------
%Save covariance matrix
fixpind=[numfixind+1,denomfixind+numord+2]';
if strcmp(delaytreat,'f'), fixpind=[fixpind;numord+denomord+3]; end
if Cpreq|CRreq
  realmini=realmin;
  %
  %Generate CR
  if ~isempty(CRint)
    ind0=find(diag(CRint)==0);
    if ~isempty(ind0), CRint=CRint+realmini*eye(size(CRint)); end
    [cdatint,CR]=expcov(CRint,fixpind);
    if ~isempty(M)
      if M<=3, CR=NaN*CR; 
      else CR=CR*(M-1)/(M-3);
      end
    end
    %stdv=sqrt(diag(CR));
  end
  %
  %Generate Cp
  if Cpreq>0
    ind0=find([diag(Cpint);1]==0);
    if ~isempty(ind0), Cpint=Cpint+realmini*eye(size(Cpint)); end
  end
  if ~isempty(Cpint)
    if ~strcmp(call,'new')&(nargoutelis>=3)
      [cdatint,Cp]=expcov(Cpint,fixpind);
    end
    if ~isempty(cfile), expcov(Cpint,fixpind,cfile); end
  end
  %
end %Cpreq
%
%---------------------------------------------
%Save results to parameter file
freqsh=freq(1:length(freq)/length(expi));
if nargoutelis>=0
  if strcmp(Fdatin,'object') %object-based call
    if any(domain=='zq'), domainm='z^-1'; else domainm=domain; end
    pdat=fidmodel(domainm,num,denom,delay*delayscale,fs,[],[],...
      trnum,Znum,Zdenom,Znumt,freqsh*fs,'Coefficients',coeffs,'tauR',tauR);
    if ~isempty(Fdat.outputname)
      set(pdat,'outputname',Fdat.outputname,'noconsistency')
    end
    if ~isempty(Fdat.inputname)
      set(pdat,'inputname',Fdat.inputname,'noconsistency')
    end
    if any(domain=='zq')&strcmp(get(Fdat,'InputCharacter'),'ZOH')
      set(pdat,'InterSample','ZOH');
    end
    %
  else %old call
    pdat=exppar(domain,num,denom,delay*delayscale,fs,Znum,Zdenom,pfile,...
       '','',trnum,Znumt,coeffs,tauR);
  end
elseif ~isempty(pfile)
  %This is the only command which generates pfile:
  if ~isempty(pfile)
    exppar(domain,num,denom,delay*delayscale,fs,Znum,Zdenom,pfile,...
      '','',trnum,Znumt,coeffs,tauR);
  end
end
%
%Information on the fit
tfav=mean(abs(Nf./Df));
outprvar=(abs(Nf./Df).^2).*varx+vary;
if ~isempty(covxy)
  outprvar=outprvar-2*real(covxy.*conj(exd.^2).*conj(Nf./Df));
end
ovav=mean( outprvar./abs(uv).^2 );
mmerror=real(sqrt((2*costfnew-N)/F*(ovav)));
%clear exd uv outprvar U S V
%
fiti=zeros(19,1);
fiti(1)=costfnew; %cost function
fiti(2)=N/2*Mm; %theoretical value of the cost function (chi^2/2)
fiti(3)=chi95pr/2; %95% point of theoretical distribution of cf
fiti(4)=F; %number of frequencies
fiti(5)=Nfp; %number of free parameters
fiti(6)=cycle; %performed iterations
fiti(7)=dcostf; %last change of the cost function
fiti(8)=maxdrp; %last maximum relative change of parameters
if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd'), fiti(9)=lambda;
else fiti(9)=NaN;
end %lambda value (Levenberg-Marquardt)
fiti(10)=mmerror; %approximate mean model error
fiti(11)=tfav; %mean absolute value of the estimated transfer function
%old: AIC=costfnew+Nfp; %Usual AIC/2
%  if nfn0==0, AIC=AIC-1; end %correction because of norm=1 constraint
AIC=costfnew*(1+Nfp/F);
if nfn0==0, costfnew*(1+(Nfp-1)/F); end %correction because of norm=1 constraint
if isempty(varx)|~any(varx), lnf=log(2*F); else lnf=log(4*F); end
MDL=costfnew*(1+Nfp/(2*F)*lnf);
fiti(12)=AIC; %Akaike Information Criterion (AIC)
fiti(13)=condinum; %condition number of decomposed or inverted matrix
fiti(14)=condinumJ; %condition number of J
fiti(15)=condinumd2Cdp2; %condition number of Q=d^2C/dp^2
fiti(16)=fs; %scaling (angular) frequency
fiti(17)=termcond; %cause of termination
fiti(18)=trtime;
fiti(19)=MDL;
%
if strcmp(Fdatin,'object') %object-based call
  fits.cf=fiti(1);
  fits.cfth=fiti(2);
  fits.cf95=fiti(3);
  fits.cf05=chi05pr/2;
  fits.F=fiti(4);
  fits.freepar=fiti(5);
  fits.iterations=fiti(6);
  fits.dcf=fiti(7);
  fits.dprel=fiti(8);
  fits.lambda=fiti(9);
  fits.mmerror=fiti(10);
  fits.meanabstf=fiti(11);
  fits.AIC=fiti(12);
  fits.MDL=MDL;
  fits.condnum=fiti(13);
  fits.condnumJ=fiti(14);
  %fits.condnumQ=fiti(15);
  fits.fscale=fiti(16);
  termcauses={'itmax','dcf','dp','GUI'};
  fits.termcause=termcauses{fiti(17)};
  fits.runtime=fiti(18);
  algstruc.iteration=algstr;
  algstruc.initset=issset(initset,domain);
  algstruc.initdelay=algorithm.initdelay;
  set(pdat,'algorithm',algstruc,'noconsistency');
  if cl, try, clm(pdat), catch, end, end
  try
    if ~exist('CR'), CR=[]; end
    if strncmpi(errorweighting,'nonlinear',4), fits.errorweighting='Nonlinear';
    elseif strncmpi(errorweighting,'interpolated',6), fits.errorweighting='Interpolated nonlinear'; 
    else
      fits.errorweighting='Linear';
      %set(Fdat,'nonlinM',[],'nonlincovariance',[]);
    end
    set(pdat,'covariance',CR,'data',Fdat)
  catch, end 
  set(pdat,'fitinfo',fits,'noconsistency');
end
if strcmp(call,'new')
  %Set second output (information on the numerics)
  if ~exist('cfv'), cfv=[]; end
  fit.cfv=cfv;
  fit.J=J;
  fit.w=w;
else
  fit=fiti; %pass to outout
end
%
if ~isempty(rfile) %report file requested
  if calldrawnow, drawnow, end
  tr0=clock;
  if ~exist('Cp'), Cp=[]; end
  rfile=fnamanal(rfile,'rep');
  if ~any(dispmesg=='nc')
    fprintf(['Report file ',rfile,' is being prepared ...\n'])
  end
  if exist(rfile)==2, delete(rfile), end
  if exist(rfile)==2
    disp(['WARNING: Cannot delete existing report file ',rfile])
  end
  %
  rfid=fopen(rfile,'w');
  fprintf(rfid,'FREQUENCY DOMAIN SYSTEM IDENTIFICATION TOOLBOX FOR MATLAB\n');
  fprintf(rfid,'ELiS RUN PROTOCOL');
  clv=clock;
  fprintf(rfid,[', date and time: ',date]);
  fprintf(rfid,', %.0f:%02.0f:%02.0f\n',clv(4),clv(5),clv(6));
  fprintf(rfid,['Report file: ',rfile,'\n']);
  if ~isempty(rpf), fprintf(rfid,['Run parameter file: ',rpf]);
  else fprintf(rfid,'Default run parameters');
  end
  if narginelis>1, fprintf(rfid,', modified in command line'); end
  fprintf(rfid,'\n\n');
  if ~isempty(ffile)&isstr(ffile)
    fprintf(rfid,['Fourier file with complex amplitudes to be fitted: ',...
         ffile,'\n']);
  else
    fprintf(rfid,'Fourier data given in command line\n');
  end
  fprintf(rfid,'Experiments:');
  fprintf(rfid,' %.0f',length(expi));
  if length(expi)==1, fprintf(rfid,', ser. no. %.0f',expi); end
  if expno>1
    fprintf(rfid,'\nNumber of frequencies: %.0f = %.0f x %.0f\n',...
        F,expno,F/expno);
  else
    fprintf(rfid,'\nNumber of frequencies: %.0f\n',F);
  end
  if strcmp(vardef,'f')
    fprintf(rfid,['Variances taken from file ',vfile,'\n']);
  elseif ~( any(diff(varx))|any(diff(vary)) )
    fprintf(rfid,'Input and output variances:\n');
    fprintf(rfid,'   %.5g   %.5g\n',varx(1),vary(1));
  else
    fprintf(rfid,'Variance data given\n'); end
  if ~isempty(covxy)
    fprintf(rfid,'Input-output covariances are taken into account\n');
    if all(covxy==0)
      fprintf(rfid,'   Covariance values are all equal to zero\n');
    elseif ~any(diff(covxy))
      fprintf(rfid,'   covxy is constant: %.3g + j*%.3g\n',...
        real(covxy(1)),imag(covxy(1)));
    end
  else
    fprintf(rfid,'Input-output covariances are not given\n');
  end
  if any(domain=='spw')
    fprintf(rfid,['Fit in ',domain,'-domain, frequencies normalized ',...
        'internally\n   by omegasc = %.5g rad*Hz\n'],fs);
    if strcmp(calcrnum,'c')&strcmp(calcrdenom,'c')
      fssug=[];
      if ~isempty([rnum(:);rdenom(:)])
        if any(domain=='sp')
          fssug=mean(abs([rnum(:);rdenom(:)]))*fs;
        elseif strcmp(domain,'w')
          fssug=mean(abs([rnum(:);rdenom(:)]))^2*fs;
        end
      end
      if isempty(fssug), fssug=fs; end
    else
      if ~any(domain=='pq')
        pdat=exppar(domain,num,denom,[],[],[],[],[],[],[],[],[],coeffs,tauR); 
        fssug=fs*pdat(2);
      end
    end
    if ~isempty(fssug)
      fprintf(rfid,'   suggestion: omegasc = %.5g rad*Hz\n',fssug);
    end
  elseif any(domain=='zq')
    fprintf(rfid,['Fit in z-domain, sampling frequency: fs = %.5g Hz\n'],fs);
  end
  if delayscale~=1
    fprintf(rfid,'Extra internal scale factor for the delay: %.4g\n',delayscale);
  end
  if any(domain=='pq'), fprintf(rfid,'Orthogonal polynomials are used'); end
  fprintf(rfid,'Orders: %.0f/%.0f\n',numord,denomord);
  if strcmp(paramtreat,'a'), fprintf(rfid,'Allpass design\n'); end
  if nfn0==0
    fprintf(rfid,['No fixed nonzero parameters (norm=1 solution)\n']);
  end
  if transtreat=='t', fprintf(rfid,'Transients eliminated\n'); end
  if strcmp(paramtreat,'0')
    fprintf(rfid,['One fixed parameter:\n   coefficient of ',...
        domain,'^0 in the denominator is set to 1.0\n']);
  elseif length([numfix(:);denomfix(:)])>0 %some fixed parameters
    fprintf(rfid,'Fixed parameters:\n');
    if any(domain=='zq'),
      fprintf(rfid,['  (Coefficients from 0 to order, in ascending ',...
         'powers of z^-1,)\n']);
    else %swp
      fprintf(rfid,['  (Coefficients from 0 to order, in descending ',...
         'powers of ',domain,')\n']);
    end
    fprintf(rfid,['   Numerator (order=',int2str(numord),'):\n']);
    if any(domain=='sp'), cfs=fs; elseif strcmp(domain,'w'), cfs=sqrt(fs); end
    for k=1:length(numfixind)
      if any(domain=='spw')
        k=length(numfixind)+1-k;
        fprintf(rfid,['      ',domain,'^%.0f:  b(%.0f)=%5.3e\n'],...
                numord-numfixind(k),numord-numfixind(k),...
                numfix(k)/cfs^(numord-numfixind(k)));
      else %domain='z'
        fprintf(rfid,'      z^(-%.0f):  b(%.0f)=%5.3e\n',...
                numfixind(k),numfixind(k),numfix(k));
      end
    end %for k
    fprintf(rfid,['   Denominator (order=',int2str(denomord),'):\n']);
    if ~strcmp(paramtreat,'a')
      for k=1:length(denomfixind)
        if any(domain=='spw')
          k=length(denomfixind)+1-k;
          fprintf(rfid,'      s^%.0f:  a(%.0f)=%5.3e\n',...
                denomord-denomfixind(k),denomord-denomfixind(k),...
                denomfix(k)/cfs^(denomord-denomfixind(k)));
        elseif any(domain=='zq')
          fprintf(rfid,'      z^(-%.0f):  a(%.0f)=%5.3e\n',...
                 denomfixind(k),denomfixind(k),denomfix(k));
        end
      end %for k
    else %allpass
      fprintf(rfid,'      Allpass: reverse of numerator\n');
    end
  end %paramtreat
  %
  if strcmp(delaytreat,'f')
    fprintf(rfid,'Fixed value of the delay: ');
  else
    fprintf(rfid,'Starting value of the variable delay: ');
  end
  if any(domain=='spw')
    fprintf(rfid,' %.5g s\n',delayinit*delayscale/fs);
  else
    fprintf(rfid,[' %.',num2str(fis),'g samples\n'],delayinit*delayscale);
  end
  %
  if itmax>0
    fprintf(rfid,'Algorithm: ');
    if strcmpi(algtype,'ng')
      fprintf(rfid,'Newton-Gauss');
    elseif strcmpi(algtype,'nr')
      fprintf(rfid,'Newton-Raphson');
    elseif strcmpi(algtype,'lm')
      fprintf(rfid,'Levenberg-Marquardt');
    elseif strcmpi(algtype,'lmsvd')
      fprintf(rfid,'Levenberg-Marquardt with singular value decomposition');
    elseif strcmpi(algtype,'svd')
      fprintf(rfid,'Singular value decomposition');
    end
    if strcmp(searchmode,'l'), fprintf(rfid,', with line search'); end
  end
  if strcmp(stabmode,'r')
    fprintf(rfid,'Poles stabilized by reflection\n');
  elseif strcmp(stabmode,'c')
    fprintf(rfid,'Poles stabilized by constraction to stablimit\n');
  end
  fprintf(rfid,'\nStarting value setting: ');
  if strcmp(initset,'l')
    fprintf(rfid,'linear least squares');
  elseif strcmp(initset,'w')
    fprintf(rfid,'weighted least squares (by variance of ym)');
  elseif strcmp(initset,'v')
    fprintf(rfid,'y-weighted svd (by variance reduced to output)');
  elseif strcmp(initset,'u')
    fprintf(rfid,'u-weighted svd (by variance reduced to input)');
  elseif strcmp(initset,'a')
    fprintf(rfid,'weighted least squares (by approximate ML weight)');
  elseif strcmp(initset,'r')
    fprintf(rfid,'weighted least squares (by eq error variance)');
  elseif strcmp(initset,'y')
    fprintf(rfid,'WLS, by singular value decomposition');
  elseif strcmp(initset,'s')
    fprintf(rfid,'singular value decomposition');
  elseif strcmp(initset,'i')
    fprintf(rfid,'iqml, calculated from given model');
  elseif strcmp(initset,'x')
    fprintf(rfid,'WLS, using externally given weights');
  elseif strcmp(initset,'f')
    if isstr(initpfile)
      fprintf(rfid,['parameter file ',initpfile]);
    else %vector
      fprintf(rfid,'given model');
    end
  elseif strcmp(initset,'e')
    fprintf(rfid,['equation error method ',...
          '\n   (Signal Processing Toolbox: invfreq',domain,')']);
  end
  fprintf(rfid,'\n\n');
  %
  fprintf(rfid,'Allowed maximum number of iterations: %.0f',itmax);
  if itmax==0, cycp=0; else cycp=cycle; end
  fprintf(rfid,', iterations performed: %.0f\n',cycp);
  if strcmp(itctrllastchecked,'Finish')
    fprintf(rfid,'Iteration termination requested through GUI menu\n');
  end
  trtime=etime(clock,t0);
  fprintf(rfid,'Total run time: %.2f min',trtime/60);
  if cycle>0
    fprintf(rfid,', %.2f min/cycle',etime(tce,tc0)/cycle/60);
  end
  if plottime>0.0051*60, fprintf(rfid,', plots: %.2f min\n',plottime/60);
  else fprintf('\n')
  end
  if (pztime>0.0051*60)&(strcmp(calcrnum,'c')|strcmp(calcrdenom,'c'))
    fprintf(rfid,'   time used for pole/zero calculations: %.2f min\n',...
        pztime/60);
  end
  if cycle>0
    fprintf(rfid,['Stop if relative change of cost function is smaller',...
        ' than %.2e\n'],rcostvar);
    if costfold>0
      fprintf(rfid,'   last relative variation: %+.2e\n',dcostf/costfold);
    end
    if ~isnan(maxdrp)
      fprintf(rfid,['Stop if maximum relative change of parameters is ',...
          'smaller than %.2e\n'],rparvar);
      fprintf(rfid,'   last max. rel. variation: %+.2e\n',maxdrp);
    end
    if strcmpi(algtype,'lm')|strcmpi(algtype,'lmsvd')
      fprintf(rfid,'Starting and last values of lambda: %.5g, %.5g\n',...
        lambda0,lambda);
      fprintf(rfid,'   maximum value of lambda, to allow stopping: %.5g\n',...
         lambdalim);
    end
  end
  %
  fprintf(rfid,['Condition number of the decomposed or inverted matrix:',...
                 '\n                                 %.5g'],condinum);
  if finite(condinum)
    fprintf(rfid,', reciprocal: %.5g\n',1/condinum);
  else
    fprintf(rfid,'\n');
  end
  fprintf(rfid,'Condition number of J:           %.5g',condinumJ);
  if finite(condinumJ)
    fprintf(rfid,', reciprocal: %.5g\n',1/condinumJ);
  else
    fprintf(rfid,'\n');
  end
  if ~isempty(Cp)
    fprintf(rfid,'Condition number of Q=d^2C/dp^2: %.5g',condinumd2Cdp2);
    if finite(condinumd2Cdp2)
      fprintf(rfid,', reciprocal: %.5g\n',1/condinumd2Cdp2);
    else
      fprintf(rfid,'\n');
    end
  end
  fprintf(rfid,[  '                                 ',...
                '                   eps = %.5g\n'],eps);
  fprintf(rfid,'Value of the cost function: %.5g, its double: %.5g\n',...
                costfnew,2*costfnew);
  fprintf(rfid,'Theoretical value of the cost function: %.1f\n',N/2);
  if isequal(Mm,1)
    fprintf(rfid,['Degrees of freedom of the chi-square value (2*cfth):'...
        ' %.0f\n'],N);
  end
  fprintf(rfid,['5%%-95%% points of the theoretical distribution of the',...
                ' cf:\n   %.5g, %.5g\n'],chi05pr/2,chi95pr/2);
  fprintf(rfid,'Approximate mean model error: %.5g',abs(mmerror));
  if imag(mmerror)~=0, fprintf(rfid,'*j');
     if 2*costfnew<chi05pr, fprintf(rfid,' (too small cost function!)'); end
  end, fprintf(rfid,'\n');
  fprintf(rfid,'Mean absolute value of the transfer function: %.5g\n',tfav);
  fprintf(rfid,'Approximate relative mean model error: %.5g',...
        abs(mmerror)/tfav);
  if imag(mmerror)~=0, fprintf(rfid,'*j'); end, fprintf(rfid,'\n');
  fprintf(rfid,'Akaike information criterion (AIC): %.5g\n',AIC);
  fprintf(rfid,'Number of free parameters: %.0f\n',Nfp);
  fprintf(rfid,'\nValues of the parameters');
  if ~isempty(Cp)
    fprintf(rfid,[' (with standard deviations,',...
                 ' calculated from\n   the approximate covariance matrix)']);
  end
  fprintf(rfid,':\n');
  if ~any(domain=='pq')
    pdati=exppar(domain,num,denom,delay*delayscale,fs,[],[],[],[],[],[],[],coeffs,tauR); %non-scaled vector
    [domain,numns,denomns]=imppar(pdati,1);
  else %p-domain
    pdati=exppar(domain,num,denom,delay*delayscale,fs,Znum,Zdenom,[],[],[],[],[],coeffs);
    numns=fliplr(num); denomns=fliplr(denom);
  end
  pdati(1:4)=[];
  if any(domain=='spw') man=[domain,'^']; else man='z^-'; end
  if exist('CRint')&~isempty(CRint) %calculate standard deviations
    cdat=expcov(CRint,fixpind);
    k=numord+denomord+3; stdv=[];
    while length(cdat)>0
      stdv=[stdv;sqrt(cdat(1))]; cdat(1:k)=[]; k=k-1;
    end
    if any(imag(stdv)~=0), unrel=1; else unrel=0; end
    fpi=[fixpind;inf];
    if any(domain=='sp')
      ind=find(fpi<=numord+1); fpi(ind)=numord+2-fpi(ind);
      ind=find( (numord+1+denomord+1>=fpi)&(fpi>numord+1) );
      fpi(ind)=2*(numord+1)+denomord+2-fpi(ind);
      fpi=sort(fpi);
    end
  end
  fprintf(rfid,'Numerator:\n');
  for k=0:numord
    if pdati(2*(k+1))>=0, sisp=' '; else sisp=''; end
    if k<10, sisp=[sisp,' ']; end
    fprintf(rfid,[man,'%.0f  ',sisp],k);
    if isreal(pdati(2*(k+1)))
      fprintf(rfid,'%.15e',pdati(2*(k+1)));
    else
      fprintf(rfid,'%.15e + j*%.15e',real(pdati(2*(k+1))),real(pdati(2*(k+1))));
    end
    if exist('stdv')==1
      if any(domain=='spw'), stdi=numord-k+1;
      else stdi=1;
      end
      if (unrel==0)&(fpi(1)~=(k+1)) %reliable and not fixed
        fprintf(rfid,'    std: %.4e',stdv(stdi));
        if abs(pdati(2*(k+1)))>stdv(stdi)
          fprintf(rfid,'  (%.3g%%)',100*stdv(stdi)/abs(pdati(2*(k+1))));
        else
          fprintf(rfid,'  (std very large, >100%%)');
        end
      else
        if (fpi(1)==(k+1)) %fixed
          fprintf(rfid,'    fixed'); fpi(1)=[];
        else %unreliable covariances
          fprintf(rfid,'    variances are unreliable');
        end
      end %all(imag(
      stdv(stdi)=[];
    end %exist('stdv')
    fprintf(rfid,'\n');
  end %for k
  if any(domain=='pq')
    fprintf(rfid,...
        'Orthogonal polynomial representation: Z saved with parameters\n');
  end
  pdati(1:2*(numord+1))=[];
  fprintf(rfid,'Denominator:\n');
  for k=0:denomord
    if pdati(2*(k+1))>=0, sisp=' '; else sisp=''; end
    if k<10, sisp=[sisp,' ']; end
    fprintf(rfid,[man,'%.0f  ',sisp],k);
    if isreal(pdati(2*(k+1)))
      fprintf(rfid,'%.15e',pdati(2*(k+1)));
    else
      fprintf(rfid,'%.15e + j*%.15e',real(pdati(2*(k+1))),real(pdati(2*(k+1))));
    end
    if exist('stdv')==1
      if any(domain=='spw'), stdi=denomord-k+1;
      else stdi=1;
      end
      if (unrel==0)&(fpi(1)~=(numord+1+k+1)) %reliable and not fixed
        fprintf(rfid,'    std: %.4e',stdv(stdi));
        if abs(pdati(2*(k+1)))>stdv(stdi)
          fprintf(rfid,'  (%.3g%%)',100*stdv(stdi)/abs(pdati(2*(k+1))));
        else
          fprintf(rfid,'  (std very large, >100%%)');
        end
      else
        if (fpi(1)==(numord+1+k+1)) %fixed
          fprintf(rfid,'    fixed'); fpi(1)=[];
        else %unreliable covariances
          fprintf(rfid,'    variances are unreliable');
        end
      end %all(imag(
      stdv(stdi)=[];
    end %exist('stdv')
    fprintf(rfid,'\n');
  end %for k
  if any(domain=='pq')
    fprintf(rfid,...
        'Orthogonal polynomial representation: Z saved with parameters\n');
  end
  if any(domain=='spw')
    fprintf(rfid,'Delay:\n%.5g s',delay*delayscale/fs);
  else
    fprintf(rfid,['Delay:\n%.',num2str(fis),'g samples'],delay*delayscale);
  end
  if strcmp(delaytreat,'v')&(exist('stdv')==1)
    fprintf(rfid,'    std: %.5g\n',stdv(1));
  elseif itmax>0
    fprintf(rfid,'    fixed\n');
  else
    fprintf(rfid,'\n');
  end
  if messnoC>0
    fprintf(rfid,['WARNING: d^2C/dP^2 is badly conditioned in elis: ',...
        'standard deviations may be unreliable\n']);
        fprintf(rfid,'Condition number: %.3e\n',condinumd2Cdp2);
  end
  if any(any(imag(Cpint)))
    fprintf(rfid,['WARNING: Complex element(s) in approximate ',...
        'covariance matrix\n']);
  end
  if ~all(all(finite(Cpint)))
    fprintf(rfid,['WARNING: Infinite or NaN element(s) in approximate ',...
        'covariance matrix\n']);
  end
  if any(any(imag(CRint)))
    fprintf(rfid,['WARNING: Complex element(s) in Cramer-Rao ',...
        'matrix\n']);
  end
  if ~all(all(finite(CRint)))
    fprintf(rfid,['WARNING: Infinite or NaN element(s) in Cramer-Rao ',...
        'matrix\n']);
  end
  %
  if strcmp(calcrnum,'c')
    if any(domain=='sp'), fprintf(rfid,'\nZeros (rad*Hz):\n');
    elseif strcmp(domain,'w'), fprintf(rfid,'\nZeros (sqrt(rad*Hz)):\n');
    else fprintf(rfid,'\nZeros:\n');
    end
    for i=1:length(rnum)
      if imag(rnum(i))~=0
        fprintf(rfid,'%+.4e %+.4e*j\n',real(pzsc*rnum(i)),imag(pzsc*rnum(i)));
      else %real zero
        fprintf(rfid,'%+.4e\n',real(pzsc*rnum(i)));
      end
    end %for i
  end %calcrnum
  if strcmp(calcrdenom,'c')
    if any(domain=='sp'), fprintf(rfid,'\nPoles (rad*Hz):\n');
    elseif strcmp(domain,'w'), fprintf(rfid,'\nPoles (sqrt(rad*Hz)):\n');
    else fprintf(rfid,'\nPoles:\n');
    end
    for i=1:length(rdenom)
      if imag(rdenom(i))~=0
        fprintf(rfid,'%+.4e %+.4e*j',...
            real(pzsc*rdenom(i)),imag(pzsc*rdenom(i)));
      else
        fprintf(rfid,'%+.4e              ',real(pzsc*rdenom(i)));
      end
      if ~isempty(unstab)
        if unstab(1)==i
          %fprintf(rfid,'  unstable (stablimit=%.5g)',stablimit);
          fprintf(rfid,'  unstable');
          if any(domain=='zq'), fprintf(rfid,',  |z%.0f|=%.5e',i,...
                pzsc*abs(rdenom(unstab(1))));
          elseif strcmp(domain,'w')
            if imag(rdenom(i))~=0
              fprintf(rfid,'   w%.0f^2=%+6.4g %+6.4g*j',i,...
                real((pzsc*rdenom(unstab(1)))^2),...
                imag((pzsc*rdenom(unstab(1)))^2));
            else %real unstable pole
              fprintf(rfid,'   w%.0f^2=%+6.4g',i,(pzsc*rdenom(unstab(1)))^2);
            end
          end
          unstab(1)=[];
        end %unstab(1)==i
      end %~isempty(unstab)
      fprintf(rfid,'\n');
    end %for i
    fprintf(rfid,'stablimit=%.5g\n',stablimit);
  end %calcrdenom
  if strcmp(calcrnum,'c')&strcmp(calcrdenom,'c') %Gain info
    kb=min(find(numns~=0));
    if any(domain=='spw'), szin=numord+1-kb;
    else szin=kb-1;
    end
    bk=numns(kb);
    fprintf(rfid,['\nFirst nonzero numerator coefficient is ',...
        'b(%.0f)=%.4e\n'],szin,bk);
    ka=min(find(denomns~=0));
    if any(domain=='spw'), szid=denomord+1-ka;
    else szid=ka-1;
    end
    ak=denomns(ka);
    fprintf(rfid,['First nonzero denominator coefficient is ',...
        'a(%.0f)=%.4e\n'],szid,ak);
    fprintf(rfid,'b(%.0f)/a(%.0f)=%.4e\n',szin,szid,bk/ak);
  end
  %Static gain
  if any(domain=='spw')
    gain=numns(numord+1)/denomns(denomord+1);
  elseif any(domain=='zq'), gain=sum(numns)/sum(denomns);
  end
  if finite(gain)
    fprintf(rfid,'Static gain: %.4e',gain);
    if gain>0, fprintf(rfid,', %.3g dB\n',20*log10(abs(gain)));
    else disp(' ')
    end
  elseif ~isnan(gain), fprintf(rfid,'Static gain is infinite.\n');
  else fprintf(rfid,'Simple evaluation of static gain gives NaN.\n');
  end
  %
  if isempty(pfile), pfn='-'; else pfn=pfile; end
  %fprintf(rfid,['\nParameter file to save data: ',pfn,'\n']);
  if isempty(cfile), cfn='-'; else cfn=cfile; end
  %fprintf(rfid,['Covariance file to save data: ',cfn,'\n']);
  fprintf(rfid,'Additional time for report file generation: %.1f min\n',...
      etime(clock,tr0)/60);
  fprintf(rfid,['\n%%%%%%%%%% End of report file ',rfile,' %%%%%%%%%%\n']);
  %fprintf('elis report file is ready\n')
  fclose(rfid);
end %report file generation
if ~any(dispmesg=='nc')
  msgsall=[msgsall,sprintf('Total run time: %.2f min',trtime/60)];
  if plottime>0.0051*60
    msgsall=[msgsall,sprintf(', time for plots: %.2f min',plottime/60)];
  end
  if pztime>0.0051*60
    %msgsall=[msgsall,sprintf(', for poles/zeros: %.2f min',pztime/60)];
  end
  msgsall=[msgsall,sprintf('\n')];
  if ~isempty(rfile)
    msgsall=[msgsall,sprintf('Time for report file generation: %.1f min\n',...
        etime(clock,tr0)/60)];
  end
  if ~isempty(msgsall), fprintf(msgsall), msgsall=''; end
elseif dispmesg=='c', msgsall='';
end
expimpmessages=expimpmessagessave;
%
if exist('ith')&~isempty(ith)
  %set(ith,'string','Done')
  hp=get(ith,'parent'); delete(ith)
elseif plotno>0
    hp=axes('position',[0.5,0,.5,1],'visible','off','parent',helis);
end
if plotno>0
  dbst=dbstack; putclose=1;
  if length(dbstack)>1
    caller=dbst(2).name;
    if findstr([lower(caller),'|'],'sme.m|'), putclose=0; end
    if putclose==1
      set(hp,'units','pixels');
      pos=get(hp,'Position');
      p(4)=20; p(3)=50; p(2)=pos(2); p(1)=pos(1)+pos(3)-p(3);
      hui=uicontrol('position',p,'string','Close','tag','fdident_dismiss',...
        'callback','delete(get(findobj(gcf,''tag'',''fdident_dismiss''),''parent'')'')');
      set(hui,'parent',helis)
    end
  end
end
if calliterctrl, iterctrl('Continue'); end
if isa(pdat,'fidmodel')&isempty(pdat.nu)
  disp(setstr('Yctpkpi<"kp"vjg"vtkcn"xgtukqp"pq"oqfgnu"ecp"dg"ucxgf'-2))
end
%
%%%%%%%%%%%%%%%%%%%%%%%% end of function elis %%%%%%%%%%%%%%%%%%%%%%%%

function [rdenomn,denomn,delayn,numn,unstabstays]=...
  stabmph(domain,denom,num,delay,fs,Zdenom,Znum,Esmrdenom,rdenom,dgain,pzsc,...
  unstab,stabmode,stablimit,freq,delaytreat,delayscale,numfix,numfixind,...
  denomfix,denomfixind,paramtreat,mode,coeffs,tauR)
%STABMPH Stabilize or force minimum phase in model
%mode: 'stab' or 'nmph'
if ~exist('mode'), mode='stab'; end
if ~strcmp(mode,'stab')&~strcmp(mode,'nmph')
  error('mode invalid')
end
unstabrdenom=rdenom;
unstabp=rdenom(unstab);
unstabstays=[];
if any(domain=='zq')
  if strcmp(stabmode,'r')|strcmp(stabmode,'l')
    rdenom(unstab)=min([1.0./abs(rdenom(unstab));...
        stablimit*ones(1,length(unstab))])...
      .*exp(j*angle(rdenom(unstab)));
  elseif strcmp(stabmode,'c')
    rdenom(unstab)=stablimit*exp(j*angle(rdenom(unstab)));
  end
elseif any(domain=='sp')
  if strcmp(stabmode,'r')|strcmp(stabmode,'l')
    rdenom(unstab)=min([-real(rdenom(unstab));...
        stablimit/pzsc*ones(1,length(unstab))])...
      + j*imag(rdenom(unstab));
  elseif strcmp(stabmode,'c')
    rdenom(unstab)=stablimit/pzsc + j*imag(rdenom(unstab));
  end
elseif strcmp(domain,'w')
  unstabr=find((real(rdenom(unstab))==0)|(imag(rdenom(unstab))==0));
  unstabstays=unstab(unstabr);
  unstab(unstabr)=[];
  if ~isempty(unstab)
    if strcmp(stabmode,'r')|strcmp(stabmode,'l')
      rdenom(unstab)=sqrt(min([-real(rdenom(unstab).^2);...
          stablimit/pzsc^2*ones(1,length(unstab))])...
        + j*imag(rdenom(unstab).^2));
    elseif strcmp(stabmode,'c')
      rdenom(unstab)=sqrt(stablimit/pzsc^2 + j*imag(rdenom(unstab).^2));
    end
  end
  if ~isempty(unstabr)
    if strcmp(mode,'stab')
      disp('WARNING: Cannot stabilize real or imaginary poles in the w-domain')
    else
      disp('WARNING: Cannot minimize phase of real or imaginary poles in the w-domain')
    end
  end
end
stabp=rdenom(unstab);
tfmod=tfcalc(exppar('s',real(poly(unstabp)),real(poly(stabp)),[],[],[],[],[],[],[],[],[],coeffs,tauR),freq);
if angle(tfmod(1))>pi/2, tfmod=-tfmod; signch=-1; else signch=1; end
if strcmp(delaytreat,'v') %delay free, adjustment for stabilization
  phmod=unwrap(angle(tfmod));
  delaymod=(2*pi*freq)\phmod;
  delay=delay+delaymod/delayscale;
end %if strcmp(delaytreat,'v')
if any(domain=='pq') %orthopol
  magmed1=median(abs(tfcalc(exppar(domain,num,denom,0,fs,Znum,Zdenom,[],[],[],[],[],coeffs),freq)));
  i=min(find(denom~=0));
  wop=1./Esmrdenom(1:length(freq));
  denom(i:length(denom))=signch*orthr2p(domain,rdenom,dgain,Zdenom,freq,wop);
  magmed2=median(abs(tfcalc(exppar(domain,num,denom,0,fs,Znum,Zdenom,[],[],[],[],[],coeffs),freq)));
  denom=magmed1/magmed2*denom;
else
  magmed1=median(abs(tfcalc(exppar(domain,num,denom,[],fs,[],[],[],[],[],[],[],coeffs),freq)));
  signgain=real(poly(unstabrdenom))'\denom';
  i=min(find(denom~=0));
  denom(i:length(denom))=signgain*signch*real(poly(rdenom));
  magmed2=median(abs(tfcalc(exppar(domain,num,denom,[],fs,[],[],[],[],[],[],[],coeffs),freq)));
  denom=magmed1/magmed2*denom;
end
if strcmp(paramtreat,'a'), num=denom(length(denom):-1:1); end
if ~isempty(unstabstays), unstab=unstabstays; end
if sum([numfix(:);denomfix(:)]~=0)>1
  error('Too many nonzero parameters fixed for stabilization')
end
if (length(numfixind)==0)&(length(denomfixind)==1)
  if denom(denomfixind+1)==0
    error('Stabilization/minimum phase setting yields zero value for the fixed coefficient')
  end
  num=num/denom(denomfixind+1)*denomfix;
  denom=denom/denom(denomfixind+1)*denomfix;
end
denomn=denom; rdenomn=rdenom; delayn=delay; numn=num;

function initsetstr=issset(initset,domain)
%Long text of initset
if initset=='l', initsetstr='LS';
elseif initset=='w', initsetstr='outputvar-weighted least squares';
elseif initset=='v', initsetstr='yr-weighted svd';
elseif initset=='u', initsetstr='ur-weighted svd';
elseif initset=='s', initsetstr='singular value decomposition';
elseif initset=='y', initsetstr='outputvar-weighted svd';
elseif initset=='a', initsetstr='approximate maximum likelihood';
elseif initset=='r', initsetstr='reduced equation error weighted svd';
elseif initset=='i', initsetstr='IQML';
elseif initset=='f', initsetstr='object';
elseif initset=='x', initsetstr='external weight';
elseif initset=='e', initsetstr=['equation error method (invfreq',domain,')'];
elseif initset=='t', initsetstr='scan different algorithms';
elseif initset=='g', initsetstr='GTLS';
elseif initset=='t', initsetstr='BTLS';
else error(['initset ''',initset,''' is not allowed'])
end

function [vdatout,Mout]=smoothvar(vdat,M);
%If M<7, smooth vdat

if (M>=7)|(size(vdat,1)<=1)
  vdatout=vdat; Mout=M;
  return
elseif M>=2
  if M>=3, modM=min(3,size(vdat,1)); fi=ones(modM,1)/modM;
  elseif M==2, modM=min(7,size(vdat,1)); fi=ones(modM,1)/modM;
  end
else %M<2
  error(sprintf('Cannot treat M=%.1g',M))
end
Mout=(M-1)*modM+1;
vdatout=zeros(size(vdat));
for ii=1:size(vdat,2)
  v=conv(vdat(:,ii),fi);
  lmid=(length(fi)+1)/2;
  if rem(lmid,1)==0, v=v(lmid:end-lmid+1);
    for il=1:min(lmid-1,floor((modM-1)/2))
      v(il)=v(min(lmid,floor((modM-1)/2)));
      v(end-il+1)=v(min(lmid,floor((modM-1)/2)));
    end %for il
  else
    error('length(fi) is even')
  end
  vdatout(:,ii)=v;
end

%End of file elis
